/* global.hpp */
/* MV 1.2.3 */
/* +             +            */		   
/* Deinitions of basic data types, frequently required macros,
        compiler toggles, and so forth.
*/
#define DEBUG 2
#define REPORT2(N,L,A) {}
#define ERRORCN(A) {}
#define ASSERT(EXP) { assert(EXP);}
#define ASSERT1(EXP) { assert(EXP);}
#define ASSERT2(EXP) { assert(EXP);}
#define REPORT2(N,L,A) {}
#define REP2(A) {}
#define STRACE1
#define STRACE2
#define STRACE3
#define STRACE4
#define STRACEM2(TXT,NUM) 
#define CHECKPOINT
#define BACKTRACE(TXT,NUM)
#define DISCARD(A) ((void)&(A))

/* Compiler toggles and language macros */ 
/* ------------------------------------ */

#define WATCOM 0
#define MSVC 1
#define USE32BIT 1


/* Standard language values */
/* ------------------------ */
#define NULLSTRING 

#ifndef NULL
#define NULL 0
#endif

/* Cutesy, cutesy */ 
#define false (0!=0)
#define true  (0==0) 
#define null 0 
/* null value for id values etc */

/* boolean success/failure values returned by error-prone functions */
#define SUCCESS 0
#define FAILURE (!SUCCESS)


#define OFFSET_OF_MEMBER(TYPE,MEMBER) (int)(&(((TYPE *)0)->MEMBER))


/* Macros for open-ended structures */
#if MSVC
#define OPENENDED 1
#define SIZEOF_OPENENDED(SS,MEMBER) (sizeof(SS)-sizeof((SS).MEMBER[0]))
#define SIZEOF_OPEN_ENDED(TYPE,MEMBER,N) ((int)(&(((TYPE *)0)->MEMBER[0])) + (N)*sizeof(((TYPE *)0)->MEMBER[0]))
#else
#define OPENENDED 
#define SIZEOF_OPENENDED(s,MEMBER) sizeof(s)
#define SIZEOF_OPEN_ENDED(TYPE,MEMBER,N) ((int)(&(((TYPE *)0)->MEMBER[0])) + (N)*sizeof(((TYPE *)0)->MEMBER[0]))
#endif


#define FREE(A) free(A)

#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#define MIN(a,b) (((a) < (b)) ? (a) : (b)) /* IB */

#define ABS(A) (((A)<0) ? (-(A)):(A))


/* swapping macros */
/* --------------- */

template <class T> inline  
void SWAPPTRS(T &s1, T &s2) { T t = s1; s1=s2; s2=t;}

template <class T> inline  
void SWAP(T &s1, T &s2) { T t = s1; s1=s2; s2=t;}



#if WATCOM
extern uint16 BSF_16(int16);
extern uint16 BSR_16(int16);
extern uint32 BSF_32(int32);
extern uint32 BSR_32(int32);

#pragma aux BSF_32  = "bsf eax,ebx" parm [ebx] value [eax]
#pragma aux BSR_32  = "bsr eax,ebx" parm [ebx] value [eax]


#pragma aux BSF_16  = "bsf ax,bx" parm [bx] value [ax]
#pragma aux BSR_16  = "bsr ax,bx" parm [bx] value [ax]
#endif

#if MSVC

inline int BSF_32(int x) { _asm bsf eax,x }


inline int BSR_32(int x) { _asm bsr eax,x }

#endif


/**+ Hard integer types */

typedef bool boolean;



#define BITS_PER_BYTE (8)

typedef signed char int8;
#define int8_MAX (0x7F)
#define int8_MIN (0x81)

typedef char uint8;
#define uint8_MAX (0xFF)
#define uint8_MIN (0)

typedef signed short int16;
#define int16_MAX (0x7FFF)
#define int16_MIN ((int16)0x8001)

typedef unsigned short uint16;
#define uint16_MAX (0xFFFF)
#define uint16_MIN (0)

typedef signed long int32;
#define int32_MAX (0x7FFFFFFF)
#define int32_MIN ((int32)0x80000001)

typedef unsigned long uint32;
#define uint32_MAX (0xFFFFFFFF)
#define uint32_MIN (0)


#define int8_NBIT (0x80)
#define int16_NBIT (0x8000)
#define int32_NBIT ((int32)0x80000000)
#define uint32_NBIT ((uint32)0x80000000)


#define SUPPORTS_64 1

#if SUPPORTS_64
typedef __int64 int64;
//typedef __uint64 uint64;
typedef
union { int64 all;
 		struct { uint32 low; int32 high;} parts;
 	  }	myint64;
typedef
union { int64 s_all;	struct { uint32 low; uint32 high;} parts;  }	myuint64;
typedef myuint64 uint64;
#define uint64_MAX (0xFFFFFFFFFFFFFFFFL)
#define uint64_MIN (0L)

#else
//typedef struct {uint32 low;uint32 high;} uint64;
typedef
			{	struct { uint32 low; uint32 high;} parts;
 			}	myuint64;
typedef myuint64 uint64;
typedef
			{	struct { uint32 low; int32 high;} parts;
 			}	myint64;
typedef myint64 int64;
#endif

/**+ Soft integer types =- set to fastest possibilities */
typedef int32 fint8;   	/* fastest way to process 8bit data */  
typedef uint32 fuint8; 

typedef int32 fint16;	/* fastest way to process 16bit data */
typedef uint32 fuint16;

typedef fint16 fint;    /* fint=fint16 fastest integer >=16b */ 
typedef fuint16 pfint;  /* fastest +ve int >=16b */
typedef fuint16 fuint;  /* same as pfint */
#define SIZEOF_FINT 4   /* sizeof(fint16) for compiler */

typedef fint8 fbyte; 	/* used to hold small signed values (<128)*/
typedef fuint8 fubyte;  /* used to hold +ve small signed values (<256) */


/* ========================
	  Other types
   ========================	*/
typedef double real;		/* used pass reals to multivector primatives (not necessarily same as Real in leaves) */
typedef uint32 flagstype; 	/* used to hold int32s and pointers */

typedef double myfloat;

typedef int32 generic; 	/* used to hold int32s and pointers */
#define GENERIC_0 0

typedef struct {real x;real y;real z;} vector3D;
typedef struct {real x;real y;real z;real s;} vector4D;
typedef vector3D posn3D;

//#define VEC3D_R_MAGSQ(V) ((V).x*(V).x +(V).y*(V).y +(V).z*(V).z)


#define FLOAT_0 (0.0)
#define FLOAT_1 (1.0)
#define FLOAT_2 (2.0)
#define FLOAT_HALF (0.5)
#define FLOAT_TINY (1.0/0x100000)

#define REAL_0 FLOAT_0
#define FRAC_1 FLOAT_1
#define REAL_1 FLOAT_1
#define RSQUARE(A) ((A)*(A))

#define FRAC_RANDOM ((rand()*FRAC_1)/RAND_MAX)

#define rDIVrr(A,B) ((A)/(B))

#define RRMUL(A,B) ((A)*(B))


#define REAL_TO_MYFLOAT(A) (A)
#define MYFLOAT_TO_REAL(A) (A)



/* User heap diagnostics
  ====================== */
inline void reattribute(void * p,char * f,int l) { } // reassign message and number diagnostics to a pointer  
inline bool CheckValidBlock(void * p) { return((p!=NULL));} // verify p is a valid pointer

/* end */