 /*
	 MV 1.3.0

	 Hierachical multivector type .
	 
	 Copyright (c) Ian C.G.Bell 2004.
	 
	 This code provides a C open-ended structure implimenting a hierachical 
	 compact storage of high-dimensional multivectors 
	 
	 For a description of methadologies in this code see www.ibell.co.uk/maths/geoalg.htm.

		Version Notes
		-------------

	 1.3.0 	Added k-generalised homogenised coordinates 
	 			 	Can pass mvproduct code to inv() so MV_EUCLIDEAN_PRODUCT etc now defined in mv scope
	 			 	Trapped NULL indirection when taking grade of MV_ZERO
	 			 	Fixed bug in inverse
	 			 	Fixed bpf_MV for levels>1
	 			 	Scalar product support for buffered duals.
	 			 	Added PseudoScalarPart() primitves. 


	 1.2.6	Added rapid extraction of 1-vectors
	 				Renamed global GHC extendors ep and em to mv_ep and mv_em 
					Changed mvgrade type to signed
	 1.2.5	Added .inv() alternative mv inverse invokation 
					Added mv_right_contraction()
	 1.2.4	Miscellaneous maintenance
	 1.2.3	Fixed incorrect clearing of versors
	 				Replaced .norm() by .nrm()
					Improved Inverse attempting clifford, reverse amd Hermitian conjugtioms
					Replaced LOCAL_NEWS with MV_NEWS
	 1.2.2	Altered MultivectorJoin to force Euclidean signatures, no longer returns disjoint.
	       	Altered GHC embeddings to support nonEuclidean signatures 
	 1.2.1	fixed serious addition bug introduced in 1.2
	 1.2 		replaces real;multivector union type with seperate base level multivector structure 
	 1.1 		impliments factored representations, but these remain untested..
*/


#include "stdafx.h"
#include <stdlib.h>
#include <assert.h>
#include "global.hpp"


#include <stdarg.h>
#include <math.h>
#include <float.h>


#include "multivec.hpp"


// This macro, intended to iterate INDR through the nonzero bit positions of BFR, is used in many inner loops
#define SHIFT_RGT_CNT(SHF,BFR,INDR) { uint32 SHF=BSF_32(BFR);	(BFR) >>= SHF; (BFR)>>=1;	(INDR) += (bladeindex)SHF; }


static char sourcename[]= __FILE__	;


// internal globals 

bladesflag mv_negsigs[MV_NUM_LEVEL]; 
bladesflag mv_nullsigs[MV_NUM_LEVEL];
int mv_numpos[MV_NUM_LEVEL];

mvlevel MV_toplev;	//  number of levels used
mvgrade	MV_topdim;	//  number of dimensions in top level
bladesflag MV_top_ps;

#define MV_lev_ps (1<<((1<<MV_DIM0)-1))



mvconjugation mv_conjugation_modulatory;

// externally visible globals 
int g_ghc;	 // number of generlaised homeenised extensions
boolean g_i_neg; // 1 if pseudoscalar i has i^2=-1
mvgrade g_N;				 // globally visible current dimension
mvgrade g_N0;				 // globally visible current dimension exluding extensions
fullbladeindex g_negs;  // bitflags for negative signatures
fullbladeindex g_nulls;	// bitflags for null signatures




 uint8 BitsSetCountLUT[256]={
0,1,1,2,1,2,2,3, 1,2,2,3,2,3,3,4, 1,2,2,3,2,3,3,4, 2,3,3,4,3,4,4,5, 
1,2,2,3,2,3,3,4, 2,3,3,4,3,4,4,5, 2,3,3,4,3,4,4,5, 3,4,4,5,4,5,5,6, 
1,2,2,3,2,3,3,4, 2,3,3,4,3,4,4,5, 2,3,3,4,3,4,4,5, 3,4,4,5,4,5,5,6, 
2,3,3,4,3,4,4,5, 3,4,4,5,4,5,5,6, 3,4,4,5,4,5,5,6, 4,5,5,6,5,6,6,7, 
1,2,2,3,2,3,3,4, 2,3,3,4,3,4,4,5, 2,3,3,4,3,4,4,5, 3,4,4,5,4,5,5,6, 
2,3,3,4,3,4,4,5, 3,4,4,5,4,5,5,6, 3,4,4,5,4,5,5,6, 4,5,5,6,5,6,6,7, 
2,3,3,4,3,4,4,5, 3,4,4,5,4,5,5,6, 3,4,4,5,4,5,5,6, 4,5,5,6,5,6,6,7, 
3,4,4,5,4,5,5,6, 4,5,5,6,5,6,6,7, 4,5,5,6,5,6,6,7, 5,6,6,7,6,7,7,8};

bladesflag mv_grade_inhibit[MV_DIM0+1] =
#if MV_DIM0==1
 { 0xFE, 0xFD };
#endif
#if MV_DIM0==2
 { 0xFE, 0xF9, 0xF7 };
#endif
#if MV_DIM0==3
{ 0xFE, 0xE9, 0x97, 0x7F };
#endif
#if MV_DIM0==4
{ 0xFFFFFFFE, 0xFFFFFEE9, 0xFFFFE997, 0xFFFF977F, 0xFFFF7FFF }; // could use 16bit
#endif
#if MV_DIM0==5
{ 0xFFFFFFFE, 0xFFFEFEE9, 0xFEE9E997, 0xE997977F, 0x977F7FFF, 0x7FFFFFFF };
#endif



fullbladeindex 	mv_bf_em[MV_MAX_GHC];
fullbladeindex 	mv_bf_ep[MV_MAX_GHC];

bladesflag mv_conj[MV_MAX_CONJUGATION][MV_NUM_LEVEL];

multivector0 MV_scalar; // used to temporarilly hold scalars

multivector MV_basis[MV_DIM1];

#if MV_MAX_GHC
multivector MV_e0[MV_MAX_GHC],MV_einf[MV_MAX_GHC],MV_e0_inv[MV_MAX_GHC],MV_einfi[MV_MAX_GHC];
multivector MV_em[MV_MAX_GHC],MV_ep[MV_MAX_GHC]; // GHC vectors
#endif

multivector MV_pool; // Use as handle to freed multivectors pool (not implimented)


#define MV_NUM_REGISTERS (2)

#if 1

multivector mv_reg[MV_NUM_LEVEL][MV_NUM_REGISTERS];	 // internal workspace non-compacted multivectors 

#else // if level hierachy isn't strictly descending need to stack workspace on expanding factored forms

int MV_stk_index=0;
#define MV_STK_DEPTH (2)
multivector MV_stk[MV_STK_DEPTH][MV_NUM_LEVEL][MV_NUM_REGISTERS];
#define mv_reg MV_stk[MV_stk_index]

#endif


mvgrade BitsSetFbi(fullbladeindex bi)
{	
#if MV_DIM1<=8
	return(BitsSetCountLUT[bi]);
#else
	 mvgrade i=0;
 	while(bi)
 	{	i+=BitsSetCountLUT[(uint8)bi];
 		bi>>=8;
 	}
 	return(i);
#endif
}

bladeindex BitsSetBi(bladeindex bi) { return(BitsSetCountLUT[bi]);}


#if MV_DIM0<=3
bladeindex BitsSetBf(bladesflag bf) { return(BitsSetCountLUT[bf]);}
#else
#if MV_DIM0==4
bladeindex BitsSetBf(bladesflag bf) { return(BitsSetCountLUT[(bf)&0xFF]+BitsSetCountLUT[bf>>8]);}
#else


bladeindex BitsSetBf(bladesflag bf)	// returns number of nonzero bits in bf (assumed an unsigned integer type)
{	bladeindex c=0;	while(bf)	{	c += BitsSetCountLUT[bf & 0xFF];bf >>= 8;	}	return(c);}
#endif

#endif

#if MV_DIM0<=8
boolean BitParity(bladeindex bi) {	return(BitsSetCountLUT[bi]&1);}	// returns 1 iff odd number nonzero bits
#endif


#if MV_NEWS
#define CopyMultivector(T,S) CopyMultivectorCore(__FILE__, __LINE__, T,S)
#define CopyMultivector00(F,L,T,S) CopyMultivectorCore(F,L,T,S)
multivector CopyMultivectorCore(char * fi,int line, multivector tgt,multivector src);
multivector0 CopyMultivectorCore0(char * fi,int line, multivector0 tgt,multivector0 src);
#else
#define CopyMultivector(T,S) CopyMultivectorCore(T,S)
#define CopyMultivector00(F,L,T,S) CopyMultivectorCore(T,S)
multivector CopyMultivectorCore(multivector tgt,multivector src);
multivector0 CopyMultivectorCore0(multivector0 tgt,multivector0 src);
#endif


multivector CopyMultivectorScaled(multivector tgt,multivector src,Real smult);


boolean ValidReal0(Real * pr,char * fi,int li) // detects bogus floats without triggering FP exception.
{	bool b=1;
#if WATCOM
	unsigned int old_fp = _control87(0,0);	_control87(_MCW_EM,_MCW_EM); // Disable FP exceptions prior to loading into FPU 
 	myfloat fl=*pr; int bb=_finite(fl);																	 // Stooopid library routine may generate exceptions.
	_fpreset();
	if(!bb) b=0;
//	if(b)	{	if(fl < -100000.00) b=0;	else	if(fl > 100000.00) b=0;	}	// check inside anticpated range
	_fpreset();
	_control87(old_fp & _MCW_EM,_MCW_EM);
	if(b==0) REP2(("\n ValidReal0 fails from %s %i",fi,li));
	_fpreset();
#else
	// return(!isnan(*pr));
#endif
	return(b);
}

#if MV_DEBUG

int DiagnoseMV(multivector m)
{	// Dignostic : Check integrity of multivector, return 0 or 1 for MV_ZERO or failure code
	if(m==MV_ZERO) return(1);

#if !MV_RELEASE
#if MV_DEBUG>1
	if(!CheckValidBlock(m)) return(3);
#endif
#endif

	if(m->h.noncompacted) return(2);

	if(m->h.freed) return(4);
	if(m->h.level < 0) return(8);
	if(m->h.level >= MV_NUM_LEVEL) return(9);
//	if(m->h.level >= MV_EXP_LEVEL) return(7);
	int used = BitsSetBf(m->h.present);
	if(used > m->h.capacity+1) return(16);

#if MV_DEBUG>1
	if(m->h.level>0)
	{	for(int i=0;i<used;i++)
 		{ multivector m2=m->m[i];
			if(m2!=MV_ZERO) 
			{	if(m2->h.level>=m->h.level) return(10);
				int di=DiagnoseMV(m2); if(di) return(di<<8);
			}
		}
	}
	else
	{	for(int i=0;i<used;i++)	{	if(!ValidReal(((multivector0)m)->r[i])) return(0x20);	}
	}
#endif

	return(0);
}

boolean CheckValidMV(multivector m) { return(DiagnoseMV(m)==0);}

#else	// MV_DEBUG
#define CheckValidMV(M) (1)
#endif

// ------------------------------------------
//   Multivector creation and storage admin
// ------------------------------------------

multivector Reference(multivector a)
{	if(a!=MV_ZERO)
	{	if(a->h.refcount >= MV_MAX_REFCNT-1) return( CopyMultivector(NULL,a));
 		a->h.refcount++;
	}
	return(a);
}


multivector EnsureWritable(multivector mv)
{	if(mv->h.refcount <= 1) return(mv);
	mv->h.refcount--;
	return(CopyMultivector(NULL,mv));
}

// ----------------------------------------------------------------------
//   Routines to change capacity and/or level of an existing multivector
// ----------------------------------------------------------------------

multivector0 ExtendMultivectorCapacity0(mvcapacity capacity,multivector0 src) // increase capacity retaining zero level
{	
ASSERT(src!=NULL);
ASSERT(src->h.level==0);
ASSERT(capacity>0);
ASSERT(src->h.refcount<=1);
	multivector0 mv = NewMultivector0(capacity,NULL);
	mv->h.present = src->h.present;
	for(int i=0;i<=src->h.capacity;i++) { mv->r[i] = src->r[i]; }
	KillMultivector0(&src);
	return(mv);
}


multivector ExtendMultivectorCapacity(mvcapacity capacity,multivector src) // increase capacity, retaining level
{	
ASSERT(capacity>0);
ASSERT(src!=NULL);
ASSERT(src->h.refcount<=1);

REPORT2(MV_NEWS,4,("\n Extending capacity of %p lev=%X to %X ",src,src->h.level,capacity))

	if(src->h.level==0) return((multivector)ExtendMultivectorCapacity0(capacity,(multivector0)src));

ASSERT2(src->h.level<MV_NUM_LEVEL);
	{ multivector mv = NewMultivector(capacity,NULL,src->h.level);
		mv->h.present = src->h.present;
		for(int i=0;i<=src->h.capacity;i++) { mv->m[i] = src->m[i]; src->m[i]=MV_ZERO; }
		KillMultivector(&src);
		return(mv);
	}
}


#if MV_NEWS
#define EnsureCapacity0(T,C) EnsureCapacityCore0(__FILE__,__LINE__,T,C)
#define EnsureCapacity20(T,C,M) EnsureCapacityCore0(__FILE__,__LINE__,T,C,M)
multivector0 EnsureCapacityCore0( char * f, int line, multivector0 tgt,mvcapacity capacity,mvcapacity margin=0)
#else
#define EnsureCapacity20(T,C,M) EnsureCapacity(T,C,M)
multivector0 EnsureCapacity0(multivector0 tgt,mvcapacity capacity,mvcapacity margin=0)
#endif
{	int cpm = capacity+margin;
	if(cpm>(1<<MV_DIM0)) cpm=(1<<MV_DIM0);
	if(cpm<=0) cpm=1;
	if(tgt==MV_ZERO)
	{ tgt=NewMultivector0(cpm,NULL);
#if MV_NEWS
	 reattribute(tgt,f,line);	
#endif
		return(tgt);
	}
	if(tgt->h.level>0)
	{
		KillMultivector((multivector *)&tgt);
		tgt=NewMultivector0(cpm,NULL);
#if MV_NEWS
	 reattribute(tgt,f,line);	
#endif
		return(tgt);
	}
ASSERT(tgt->h.level==0);
	if(tgt->h.capacity+1 < capacity) tgt = ExtendMultivectorCapacity0((mvcapacity)cpm, tgt);
ASSERT(tgt!=NULL);
	return(tgt);
}



#if MV_NEWS
#define EnsureCapacity(T,L,C) EnsureCapacityCore(__FILE__,__LINE__,T,L,C)
#define EnsureCapacity2(T,L,C,M) EnsureCapacityCore(__FILE__,__LINE__,T,L,C,M)
multivector EnsureCapacityCore( char * f, int line, multivector tgt,mvlevel level,mvcapacity capacity,mvcapacity margin=0)
{	if(level==0) return((multivector)EnsureCapacityCore0(f,line,(multivector0)tgt,capacity,margin));
#else
#define EnsureCapacity2(T,L,C,M) EnsureCapacity(T,L,C,M)
multivector EnsureCapacity(multivector tgt,mvlevel level,mvcapacity capacity,mvcapacity margin=0)
{	if(level==0) return((multivector)EnsureCapacity0((multivector0)tgt,capacity,margin));
#endif

	int cpm = capacity+margin;
	if(cpm>(1<<MV_DIM0)) cpm=(1<<MV_DIM0);
	if(cpm<=0) cpm=1;
	if(tgt==MV_ZERO)
	{ tgt=NewMultivector(cpm,NULL,level);
#if MV_NEWS
	 reattribute(tgt,f,line);	
#endif
		return(tgt);
	}
	if(tgt->h.level==0)
	{ KillMultivector0((multivector0 *)&tgt);
		tgt=NewMultivector(cpm,NULL,level);
		return(tgt);	
	}
	if(tgt->h.capacity+1 < capacity) tgt = ExtendMultivectorCapacity((mvcapacity)cpm, tgt);

	if(level!=tgt->h.level)
	{	tgt=EnsureWritable(tgt);
		for(int i=0;i<=tgt->h.capacity;i++) KillMultivector(&(tgt->m[i]));
		tgt->h.level=level;
	}
ASSERT(tgt!=NULL);
	return(tgt);
}

multivector AllocateSpace(multivector tgt,mvlevel level,bladesflag bf) {	return(EnsureCapacity(tgt,level,BitsSetBf(bf)));}

// -------------------------------------------------------



void ZeroMultivector(multivector a, mvlevel level)
{	// zero multivector and force its level, but leave unused multivector coefficients instanted
REPORT2(MV_NEWS,7,("\n ZeroMultivector(%M lev=%X)",a,level))
	mvcapacity i;
	if(a!=MV_ZERO)
	{	if(level==0) {	if(a->h.level>0)	for(i=0;i<=a->h.capacity;i++) KillMultivector(&(a->m[i]));	}
		else
		{	if(a->h.level>0)	for(i=0;i<=a->h.capacity;i++) ZeroMultivector(a->m[i], (mvlevel)(level-1));
			else		 					for(i=0;i<=a->h.capacity;i++) ((multivector0)a)->r[i] = REAL_0;
 		}
 		a->h.level = level;a->h.present = MV_FLAG_NONE;
		a->h.noncompacted=0;
 	}
}


multivector0 Condense0(multivector0 r,multivector0 a)	// delete r and return MV_ZERO if zero
{
ASSERT(a!=NULL);
ASSERT2(a->h.level==0);
	bladesflag bfr=a->h.present ;
	mvcapacity cap = BitsSetBf(bfr);
	if(cap>0)
	{
#if MV_NEWS>=4
if(r!=MV_ZERO) REPORT2(MV_NEWS,5,("\n Condense0ing %M to cap %X level %X ",r,cap,a->h.level))	
else REPORT2(MV_NEWS,6,("\n Condense0ing NULL into a cap %X level %X MV",r,cap,a->h.level))	
#endif
		r = EnsureCapacity0(r,cap);
		r->h.noncompacted=0;
		r->h.present = bfr;
 		bladeindex indr=0;	bladeindex cntr=0;
		while(bfr)
	 	{	SHIFT_RGT_CNT(shf,bfr,indr)
 			{	// swap coeff entries between r and a to ensure not lost
				if(!EQUIVALENT_REALS(((multivector0)a)->r[indr], Real_0)) 	((multivector0)r)->r[cntr++] = ((multivector0)a)->r[indr];
				else r->h.present ^= (1<<indr);
			}
			indr++;
		} // endwhile
 REPORT2(MV_NEWS,5,("\n Condensed result=%M ",r))	
 	}
	else KillMultivector0(&r);
	return(r);
}

multivector Condense(multivector r,multivector a)	// delete r and return MV_ZERO if zero
{
	bladesflag bfr=a->h.present ;
	mvcapacity cap = BitsSetBf(bfr);
	if(cap>0)
	{
REPORT2(MV_NEWS,5,("\n Condensing %M to cap %X level %X ",r,cap,a->h.level))	
		r = EnsureCapacity(r,a->h.level,cap);
		r->h.present = bfr;
 		bladeindex indr=0;	bladeindex cntr=0;
	 	if(a->h.level>0)
		{	while(bfr)
	 		{	SHIFT_RGT_CNT(shf,bfr,indr)
	 			{	
ASSERT2(a->m[indr]!=MV_ZERO);
					if(a->m[indr]->h.present!=0)
					{	// swap coeff entries between r and a to ensure not lost
						SWAPPTRS(r->m[cntr] , a->m[indr]);
						cntr++;
					}
					else r->h.present ^= (1<<indr);
				}
				indr++;
			} // endwhile
 		}
 		else
		{	while(bfr)
	 		{	SHIFT_RGT_CNT(shf,bfr,indr)
	 			{	// swap coeff entries between r and a to ensure not lost
					if(!EQUIVALENT_REALS(((multivector0)a)->r[indr], Real_0)) 	((multivector0)r)->r[cntr++] = ((multivector0)a)->r[indr];
					else r->h.present ^= (1<<indr);
				}
				indr++;
			} // endwhile
 		}
		r->h.noncompacted=0;
REPORT2(MV_NEWS,5,("\n Condensed result=%M ",r))	
 	}
	else KillMultivector(&r);
	return(r);
}


multivector0 CopyMultivectorCore0(
#if MV_NEWS
char * fi, int line,
#endif
 multivector0 tgt,multivector0 src)
{	// Deep copy allocating new memory and creating an independant instance 
	if(tgt==src) return(tgt);

ASSERT(src->h.level==0);
	mvcapacity used=BitsSetBf(src->h.present);

	tgt = EnsureCapacity0(tgt,used);

ASSERT(tgt->h.level==0);

#if MV_NEWS
reattribute(tgt,fi,line);
#endif
	tgt->h.present = src->h.present;
	mvcapacity i=0;
	for(;i<used;i++) tgt->r[i] = src->r[i]; // zero unused coefficients 
	return(tgt);
}

multivector CopyMultivectorCore(
#if MV_NEWS
char * fi, int line,
#endif
 multivector tgt,multivector src)
{	// Deep copy allocating new memory and creating an independant instance 
	if(tgt==src) return(tgt);
	mvcapacity used;
	if(src->h.level>=MV_VSR_LEVEL) used=(mvcapacity)src->h.present; else used=BitsSetBf(src->h.present);

	tgt = EnsureCapacity(tgt,src->h.level,used);
#if MV_NEWS
reattribute(tgt,fi,line);
#endif
	tgt->h.present = src->h.present;
	mvcapacity i=0;
	if(src->h.level)	for(;i<used;i++) tgt->m[i] = CopyMultivector00(fi,line,tgt->m[i], src->m[i]);
	else							for(;i<used;i++) ((multivector0)tgt)->r[i] = ((multivector0)src)->r[i];
 // zero unused coefficients 
	if(tgt->h.level>0)
	{	if(tgt->h.level>=MV_VSR_LEVEL) for(;i<=tgt->h.capacity;i++)	 KillMultivector(&tgt->m[i]);
		else for(;i<=tgt->h.capacity;i++) ZeroMultivector(tgt->m[i],(mvlevel)(tgt->h.level-1));
	}
	return(tgt);
}

multivector CopyMultivectorScaled(multivector tgt,multivector src,Real smult)
{
	if(src==MV_ZERO) {	KillMultivector(&tgt); return(MV_ZERO); }

	mvcapacity used=BitsSetBf(src->h.present);
	tgt=EnsureCapacity(tgt,src->h.level,used);
	tgt->h.present = src->h.present;
	mvcapacity i=0;
	if(src->h.level)
	{	for (;i<used;i++)	tgt->m[i] = CopyMultivectorScaled(tgt->m[i], src->m[i], smult);
	}
	else
	{	for(;i<used;i++)	((multivector0)tgt)->r[i] = rrMUL(((multivector0)src)->r[i],smult);
	}
	// zero unused coefficients 
	if(tgt->h.level>0) for(;i<=tgt->h.capacity;i++) ZeroMultivector(tgt->m[i],(mvlevel)(tgt->h.level-1));
	return(tgt);
}



multivector0 NewMultivector00(int cap,multivector0 src)
{
ASSERT(cap>0);
#if MV_RELEASE
	multivector0 mv=(multivector0)malloc(SIZEOF_OPENENDED(*mv,r) +  cap * sizeof(mv->r[0]));
ASSERT(mv!=NULL);
#else
	multivector0 mv;	SAFEMALLOC(mv,SIZEOF_OPENENDED(*mv) +  cap * sizeof(mv->r[0]));
#endif
	mv->h.present = MV_FLAG_NONE;
	mv->h.level = 0;
	mv->h.noncompacted = 0;
	mv->h.capacity = (mvcapacity)(cap-1);
	mv->h.freed = false;
	mv->h.refcount = 1;	// create with refcnt = 1 => 1 reference assumed
#if MV_NEWS
	if(src){	mv=CopyMultivectorCore0(__FILE__,__LINE__,mv,src); }
#else
	if(src){	mv=CopyMultivectorCore0(mv,src); }
#endif
	return(mv);
}

multivector0 NewMultivectorCore0(
#if MV_NEWS
char * file, int line,
#endif
int cap,multivector0 src)
{
ASSERT(cap>0);
	multivector0 mv = NewMultivector00(cap,src);
#if MV_NEWS
reattribute(mv,file,line);
#endif
	return(mv);
}


multivector NewMultivectorCore(
#if MV_NEWS
char * file, int line,
#endif
int cap,multivector src,mvlevel level)
{
ASSERT(cap>0);

	if(level)
	{
#if MV_RELEASE
	multivector mv=(multivector)malloc(SIZEOF_OPENENDED(*mv,m) +  cap * sizeof(mv->m[0]));
ASSERT(mv!=NULL);
#else
	multivector mv;	SAFEMALLOC(mv,SIZEOF_OPENENDED(*mv) +  cap * sizeof(mv->m[0]));
#endif

#if MV_NEWS
reattribute(mv,file,line);
#endif
	mv->h.present = MV_FLAG_NONE;
	mv->h.level = level;
	mv->h.noncompacted = 0;
	mv->h.capacity = (mvcapacity)(cap-1);
	mv->h.freed = false;
	mv->h.refcount = 1;	// create with refcnt = 1 => 1 reference assumed
  for(int i=0;i<cap;i++) mv->m[i]=MV_ZERO;
	if(src){	mv=CopyMultivector00(file,line,mv,src); }
	return(mv);
	}
#if MV_NEWS
	return((multivector)NewMultivectorCore0(file,line,cap,(multivector0)src));
#else
	return((multivector)NewMultivectorCore0(cap,(multivector0)src));
#endif
}

void KillMultivectorCore0(multivector0 * pmv
#if MV_NEWS
,char * fi,int li
#endif
)
{ // Recursively delete multivector 	
ASSERT(pmv!=NULL);
	if(*pmv!=NULL)
	{	if((*pmv)->h.refcount <= 1)
		{


#if MV_NEWS
DISCARD(fi); DISCARD(li);
#if MV_DEBUG
if(!CheckValidBlock(*pmv)) { ERRORCN(("Freeing a dosgy multivector0 at %i of %s ",li,fi))	; *pmv=NULL;return;}
if((*pmv)->h.freed)
{
	ERRORCN(("\Freeing a free multivector0 %p , %M [%s,%i]",*pmv,*pmv,fi,li))
  *pmv=NULL;
	return;
}	
#endif
#endif
			(*pmv)->h.freed = true;
	 		FREE(*pmv);
		}
		else (*pmv)->h.refcount --;
		*pmv=MV_ZERO;
	}
}

void KillMultivectorCore(multivector * pmv
#if MV_NEWS
,char * fi,int li
#endif
)
{ // Recursively delete multivector 	
ASSERT(pmv!=NULL);
	if(*pmv!=NULL)
	{	if((*pmv)->h.refcount <= 1)
		{

#if MV_DEBUG && MV_NEWS
if(!CheckValidBlock(*pmv)) { ERRORCN(("Freeing a dosgy multivector at %i of %s ",li,fi))	; *pmv=NULL;return;}
if((*pmv)->h.freed)
{
	ERRORCN(("\Freeing a free multivector %p , %M [%s,%i]",*pmv,*pmv,fi,li))
  *pmv=NULL;
	return;
}	
#else
ASSERT(!((*pmv)->h.freed));
#endif
			if((*pmv)->h.level)
			{	for (mvcapacity i=0;i<=(*pmv)->h.capacity;i++)	KillMultivector(&((*pmv)->m[i]));
			}
			(*pmv)->h.freed = true;
	 		FREE(*pmv);
		}
		else (*pmv)->h.refcount --;
		*pmv=MV_ZERO;
	}
}


// ------------------------------
//   Multivector construction
// ------------------------------

multivector Multivector0r(bladesflag bf,...)
{	// create a level 0 multivector passing real coefficients 
	va_list ap;	va_start(ap,bf);
	bladeindex passed=BitsSetBf(bf);
	multivector0 mv = NewMultivector0(passed, NULL);
	mv->h.present=bf;
ASSERT(passed < 100);
ASSERT(passed >=0);
ASSERT(mv->h.level==0);
	for (bladeindex i=0;i<passed;i++)	{ mv->r[i] = realToReal(va_arg(ap,real));}
  va_end(ap);
	return((multivector)mv);
}

multivector MultivectorScalarReal(Real x,mvcapacity cap)
{	multivector0 r = NewMultivector0(cap, NULL);	r->h.present=1;  r->r[0]=x;	return((multivector)r);}

multivector MultivectorScalar(real x) {	return(Multivector0r(0x1, x));}

multivector MultivectorPseudoScalar(Real x,mvcapacity cap) {	return( MultivectorBladeReal(x,(fullbladeindex)((1<<g_N)-1),cap));}

multivector Multivector3D1(real x,real y,real z) {	return(Multivector0r(0x16, x,y,z));}

multivector Multivector4D1(real x,real y,real z,real t)
{
#if MV_DIM0>=4
	return(Multivector0r(0x116, x,y,z,t));
#else
#if MV_DIM0==3
	multivector mv = NewMultivector(2,NULL,1);
	mv->h.present=3;
	mv->m[0] = Multivector3D1(x,y,z);
	mv->m[1] = MultivectorScalar(t);
#else
error

#endif

	return(mv);

#endif
}

multivector Multivector5D1(real x,real y,real z,real t,real u)
{
#if MV_DIM0>=5
	multivector mv = Multivector0r(0x80000116, x,y,z,t,u);
#else
#if MV_DIM0==4
	multivector mv = NewMultivector(2,NULL,1);
	mv->h.present=3;
	mv->m[0] = Multivector4D1(x,y,z,t);
	mv->m[1] = MultivectorScalar(u);
#else
#if MV_DIM0==3
	multivector mv = NewMultivector(3,NULL,1);
	mv->h.present=7;
	mv->m[0] = Multivector3D1(x,y,z);
	mv->m[1]= MultivectorScalar(t);
	mv->m[2]= MultivectorScalar(u);
#else
error
#endif

#endif

#endif
	return(mv);
}

real SquaredVector(real * p,int N) // calculate square of passed real vector accounting for current signatures
{	real tot=REAL_0;
	fullbladeindex bit=0x1;
	for(int i=0;i<N;i++)
	{	if(!(g_nulls & bit))
		{	real r = RRMUL(p[i],p[i]);
			if(g_negs & bit) r=-r;
			tot+=r;
		}
		bit<<=1;
	} 
	return(tot);
}

Real SquaredVectorf(Real * p,int N) // calculate square of passed real vector accounting for current signatures
{	Real tot=REAL_0;
	fullbladeindex bit=0x1;
	for(int i=0;i<N;i++)
	{	if(!(g_nulls & bit))
		{	Real r = rrMUL(p[i],p[i]);
			if(g_negs & bit) r=-r;
			tot+=r;
		}
		bit<<=1;
	} 
	return(tot);
}

multivector MultivectorSphere3D0(vector3D * p,real r) 
{	
	multivector mv;
	Real v = realToReal(r)/2;
	multivector t=MV_ZERO;
	mv = Multivector3D1(p->x,p->y,p->z);
	t=MultivectorAddScaled1(t,mv, MV_ep[0], v-Real_HALF);
	mv=MultivectorAddScaled1(mv,t, MV_em[0], v+Real_HALF);
	KillMultivector(&t);
	return(mv);
}




multivector MultivectorSphere3D(vector3D * p,real sqdRadius,boolean dual) 
{	multivector mv = MultivectorSphere3D0(p, SquaredVector((real *)p,3) - sqdRadius); 

	if(dual)
	{	multivector mv2=MultivectorDual(mv,g_N,MV_GEOMETRIC_PRODUCT);
		KillMultivector(&mv);
REPORT2(MV_NEWS,5,("return dual sphere %M ",mv2))
		return(mv2);
	}
REPORT2(MV_NEWS,5,("return sphere %M ",mv))
	return(mv);
}

multivector MultivectorPoint(vector3D * p) { return( MultivectorSphere3D0(p, SquaredVector((real *)p,3) ) );}

multivector MultivectorPoint(vector4D * p) 
{
STRACE1
	multivector mv = NewMultivector(3,NULL,1);
#if MV_DIM0==4
	mv->h.present = (1 | (1<<(mv_bf_ep[0]>>MV_DIM0)) | (1<<(mv_bf_em[0]>>MV_DIM0))) ;// want 1,e6,e7
#endif
	real r = SquaredVector((real *)p,4);
	Real v = realToReal(r)/2;
	mv->m[0] = Multivector4D1(p->x,p->y,p->z,p->s);
	mv->m[1] = MultivectorScalarReal(v-Real_HALF);
	mv->m[2] = MultivectorScalarReal(v+Real_HALF);
	return(mv);
}

multivector MultivectorBladeReal(Real co,fullbladeindex bi,mvcapacity cap)
{	mvlevel lv=0;
	multivector mv;
	do
	{	bladeindex sub=(bladeindex)(bi & MV_INDEX_MAX);
		bi>>=MV_DIM0;
		multivector mv2;
		if(bi==0) mv2=NewMultivector(cap,NULL,lv);
		else mv2=NewMultivector(1,NULL,lv);
REPORT2(MV_NEWS,5,("\n MvBladeReal %p lv=%X has blade %X [bi=%X] coeff=%r cap=%X",mv2, lv,sub,bi,co,cap));
		mv2->h.present = 1<<sub;
		if(lv) mv2->m[0] = mv; else ((multivector0)mv2)->r[0] = co;
		mv = mv2;
		lv++;
	}	while(bi);
	return(mv);
}

multivector MultivectorBlade(real co,fullbladeindex bi,mvcapacity cap)
{
	return(MultivectorBladeReal(realToReal(co),bi,cap));
}


void VectorFromMultivectorFast(real * pr,multivector0 a,mvgrade k)
{	// Assumes a is a 1-vector, returns first k coordinates. 
	ASSERT(a->h.level==0);
	ASSERT2((a->h.present&1)==0);
	ASSERT2(k>0);
	ASSERT2(k<=MV_DIM0);
	fint j=0;
	if(a->h.present&0x0002)	pr[0]=RealToreal(a->r[j++]);else pr[0]=REAL_0;
	if(k<2) return;
	if(a->h.present&0x0004)	pr[1]=RealToreal(a->r[j++]);else pr[1]=REAL_0;
	if(k<3) return;
	if(a->h.present&0x0010)	pr[2]=RealToreal(a->r[j++]);else pr[2]=REAL_0;
#if MV_DIM0>=4
	if(k<4) return;
	if(a->h.present&0x0100)	pr[3]=RealToreal(a->r[j++]);else pr[3]=REAL_0;
#endif
#if MV_DIM0>=5
	if(k<5) return;
	if(a->h.present&0x80000116)	pr[3]=RealToreal(a->r[j++]);else pr[3]=REAL_0;
#endif
}


void Multivector0FromVectorfFast(multivector0 a,Real * pr,mvgrade k)
{	// Assumes a is a 1-vector broad enough to hold it, returns first k coordinates. 
	ASSERT(a->h.level==0);
	ASSERT2(k>0);
	ASSERT2(k<=MV_DIM0);

	if(a->h.capacity<k-1)
	{
		ERRORCN(("Multivector0FromVectorfFast k=%X has undercapicited at %X ",k,a->h.capacity))
		k=(mvgrade)(a->h.capacity+1);
	}



	for(fint j=0;j<k;j++)	a->r[j]=pr[j]; 
}

void MultivectorFromVectorfFast(multivector a,Real * pr,mvgrade k)
{
	
	if(a->h.level==0)
	{
		if(k<=MV_DIM0)
		{	Multivector0FromVectorfFast((multivector0)a,pr,k);
			return;
		}
		else
		{
ERRORCN(("\n Bugger! Prepping L=0 multivector with k=%X",k)) 

		}
 	} 
	else
	{
	ASSERT2(a->h.level==1);
	ASSERT2(a->h.present &1);
ASSERT(k<2*MV_DIM0);
	Multivector0FromVectorfFast((multivector0)a->m[0],pr,MV_DIM0);
	for(fint j=1;j<=k-MV_DIM0;j++)
	{
ASSERT2(a->m[j]->h.level==0);
ASSERT2(a->m[j]->h.present==1);
		((multivector0)a->m[j])->r[0] =  pr[j+MV_DIM0-1];		
	}
	}
}

void MultivectorFromVectorFast(multivector a,real * pr,mvgrade k)
{	Real RR[MV_DIM1];
	for(fint j=0;j<k;j++) RR[j]=realToReal(pr[j]);
	MultivectorFromVectorfFast(a,RR,k);
}


void MultivectorFromPointFast(multivector a,posn3D * p) // x + em(x^2+1)/2 +ep(x^2-1)/2
{
	Real RR[5];
	RR[0]=realToReal(p->x);
	RR[1]=realToReal(p->y);
	RR[2]=realToReal(p->z);
	RR[3]=(SquaredVectorf(RR,3)+Real_1)/2 ;
	RR[4]=RR[3]-Real_1; 
	MultivectorFromVectorfFast(a,RR,5);
}

void MultivectorFromSphereFast(multivector a,posn3D * p,real Rsqrd)
{
	Real RR[5];
	RR[0]=realToReal(p->x);
	RR[1]=realToReal(p->y);
	RR[2]=realToReal(p->z);
	RR[3]=(SquaredVectorf(RR,3)- realToReal(Rsqrd) + Real_1)/2 ;
	RR[4]=RR[3]-Real_1; 
	MultivectorFromVectorfFast(a,RR,5);
}


#if MV_MAX_GHC>1
void MultivectorFromSphereFastDbl(multivector a,posn3D * p,real Rsqrd)
{
	Real RR[7];
	Real ro2=  realToReal(Rsqrd);
	RR[0]=realToReal(p->x);
	RR[1]=realToReal(p->y);
	RR[2]=realToReal(p->z);
	RR[3]=(SquaredVectorf(RR,3)- ro2 + Real_1)/2 ;	// em[0] component = (1+c^2-R^2)/2
	RR[4]=RR[3]-Real_1; 
	// Have is a GHC(1) vector of square ro2
	RR[5]=(ro2+Real_1)/2;  // em[1] component
	RR[6]=RR[5]-Real_1; 	 // ep[1] component
	MultivectorFromVectorfFast(a,RR,7);
}
#endif


multivector BuildMultivectorr(int cnt,...)
{	fullbladeindex bi[1<<MV_DIM0];
	Real	coeff[1<<MV_DIM0];
	va_list ap;	va_start(ap,cnt);
	fullbladeindex max=0;
ASSERT(cnt<10);
	int i,j;	
 	for(i=0;i<cnt;i++)
	{	coeff[i] = realToReal(va_arg(ap,real));
	 	bi[i]=va_arg(ap,fullbladeindex);
		if(bi[i]>=max) max=bi[i],j=i;
	}
  va_end(ap);
	multivector mv = MultivectorBladeReal(coeff[j],bi[j],(mvcapacity)cnt);
	cnt--;
  bi[j]=bi[cnt]; coeff[j]=coeff[cnt];
	while(--cnt>=0)	{ multivector mv2 = MultivectorBladeReal(coeff[cnt],bi[cnt],1);	mv=MultivectorAddMerge(mv,mv2);	}
	return(mv);
}	 

multivector BuildMultivectorReal(int cnt,...)
{	fullbladeindex bi[1<<MV_DIM0];
	Real	coeff[1<<MV_DIM0];
	va_list ap;	va_start(ap,cnt);
	fullbladeindex max=0;
ASSERT(cnt<10);
	int i,j;	
	for(i=0;i<cnt;i++)
	{	coeff[i] = va_arg(ap,Real);
	 	bi[i]=va_arg(ap,fullbladeindex);
		if(bi[i]>=max) max=bi[i],j=i;
	}
  va_end(ap);
	multivector mv = MultivectorBladeReal(coeff[j], bi[j],(mvcapacity)cnt);
	cnt--;
	if(cnt!=j) { bi[j]=bi[cnt]; coeff[j]=coeff[cnt];}
	while(--cnt>=0)	mv=MultivectorAddMerge(mv, MultivectorBladeReal(coeff[cnt],bi[cnt],1));
//	{ multivector mv2 = MultivectorBladeReal(coeff[cnt],bi[cnt],1);
//		mv=MultivectorAddMerge(mv,mv2);
//	}
	return(mv);
}	 


// --------------------------------------------
//   functions acting on single-level blades
// --------------------------------------------

boolean OddBlade(bladeindex bi) { return(BitParity(bi));}

bladeindex FirstBlade(bladesflag bf) { return((bladeindex)BSF_32(bf));	}

mvlevel LevelForBlade(fullbladeindex fbi)
{	int lv=-1;
	do { lv++;fbi>>=MV_DIM0;}	while(fbi>0);
	return((mvlevel)lv);
}


#if 1
boolean AntiCommutingBlades(bladeindex bi,bladeindex bj) // return 1 iff blades anticommute                
{	if(OddBlade(bj)) return(BitParity((bladeindex)(bi & (~bj))));
	else return(BitParity(bi & bj));
}

#else // alternate slower method
boolean AntiCommutingBlades(bladeindex bi,bladeindex bj) // return true iff blades anticommute
{	boolean oddj = OddBlade(bj);
	boolean sign=0;
	bladesflag	bit = 1<<((1<<MV_DIM0)-1);
	do
	{	if(bi & bit)
		{	if(bj & bit) {	if (!oddj) sign ^=1; }
			else sign ^= oddj;
 		}
		bit>>=1;
 	}	while(bit);
REPORT2(MV_NEWS,5,("\n Anticommution of %O and %O is %X ",bi,0,bj,0,sign));
 	return(sign);
}
#endif


int	BladeSignature(bladeindex bi,mvlevel lv,mvproduct pro=MV_GEOMETRIC_PRODUCT)
{	bladeindex r=BladeBladeProduct(bi,bi,&pro,lv);
	if(r==MV_INDEX_NIL) return(0);
	if(pro & MV_LEFT_NEGATION) return(-1);
	return(1);
}

// ------------------------------------------------
//   functions acting on full (multi-level) blades
// ------------------------------------------------


int BladeSignature(fullbladeindex bi)
{	int r=1;
	int grds[MV_NUM_LEVEL];
	mvlevel lv=0;
	do
 	{	bladeindex b= (bladeindex)(bi & MV_INDEX_MAX);
ASSERT(lv<MV_NUM_LEVEL);
		int sig=BladeSignature(b,lv);
		if(sig==0) return(0);
		if(sig<0)  r=-r;
		grds[lv]=BitsSetBi(b);
		if(grds[lv]&1)	for(mvlevel lv2=0;lv2<lv;lv2++)	if(grds[lv2]&1) r=-r;
		lv++;
		bi >>= MV_DIM0;
	}	while(bi>0);
	return(r);
}


// -----------------------------------------------------
//   levelity specific functions acting on multivectors
// -----------------------------------------------------


multivector MultivectorInhibitBlades(multivector tgt, multivector src, bladesflag * inhibit)
{	// This inbits blades at each level rather entire blades, accepting NUM_LEVEL bladeflags rather than a 2^(2^N) bit stream
	// It is thus implimenmtation-dependant, effected by choice of MV_DIM0.
	// Passing ~1 for each level will result in the scalar part overall but passing ~(1<<MV_DIM0) will return exclusively the
	// pseudoscalar component only for level 0 multivectors
	bladesflag bftgt=(bladesflag)(src->h.present & (~inhibit[src->h.level]));
	
	if(bftgt==MV_FLAG_NONE) {	KillMultivector(&tgt);	return(MV_ZERO);}
	
	tgt=EnsureCapacity(tgt,src->h.level,BitsSetBf(bftgt));
	tgt->h.present = bftgt;
	bladesflag bit=1;
	int i=0; int j=0;
	if(tgt->h.level==0)
	{	while(bit && (bit <= tgt->h.present))
		{	if(src->h.present & bit) {	if(!(inhibit[0] & bit)) ((multivector0)tgt)->r[j++] = ((multivector0)src)->r[i]; i++;	}
			bit<<=1;
		}
	}
	else
	while(bit && (bit <= tgt->h.present))
	{	if(src->h.present & bit)
		{	if(!(inhibit[src->h.level] & bit)) 	{ tgt->m[j] = MultivectorInhibitBlades(tgt->m[j], src->m[j], inhibit);j++;	}
			i++;
		}
		bit<<=1;
	} // endwhile
	return(tgt);
}


// --------------------------------------------------------
//   levelity independant functions acting on multivectors
// --------------------------------------------------------


// ------------------------------
//   Multivector computations
// ------------------------------

boolean IsSmall(multivector a,Real d)
{	if(a==MV_ZERO) return(true);
	int used=BitsSetBf(a->h.present);	if(used==0) return(true);
	int i=0;
	if(a->h.level>0)	{	for(;i<used;i++) if(!IsSmall(a->m[i],d)) return(false);	}
	else							{	for(;i<used;i++) if((((multivector0)a)->r[i]>d)||(((multivector0)a)->r[i]<-d)) return(false);	}
	return(true);
}

boolean IsZero(multivector a)	// Could use IsSmall but keep seperate to optimise
{	if(a==MV_ZERO) return(true);
	int used=BitsSetBf(a->h.present);	if(used==0) return(true);
	int i=0;
	if(a->h.level>0)	{	for(;i<used;i++) if(!IsZero(a->m[i])) return(false);	}
	else							{	for(;i<used;i++) if(!EQUIVALENT_REALS(((multivector0)a)->r[i], Real_0)) return(false);	}
	return(true);
}



boolean IsScalar(multivector a)
{	if(a==MV_ZERO) return(true);

if(!CheckValidMV(a)) return(false);

	int used=BitsSetBf(a->h.present);
	if(used==0) return(true);
	int i;
	if(a->h.level>0)
	{	for(i=0;i<used;i++)
		{ if(!IsScalar(a->m[i])) return(false);
			Real r=ScalarPart(a->m[i]);
ASSERT2(ValidReal(r));
			if( !( EQUIVALENT_REALS(r,Real_0) || ((i==0)&&(a->h.present&1)) ) )	return(false);
		}
	}
	else
	{	
		for(i=(a->h.present&1);i<used;i++)
		{	Real r = ((multivector0)a)->r[i];
ASSERT2(ValidReal(r));
		 	if(!EQUIVALENT_REALS(r, Real_0)) return(false);
		}
	}
	return(true);
}

int NumBlades(multivector a)
{	if(a==MV_ZERO) return(0);
	int used = BitsSetBf(a->h.present);
	if(a->h.level==0) return(used);
	int tot=0; for(int i=0;i<used;i++) tot+=NumBlades(a->m[i]);
	return(tot);
}

int NumBladesWithCorrection0(multivector0 a)
{	if(a==MV_ZERO) return(0);
	ASSERT(a->h.level==0);

ASSERT2(CheckValidMV((multivector)a));

REPORT2(MV_NEWS,4,("\n NumBladesWithCorrect0 acts on (%p=%M) pres=%X ",a,a, a->h.present))
	
	bladesflag bfr=a->h.present;
	bladeindex indr=0;	int cntr=0;int cnts=0;
	while(bfr)
 	{	SHIFT_RGT_CNT(shf,bfr,indr)
		if(!EQUIVALENT_REALS(a->r[cnts], Real_0)) a->r[cntr++]=a->r[cnts];
		else
		{
REPORT2(MV_NEWS,2,("\n NumBlades0 pruning zero real[%X] from %M clearing bit %X of %X ",cnts,a,(1<<indr),a->h.present))
			 a->h.present ^= (1<<indr);
		}
		cnts++;	indr++;
	} // endwhile
REPORT2(MV_NEWS,5,("\n NumBladesWithCorrect0 concludes with pf=%X res=%X ",a->h.present,cntr))
	return(cntr);
}


int NumBladesWithCorrection(multivector a)
{	if(a==MV_ZERO) return(0);
ASSERT2(CheckValidMV(a));
	if(a->h.level==0) return(NumBladesWithCorrection0((multivector0)a));
REPORT2(MV_NEWS,4,("\n NumBladesWithCorrect acts on (%p=%M) pres=%X ",a, a, a->h.present))

	bladesflag bfr=a->h.present;
	bladeindex indr=0;	int cntr=0; int cnts=0;
	int tot=0;
	while(bfr)
 	{	SHIFT_RGT_CNT(shf,bfr,indr)
		int cnt= NumBladesWithCorrection(a->m[cnts]);
		if(cnt)	{ a->m[cntr++]=a->m[cnts]; tot+=cnt;}
		else
		{
REPORT2(MV_NEWS,2,("\n NumBlades pruning zero MV[%X] %p from (%p=%M) lv=%X removing %X from pres=%X  ",cnts, a->m[cnts], a, a, a->h.level, (1<<indr), a->h.present ))
			 KillMultivector(a->m + cnts);
			 a->h.present ^= (1<<indr);
REPORT2(MV_NEWS,5,("... res is %X cntr=%X cnts=%X",a->h.present, cntr, cnts ))
		}
		cnts++;	indr++;
	} // endwhile

REPORT2(MV_NEWS,4,("\n NumBladesWithCorrect concludes with pf=%X res=%X ",a->h.present,cntr))
ASSERT2(CheckValidMV(a));
	return(tot);
}


mvgradesflag GradesPresent(multivector a) // Bit i set in result if a constains grade i components 
{	if(a==MV_ZERO) return(0);
	mvgradesflag gf=0;
	bladesflag bit=1;
	int i=0;bladeindex inda=0;
	
	if(a->h.level>0)
	{	while(bit && (bit <= a->h.present))
		{	if(a->h.present & bit)
			{	int grdb = BitsSetBi(inda);
				gf |= (GradesPresent(a->m[i])<<grdb);
				i++;
			}
			inda++;	bit<<=1;
		} // endwhile
	}
	else
	{	while(bit && (bit <= a->h.present))
		{	if(a->h.present & bit)
			{	if(!EQUIVALENT_REALS(((multivector0)a)->r[i],Real_0))	gf |= (1<<BitsSetBi(inda));
				i++;
			}
			inda++;	bit<<=1;
		} // endwhile
	} // endelse
	return(gf);
}


Real PrincipleBlade(fullbladeindex * fbi,multivector a) // return coordinate and index of maximal-modulus coordinate basis blade 
{	if(a==MV_ZERO) return(0);
	bladesflag bit=1;
	Real max=Real_0;
	Real v=Real_0;
	int i=0;bladeindex inda=0;
	if(a->h.level>0)
	{
#if MV_VSR_LEVEL
ASSERT(a->h.level < MV_VSR_LEVEL);
#endif
		while(bit && (bit <= a->h.present))
		{	if(a->h.present & bit)
			{
				fullbladeindex b;
				Real c = PrincipleBlade(&b,a->m[i]);
				Real absc = Real_ABS(c);
				if(absc>max) { max=absc; v=c; *fbi=(inda<<(a->h.level*MV_DIM0)) | b; }			
				i++;
			}
			inda++;	bit<<=1;
		} // endwhile
	}
	else
	{	while(bit && (bit <= a->h.present))
		{	if(a->h.present & bit)
			{	Real c = ((multivector0)a)->r[i];
				Real absc = Real_ABS(c);
				if(absc>max) { max=absc; v=c; *fbi=inda; }
				i++;
			}
			inda++;	bit<<=1;
		} // endwhile
	} // endelse
	return(v);
}



#if MV_DIM1<=32


int UniGrade(multivector a) // return -1 if multigrade or grade if single-graded
{ mvgradesflag gf=GradesPresent(a);
	if(BitsSetFbi(gf)==1)
	 return(BSF_32(gf));
	return(-1);
}


mvgrade MinGrade(multivector a)	{ mvgradesflag gf=GradesPresent(a); if(gf) return((mvgrade)BSF_32(gf)); else return(-1);}

mvgrade MaxGrade(multivector a)	{ mvgradesflag gf=GradesPresent(a);	if(gf) return((mvgrade)BSR_32(gf)); else return(-1);}


#else
#error "unwritten code"
#endif



#define MV_error(TXT) ERROREX(TXT) 


multivector DeFactorise(multivector a)
{	// expands a VSR or OTR factored multivector a into a new multivector, preserving a
 //	if(a==MV_ZERO) return(MV_ZERO);

ASSERT(a!=MV_ZERO);
	if(a->h.level<MV_VSR_LEVEL) return(Reference(a));
	int used = a->h.present;
	if(used==0) return(MV_ZERO);
	multivector r = DeFactorise(a->m[0]);

	if(used>1)
	{	
//		if(MV_stk_index>=MV_STK_DEPTH-1) MV_error(("MV-Stack overflow.")); 
//		MV_stk_index++;

		if(a->h.level==MV_VSR_LEVEL)
		for(int i=1;i<used;i++)
		{	multivector s=DeFactorise(a->m[i]);
			r=MultivectorProduct(r,r,s);
			KillMultivector(&s);
		}
		else
		for(int i=1;i<used;i++)
		{	multivector s=DeFactorise(a->m[i]);
			r=MultivectorProduct(r,r,s,MV_OUTTER_PRODUCT);
			KillMultivector(&s);
		}

//		MV_stk_index--;
	}

	return(r);
}


void DeFactoriseMe(multivector * a)
{	if((*a)->h.level>=MV_VSR_LEVEL)
	{	multivector r= DeFactorise(*a);
		KillMultivector(a);
		*a=r;
	}
}



multivector MultivectorRestrictGrade(multivector tgt, multivector src, int grade)
{
	if(src==MV_ZERO)	{	ZeroMultivector(tgt,0);	return(tgt);}

	if(tgt==src) { multivector r = MultivectorRestrictGrade(NULL,src,grade);	KillMultivector(&src);return(r);	}
	
#if MV_VSR_LEVEL
	if(src->h.level>=MV_VSR_LEVEL)
	{ DeFactoriseMe(&src); // if versor multiply out then take scalar part :-(
		tgt = MultivectorRestrictGrade(tgt,src,grade);
		KillMultivector(&src);
		return(tgt);
	}
#endif	

	if(grade==-1)	grade=MinGrade(src);
	if(grade==-2)	grade=MaxGrade(src);
	
	// Check for trivial cases
	if((grade<0)||(grade>g_N)||(src==MV_ZERO))	{	KillMultivector(&tgt);return(MV_ZERO);}

	// Use optimised code for scalar restriction
	if(grade==0) 	{	multivector t=MultivectorScalarReal(ScalarPart(src),1);		KillMultivector(&tgt);	return(t);	}
	
	if(src->h.level==0)
	{	if(grade>MV_DIM0) { KillMultivector(&tgt); return(MV_ZERO); }
	 	return( MultivectorInhibitBlades(tgt,src,&(mv_grade_inhibit[grade])) );
	}

REPORT2(MV_NEWS,5,("\n Restrict %M to grade %i",src,grade))

	tgt=EnsureCapacity(tgt,src->h.level,(mvcapacity)(src->h.capacity+1));

	ZeroMultivector(tgt,src->h.level);

ASSERT(tgt!=NULL);

	tgt->h.present = MV_FLAG_NONE;
	bladesflag bit=1;
	int i=0;bladeindex inds=0;
	int j=0;
	while(bit && (bit <= src->h.present))
	{	if(src->h.present & bit)
		{	int grdb = BitsSetBi(inds);
			if(grdb<=grade)
			{	tgt=EnsureCapacity2(tgt,src->h.level,(mvcapacity)(j+2),1);
				if(src->m[i]!=MV_ZERO)	tgt->m[j] = MultivectorRestrictGrade(tgt->m[j], src->m[i], grade-grdb);
				if(tgt->m[j]!= MV_ZERO)	{	tgt->h.present |= bit;	j++;	}		
			}
			i++;
		}
		inds++;
		bit<<=1;
	} // endwhile
REPORT2(MV_NEWS,6,("\n Restricted result is %M",tgt))
	return(tgt);
}



multivector MultivectorDual(multivector a,int axies,mvproduct pro)
{	// Right multiply by MV_i_inv -- optimised routine may be faster
DISCARD(axies);
STRACE2
	return(MultivectorProduct(NULL,a,mv_i_inv.m,pro));
}


void ConjugatedCopies(multivector * tgt,multivector * src,int num,mvconjugation cnj) 
{	int i;

ASSERT(cnj!=MV_CONJUGATION_NONE);
ASSERT(tgt!=src);

	if(cnj & MV_CONJUGATION_REVERSE)
	{	tgt[0]= MultivectorConjugation(src[num-1],cnj);
		cnj &= (~MV_CONJUGATION_NEGATION);
		for(i=1;i<num;i++) tgt[i]= MultivectorConjugation(src[num-1-i],cnj);
	}
	else
	{	tgt[0]= MultivectorConjugation(src[0],cnj);
		cnj &= (~MV_CONJUGATION_NEGATION);
		if(cnj) for(i=1;i<num;i++) tgt[i]=Reference(src[i]);
		else for(i=1;i<num;i++) tgt[i]= MultivectorConjugation(src[i],cnj);
	}
}

multivector MultivectorConjugation(multivector a, mvconjugation cnj)
{	multivector tgt;
ASSERT(a!=MV_ZERO);
	if(cnj==MV_CONJUGATION_NONE) return(Reference(a));

REPORT2(MV_NEWS,6,("\n Conjugation(%X) of %M",cnj,a))

	mvlevel lev=a->h.level;
#if MV_VSR_LEVEL
	if(lev >=MV_VSR_LEVEL)
	{	// conjugate versor or blade 
		int used= a->h.present;
		if(used>0)
		{	tgt=NewMultivector(used,NULL,lev);
			ConjugatedCopies(&(tgt->m[0]),&(a->m[0]),used,cnj); 
			return(tgt);
		}			
		return(MV_ZERO);
	}
#endif

	int i=0;

	bladesflag bfa=a->h.present;
	bladesflag bit=1;
	int used = BitsSetBf(bfa);
	if(used)
	{	tgt=NewMultivector(used,NULL,lev);
		if(lev==0)
		{	while(bit && (bit <= a->h.present))
			{	if(a->h.present & bit) 
	 			{	if(mv_conj[cnj][0] & bit)	((multivector0)tgt)->r[i] = -((multivector0)a)->r[i];else ((multivector0)tgt)->r[i] = ((multivector0)a)->r[i];
					i++;
				}
				bit<<=1;
			}	// endwhile
REPORT2(MV_NEWS,6,(" at level0 is %M",a))
		}
		else
		{	bladeindex bi=0;
			while(bit && (bit <= a->h.present))
			{	if(a->h.present & bit)
				{	mvconjugation cnj2=cnj; 
	REPORT2(MV_NEWS,5,("\n Conjugating blade %X=%O bit=%X data=%X -> %X ",bi,bi,a->h.level,bit, mv_conj[cnj][a->h.level] , bit & mv_conj[cnj][a->h.level]))
					if( (cnj & MV_CONJUGATION_REVERSE) && OddBlade(bi) )
					{
	REPORT2(MV_NEWS,5,("\n reversing so commute across coeff %M",a->m[i]))
					 cnj2 ^=MV_CONJUGATION_INVOLUTION;
					}
			 		if(bit & mv_conj[cnj][a->h.level])
			 		{
	REPORT2(MV_NEWS,5,("\n blade %O flips under %X so negate coeff %M",bi,a->h.level, cnj, a->m[i]))
			 			cnj2 ^= MV_CONJUGATION_NEGATION;
					}
	ASSERT(a->m[i]!=NULL);
			 		tgt->m[i]=MultivectorConjugation(a->m[i], cnj2);
					i++;
				}
				bi++;
				bit<<=1;
			}	
		}
		return(tgt);
	}
	return(MV_ZERO);
}





void MultivectorConjugate(multivector a, mvconjugation cnj)
{
ASSERT(a!=NULL);
REPORT2(MV_NEWS,6,("\n Conjugating(%X) %M",cnj,a))
	bladesflag bfa=a->h.present;		 
	bladesflag bit=1;
	int i=0;
	if(a->h.level==0)
	{	while(bit && (bit <= a->h.present))
		{	if(a->h.present & bit) 
	 		{	if(mv_conj[cnj][0] & bit)	((multivector0)a)->r[i] = -((multivector0)a)->r[i];
				i++;
			}
			bit<<=1;
		}	
REPORT2(MV_NEWS,6,(" at level0 is %M",a))
	}
	else
	{
#if MV_VSR_LEVEL
		if(a->h.level>=MV_VSR_LEVEL)
		{	// conjugate versor or blade 
			int used= a->h.present;
			if(used>0)
			{	int i;
				if(cnj & MV_CONJUGATION_REVERSE) for(i=0;i<used/2;i++) SWAPPTRS(a->m[i], a->m[used-1-i]);
				MultivectorConjugation(a->m[0],cnj);
				cnj &= (~MV_CONJUGATION_NEGATION);
				for(i=1;i<used;i++) MultivectorConjugate(a->m[i],cnj);
			}
		}
		else
#endif
		{	bladeindex bi=0;
			while(bit && (bit <= a->h.present))
			{	if(a->h.present & bit)
				{	mvconjugation cnj2=cnj; 
	REPORT2(MV_NEWS,5,("\n Conjugating blade %X=%O bit=%X data=%X -> %X ",bi,bi,a->h.level,bit, mv_conj[cnj][a->h.level] , bit & mv_conj[cnj][a->h.level]))
					if( (cnj & MV_CONJUGATION_REVERSE) && OddBlade(bi) )
					{
	REPORT2(MV_NEWS,5,("\n reversing so commute across coeff %M",a->m[i]))
					 cnj2 ^=MV_CONJUGATION_INVOLUTION;
					}
			 		if(bit & mv_conj[cnj][a->h.level])
			 		{
	REPORT2(MV_NEWS,5,("\n blade %O flips under %X so negate coeff %M",bi,a->h.level, cnj, a->m[i]))
			 			cnj2 ^= MV_CONJUGATION_NEGATION;
					}
	ASSERT(a->m[i]!=NULL);
			 		MultivectorConjugate(a->m[i], cnj2);
					i++;
				}
				bi++;
				bit<<=1;
			}	
		}
	}
}


void MultivectorScale(multivector a, Real lambda)
{
ASSERT(a!=NULL);
	bladesflag bfa=a->h.present;
	bladesflag bit=1;
	int i=0;
	if(a->h.level==0)
	{	while(bit && (bit <= a->h.present))
		{	if(a->h.present & bit) {	((multivector0)a)->r[i] = rrMUL(lambda,((multivector0)a)->r[i]);i++;}
			bit<<=1;
		}	
	}
	else
	{
#if MV_VSR_LEVEL
		if(a->h.level>=MV_VSR_LEVEL) { if(a->h.present>0) MultivectorScale(a->m[0],lambda);}
		else
#endif
		{	bladeindex bi=0;
			while(bit && (bit <= a->h.present))
			{	if(a->h.present & bit) {	MultivectorScale(a->m[i], lambda);i++;}
				bit<<=1;
			}	
		}
	}
}


multivector MultivectorAdd(multivector a, multivector b)
{	// a = a + b , leave b extant
	if(b==MV_ZERO) return(a);
ASSERT(a!=NULL);
ASSERT(a!=b);
REPORT2(MV_NEWS,5,("\n MultivectorAdd %M and %M %X vs %X ",a,b,a->h.level, b->h.level));

#if MV_VSR_LEVEL
	DeFactoriseMe(&a);
	b=DeFactorise(b);
#endif

	if(a->h.level==b->h.level)
	{	bladesflag bfr = (bladesflag)(a->h.present | b->h.present);
		a=AllocateSpace(a,a->h.level,bfr);		
		bladeindex cnta=0; bladeindex cntt=0;	bladeindex cntb=0;
		bladeindex maxa=BitsSetBf(a->h.present);
		bladesflag bit=1;
	 	while(bit && (bit<=bfr))
	 	{	if(bit & b->h.present)
			{	if(bit & a->h.present)
 				{	if(a->h.level==0)
 					{	Real c = ((multivector0)a)->r[cnta] + ((multivector0)b)->r[cntb];
	 					if(EQUIVALENT_REALS(c,Real_0)) bfr ^= bit;
	 					else	((multivector0)a)->r[cntt++]=c;
					}
 					else
 					{ a->m[cntt++] = MultivectorAdd(a->m[cnta] , b->m[cntb]);
					}
				}
				else
				{
					if(a->h.level>0)
					{	for(mvcapacity i=maxa;i>cnta;i--) a->m[i] = a->m[i-1];
						a->m[cntt++] = CopyMultivector(NULL,b->m[cntb]);
					}
					else 
					{
						for(mvcapacity i=maxa;i>cnta;i--) ((multivector0)a)->r[i] = ((multivector0)a)->r[i-1];
						((multivector0)a)->r[cntt++] = ((multivector0)b)->r[cntb];
					}
					maxa++;
				}
				cnta++; cntb++; 
			}
			else if(bit & a->h.present)
			{	if(a->h.level>0) a->m[cntt++] = a->m[cnta++];
			 	else ((multivector0)a)->r[cntt++] = ((multivector0)a)->r[cnta++];
			}
			bit<<=1;
		}	// endwhile
		a->h.present = bfr;
REPORT2(MV_NEWS,5,("\n MultivectorAdd res = %M ",a));
	}
	else
	{
		if(a->h.level>b->h.level)
	 	{	if(a->h.present & 1) a->m[0]= MultivectorAdd(a->m[0] , b);
			else
			{	bladeindex maxa=BitsSetBf(a->h.present);
				a = EnsureCapacity(a, a->h.level, (mvcapacity)(maxa+1));
				for(mvcapacity i=maxa;i>0;i--) a->m[i] = a->m[i-1];
				a->h.present |=	1;
				a->m[0] = CopyMultivector(NULL,b);
			}
		}
		else
		{	// b->h.level > a->h.level 
REPORT2(MV_NEWS,6,("\n b=%p=%M added over %M ",b,b,a));
			bladesflag bftgt= (bladesflag)(b->h.present | 1);
			mvcapacity cap = BitsSetBf(bftgt);
			multivector mv = NewMultivector(cap,NULL,b->h.level);
			mv->m[0] = a;
REPORT2(MV_NEWS,7,("\n give mv=%p a=%p ",mv,a));
			mvcapacity cntb = (mvcapacity)(b->h.present & 1);
			if(cntb) mv->m[0] = MultivectorAdd(mv->m[0] , b->m[0]);
			for(bladeindex cnta=1;cnta<cap;cnta++)
			{ mv->m[cnta] = CopyMultivector(NULL, b->m[cntb] );	
				cntb++;
			}
			mv->h.present = bftgt;
			a=mv;
		}
	}
#if MV_VSR_LEVEL
	KillMultivector(&b); // will just dereference unless b was originally factorised
#endif
	return(a);
}


multivector MultivectorAddMerge(multivector a, multivector b)
{	// Return a + b , using either a or b to hold it, freeing the other 
	if(b==MV_ZERO) return(a);
	if(a==MV_ZERO) return(b);

REPORT2(MV_NEWS,5,("\n MultivectorAddMerge %M and %M %p and %p",a,b,a,b));

ASSERT(a!=b);

	DeFactoriseMe(&a); DeFactoriseMe(&b);


	if(a->h.level == b->h.level)
	{	if(a->h.capacity < b->h.capacity) SWAPPTRS(a,b);
		a=MultivectorAdd(a,b);
		KillMultivector(&b);
		return(a);
	}

	if(a->h.level < b->h.level)	SWAPPTRS(a,b);

ASSERT(a->h.level >	b->h.level);

	if(a->h.present & 1)
	{ a->m[0] = MultivectorAddMerge(a->m[0] , b);
	}
	else
	{	bladeindex maxa=BitsSetBf(a->h.present);
REPORT2(MV_NEWS,5,("\n merging %M =%p into sclar slot of %M =%p",b,b,a,a));
		a = EnsureCapacity(a, a->h.level, (mvcapacity)(maxa+1));

ASSERT(a->h.level >	0);

		for(mvcapacity i=maxa;i>0;i--) a->m[i] = a->m[i-1];
		a->h.present |=	1;
		a->m[0] = b;
REPORT2(MV_NEWS,6,("\n return %M =%p sclar slot %M=%p",a,a,a->m[0],a->m[0]));
	}
	return(a);
}

multivector MultivectorAddMergeScaled(multivector a, multivector b,Real bmult)
{	// Return a + bmult*b , freeing a and b
	if(b==MV_ZERO) return(a);
	if(a==MV_ZERO) return(CopyMultivectorScaled(b,b,bmult));
	
	DeFactoriseMe(&a); DeFactoriseMe(&b);
	multivector r=MultivectorAddScaled1(NULL,a,b,bmult);
	KillMultivector(&a); KillMultivector(&b);
	return(r);	
}


multivector MultivectorAddScaled10(multivector a, multivector b, Real bmult)
{ // a = a + bmult*b
	if(b==MV_ZERO) return(a);
ASSERT2(CheckValidMV(a));
ASSERT2(CheckValidMV(b));
	multivector r=MultivectorAddScaled1(NULL, a, b, bmult);
	KillMultivector(&a);
	return(r);
}


multivector MultivectorAddScaled1(multivector tgt, multivector a, multivector b, Real bmult)
{ // tgt = a + bmult*b
ASSERT2(CheckValidMV(a));
	if(b==MV_ZERO) return(CopyMultivector(tgt,a));
ASSERT2(CheckValidMV(b));

REPORT2(MV_NEWS,5,("\n MultivectorAddScaled1 %M + %r*%M ",a, bmult,b));

ASSERT(tgt!=a);
ASSERT(tgt!=b);

#if MV_VSR_LEVEL
	a=DeFactorise(a);
	b=DeFactorise(b);
#endif

STRACE1
	if(a->h.level == b->h.level)
	{	// a and b same level
STRACE1
		bladesflag bftgt = (bladesflag)(a->h.present | b->h.present);
		mvcapacity cap = BitsSetBf(bftgt);
		tgt = AllocateSpace(tgt,a->h.level,bftgt);
//		EnsureCapacity(tgt, a->h.level, cap, MV_DIM0);
ASSERT(tgt!=NULL);
		tgt->h.present = bftgt;
		bladesflag bit =1;
		int cnttgt=0;		int cnta=0;		int cntb=0;
fint safety =300;
		if(a->h.level==0)	// a and b both zero level
		{	while((bit!=0) && (bit <= bftgt))
			{	if(bit & bftgt)
				{	if( a->h.present & bit)
					{	Real c = ((multivector0)a)->r[cnta++];
	if(cnttgt >= tgt->h.capacity+1) ERRORCN(("cnttgt=%X >= capacity %X ",cnttgt,tgt->h.capacity+1));
						if( b->h.present & bit)	c += rrMUL(bmult ,((multivector0)b)->r[cntb++]);	
						if(EQUIVALENT_REALS(c,Real_0)) tgt->h.present ^= bit;
						else ((multivector0)tgt)->r[cnttgt++] = c;
					}
					else ((multivector0)tgt)->r[cnttgt++] = rrMUL(bmult , ((multivector0)b)->r[cntb++]);	
				}
				bit<<=1;
ASSERT(--safety > 0);
			}	// endwhile
STRACE1
		}
		else // a and b have same nonzero level
		{
			while((bit!=0) && (bit <= bftgt))
			{	if(bit & bftgt)
				{	if( a->h.present & bit)
					{	if(b->h.present & bit)
							tgt->m[cnttgt] = MultivectorAddScaled1(tgt->m[cnttgt] , 	a->m[cnta++], b->m[cntb++], bmult);
						else tgt->m[cnttgt] = CopyMultivector(tgt->m[cnttgt], a->m[cnta++]);	
					}
					else
					{	tgt->m[cnttgt] = CopyMultivectorScaled(tgt->m[cnttgt], b->m[cntb++], bmult);	
					}
					cnttgt++;
				}
				bit<<=1;
	ASSERT(--safety > 0);
			}	// endwhile
		}
STRACE1
 	
 	}	
 	else
 	{	// a and b different levels
		if(a->h.level < b->h.level)
 		{ 	// b is higher level
REPORT2(MV_NEWS,5,("\n MultivectorAddScaled1 %M levl %X < %M level %X ", a,a->h.level, b, b->h.level));
		 	bladesflag bftgt= (bladesflag)(b->h.present | 1);
//		tgt= AllocateSpace(tgt,b->h.level,bftgt);
			mvcapacity used = BitsSetBf(bftgt);
	 		tgt = EnsureCapacity2(tgt, b->h.level, used, MV_DIM0);
ASSERT(tgt->h.level==b->h.level);
			int bcnt=(b->h.present & 1);
#if MV_NEWS>2
if(bcnt) REPORT2(MV_NEWS,5,("\n adding %r*%M to %M ", bmult, b->m[0], a))
else REPORT2(MV_NEWS,5,("\n copy %M to sclar slot %M ", a,tgt->m[0]))
#endif
			if(bcnt) tgt->m[0] = MultivectorAddScaled1(tgt->m[0], a, b->m[0], bmult);
			else tgt->m[0] = CopyMultivector(tgt->m[0], a);
REPORT2(MV_NEWS,5,("\n resulting scalar slot is %M ", tgt->m[0]))
			for(fint i=1;i<used;i++) tgt->m[i] = CopyMultivectorScaled(tgt->m[i], b->m[bcnt++], bmult);
			tgt->h.present = bftgt;
ASSERT2(CheckValidMV(tgt));
REPORT2(MV_NEWS,5,("\n MultivectorAddScaled1 res=%M ", tgt))
ASSERT(tgt->h.level==b->h.level);
 		}
		else
	 	{	// a is higher level
 			bladesflag bftgt= (bladesflag)(a->h.present | 1);
			mvcapacity cap = BitsSetBf(bftgt);
			tgt = EnsureCapacity2(tgt, a->h.level, cap, MV_DIM0);
			int acnt=(a->h.present & 1);
			if(acnt) tgt->m[0] = MultivectorAddScaled1(tgt->m[0], a->m[0], b, bmult);
			else tgt->m[0] = CopyMultivectorScaled(tgt->m[0], b, bmult);
			for(fint i=1;i<cap;i++) tgt->m[i] = CopyMultivector(tgt->m[i], a->m[acnt++]);
			tgt->h.present = bftgt;
	 	}
	}

#if MV_VSR_LEVEL
	KillMultivector(&a);
	KillMultivector(&b);
#endif
STRACE1
	return(tgt);
}



multivector MultivectorAddScaled(multivector tgt, multivector a, multivector b, Real amult, Real bmult)
{ // tgt = amult*a + bmult*b
	REPORT2(MV_NEWS,5,("\n MultivectorAdd %r*%M + %r*%M tgt=%M",amult, a, bmult,b, tgt));
	if(b==MV_ZERO) return(CopyMultivectorScaled(tgt,a,amult));
	if(a==MV_ZERO) return(CopyMultivectorScaled(tgt,b,bmult));

	if(tgt==a)	{	tgt = MultivectorAddScaled(NULL,a,b,amult,bmult);	KillMultivector(&a); return(tgt);	}
	if(tgt==b)  {	tgt = MultivectorAddScaled(NULL,a,b,amult,bmult);	KillMultivector(&b); return(tgt);	}

ASSERT(tgt!=a);
ASSERT(tgt!=b);

#if MV_VSR_LEVEL
	a=DeFactorise(a);
	b=DeFactorise(b);
#endif


REPORT2(MV_NEWS,5,("\n adding %r*%M + %r*%M tgt=%M",amult, a, bmult,b, tgt));



	if(a->h.level == b->h.level)
	{	// a and b same level
		bladesflag bftgt = (bladesflag)(a->h.present | b->h.present);
		mvcapacity cap = BitsSetBf(bftgt);
		tgt = EnsureCapacity2(tgt, a->h.level, cap, MV_DIM0);
ASSERT(tgt!=NULL);
		tgt->h.present = bftgt;
ASSERT(tgt->h.capacity+1>=cap);
		bladesflag bit =1;
		int cnttgt=0;	int cnta=0;	int cntb=0;
fint safety =300;
		if(a->h.level==0)	// a and b both zero level
		{
	REPORT2(MV_NEWS,5,("\n Adding zero level multivectors %M + %M ",a,b));
		
			while((bit!=0) && (bit <= bftgt))
			{	if(bit & bftgt)
				{	if( a->h.present & bit)
					{	Real c = rrMUL(amult,((multivector0)a)->r[cnta]); cnta++;
	if(cnttgt >= tgt->h.capacity+1) ERRORCN(("cnttgt=%X >= capacity %X bftgt=%X cap=%X tgt=%p a=%p b=%p",cnttgt,tgt->h.capacity+1,bftgt,cap,tgt,a,b));
						if( b->h.present & bit)	c += rrMUL(bmult ,((multivector0)b)->r[cntb++]);	
						if(EQUIVALENT_REALS(c,Real_0)) tgt->h.present ^= bit;
						else ((multivector0)tgt)->r[cnttgt++] = c;
					}
					else ((multivector0)tgt)->r[cnttgt++] = rrMUL(bmult , ((multivector0)b)->r[cntb++]);	
				}
				bit<<=1;
ASSERT(--safety > 0);
			}	// endwhile
	REPORT2(MV_NEWS,5,("\n Added %M + %M =%M ",a,b,tgt));
		}
		else
		{	// a and b have same nonzero level
			while((bit!=0) && (bit <= bftgt))
			{	if(bit & bftgt)
				{	if( a->h.present & bit)
					{	if(b->h.present & bit)
						tgt->m[cnttgt] = MultivectorAddScaled(tgt->m[cnttgt] , 	a->m[cnta++], b->m[cntb++], amult,bmult);
						else tgt->m[cnttgt] = CopyMultivectorScaled(tgt->m[cnttgt], a->m[cnta++],amult);	
					}
					else tgt->m[cnttgt] = CopyMultivectorScaled(tgt->m[cnttgt], b->m[cntb++], bmult);	
					cnttgt++;
				}
				bit<<=1;
ASSERT(--safety > 0);
			}	// endwhile
		}
 	}
 	else
 	{ if(a->h.level < b->h.level)
	 	{ 	// b is higher level
 REPORT2(MV_NEWS,5,("\n b = %M higher level ",b));
			bladesflag bftgt = (bladesflag)(1 | b->h.present);
	 	 	mvcapacity cap = BitsSetBf(bftgt);
		 	tgt = EnsureCapacity2(tgt, b->h.level, cap, MV_DIM0);
	 		tgt->h.present = bftgt;

			int bcnt=(b->h.present & 1);
			if(bcnt) 	tgt->m[0] = MultivectorAddScaled(tgt->m[0], a, b->m[0], amult,bmult);
			else 			tgt->m[0] = CopyMultivectorScaled(tgt->m[0], a,amult);

			for(fint i=1;i<cap;i++) tgt->m[i] = CopyMultivectorScaled(tgt->m[i], b->m[bcnt++], bmult);
	 	}
	 	else
	 	{	// a is higher level
	 		bladesflag bftgt= (bladesflag)(a->h.present | 1);
	 	 	mvcapacity cap = BitsSetBf(bftgt);
		 	tgt = EnsureCapacity2(tgt, a->h.level, cap, MV_DIM0);
	 		tgt->h.present = bftgt;

			int acnt=(a->h.present & 1);
			if(acnt) tgt->m[0] = MultivectorAddScaled(tgt->m[0], a->m[0], b, amult,bmult);
			else tgt->m[0] = CopyMultivectorScaled(tgt->m[0], b, bmult);
			for(fint i=1;i<cap;i++) tgt->m[i] = CopyMultivectorScaled(tgt->m[i], a->m[acnt++], amult);
	 	}
	}
#if MV_VSR_LEVEL
	KillMultivector(&a);
	KillMultivector(&b);
#endif
	return(tgt);
}


// -----------------------------------------
//      Multivector Products
// -----------------------------------------

bladeindex BladeBladeProduct(bladeindex bi,bladeindex bj,mvproduct * ppro,mvlevel level,bladesflag inhibit)
{	// Mutiply (logical) two same-level blades, returning index of resulting blade, or MV_INDEX_NIL if product vanishes .
	// Sets or clears MV_LEFT_NEGATION bit of pro according to sign of blade product (leaves extant if product vanishes).
	// level is required to access correct signatures - unneccesary if Eucluidean.
	// inhibit forces particular results to MV_INDEX_NIL.
	// Works for commutative and anticommuatative products. 

	bladeindex br = bi^bj;	 // exclusive orr gives product blade index 

	// Restrictive product filters
	if( (*ppro) & (MV_ONTO_CONTRACTIVE_PRODUCT|MV_BY_CONTRACTIVE_PRODUCT|MV_INNER_PRODUCT|MV_OUTTER_PRODUCT) )
	{	if (((*ppro) & MV_OUTTER_PRODUCT)  && ((bi & bj)!=0) ) return(MV_INDEX_NIL);
		bladeindex bri = br & bi;
		bladeindex brj = br & bj;
		if(   (((*ppro) & MV_ONTO_CONTRACTIVE_PRODUCT)&& bri )
			 || (((*ppro) & MV_BY_CONTRACTIVE_PRODUCT)  && brj )
			 || (((*ppro) & MV_INNER_PRODUCT) 					&& bri && brj )
			) return(MV_INDEX_NIL);	 
	}

	// Commutation product filters
	if( 	(((*ppro) & MV_COMMUTATOR_PRODUCT) 	 		&& (!AntiCommutingBlades(bi,bj)) )
		 || (((*ppro) & MV_ANTICOMMUTATOR_PRODUCT) 	&& ( AntiCommutingBlades(bi,bj)) )
	 	)  return(MV_INDEX_NIL);	  

	if( inhibit && ((1<<br)&inhibit))	return(MV_INDEX_NIL);	 // vanishes due to inhibition 

	// index established, now have to determine sign and spot zero if null signature involved  
	boolean sign=0;
	bladesflag	bit = 1<<((1<<MV_DIM0)-1);									// start with highest axis bit

	boolean bjparity = BitParity(bj) ;											// parity of nonzero bj bits below and including current bit
	do
	{	if(bi & bit)
		{	if(bj & bit) 																					// 1-blade present in both so signature contributes
			{	if (!((*ppro) & MV_EUCLIDEAN_PRODUCT))							// Check signatures enabled
				{	if(mv_nullsigs[level] & bit) return(MV_INDEX_NIL);// vanishes due to null signature 
					if(mv_negsigs[level] & bit) sign ^= 1;						// accrue sign flip due to negative signature
				}
				bjparity ^= 1;																			// Maintain parity counter
			}
 			sign ^= bjparity ;																		// Commute bi axis across all lower axies present in bj
		}
		else if(bj & bit) bjparity ^= 1;
		bit>>=1;
	}	while(bit);
REPORT2(MV_NEWS,6,("\n ---BladeBladeProduct[%X]  %O*%O=%O sign=%X   ",	ppro[0], bi,level,bj,level, br,level, sign));

	// Now derive sign changes due to specified conjugations 	
	mvconjugation cnj = (*ppro) & MV_CONJUGATION_ALL;
if(cnj) REPORT2(MV_NEWS,7,("[LftPcnj %X to lft blade %O -> %X] ",cnj, bi, level, mv_conj[cnj][level] & (1<<bi)));
	if(cnj && (mv_conj[cnj][level] & (1<<bi))) sign ^= 1;	//  bi conjugation

	cnj = ((*ppro) >>MV_CONJUGATION_SHIFT) & MV_CONJUGATION_ALL;
if(cnj) REPORT2(MV_NEWS,7,("[RgtPcnj %X to rgt blade %O ->%X] ",cnj, bj, level, mv_conj[cnj][level] & (1<<bj)));
	if(cnj && (mv_conj[cnj][level] & (1<<bj))) sign ^=1;	// bj  conjgation
#if MV_LEFT_NEGATION==1
	*ppro = ((*ppro) & ~1) | sign;
#else
	if(sign) *ppro |= MV_LEFT_NEGATION;	else *ppro &= ~MV_LEFT_NEGATION;
#endif
	return(br);
}



multivector0 MultivectorProduct0(multivector0 r, multivector0 a, multivector0 b, mvproduct product,bladesflag inhibit)
{	// Fast MultivectorProduct assuming a and b both have level 0	(works for commutator products too)
STRACEM2("MultivectorProduct0(",product)

#if MV_DEBUG>1
if(!CheckValidMV((multivector)a))
{  ERRORCN(("MultivectorProduct passed invalid multivector a=%p",a))
	 BACKTRACE("Hello world",123)
	 REPORT2(MV_NEWS,1,(" attempting to print"))
	 REPORT2(MV_NEWS,1,(" a=%M",a))

}
if(!CheckValidMV((multivector)b))
{  ERRORCN(("MultivectorProduct passed invalid multivector b=%p ",b))
	 BACKTRACE("Hello world",123)
	 REPORT2(MV_NEWS,1,(" attempting to print"))
	 REPORT2(MV_NEWS,1,(" b=%M",b))
}
if(CheckValidMV((multivector)r)>1)
{  ERRORCN(("MultivectorProduct passed invalid tgt multivector r=%p ",r))
	 BACKTRACE("Hello world",123)
//	 REPORT2(MV_NEWS,1,(" attempting to print"))
//	 REPORT2(MV_NEWS,1,(" r=%M",r))
}



#endif
STRACE2
ASSERT(a->h.level==0);
ASSERT(b->h.level==0);
	if((a==MV_ZERO)||(b==MV_ZERO))
	{	if(r!=MV_ZERO) // rapid zeroing
		{	 r->h.present=MV_FLAG_NONE;

#if MV_DEBUG
{ int dd=DiagnoseMV((multivector)r); if(dd>1) ERRORCN(("MultivectorProduct0 rapidzero result invlaide[%X]",dd))}
#endif
		}
		 return(r);
	}	// endif rapidzero 

ASSERT(mv_reg[0][0]!=NULL);

 	ZeroMultivector(mv_reg[0][0],0);																						// clear mv register 

ASSERT(mv_reg[0][0]!=NULL);

 	bladeindex inda,cnta,cntr; 	inda=cnta=cntr=0; 	bladesflag bfa=a->h.present;// prepare for outter loop
REPORT2(MV_NEWS,5,("\n -- Multiply00[%X] (%M) * (%M) ",product, a,b));
STRACE2

	while(bfa)
	{	SHIFT_RGT_CNT(shf,bfa,inda)
//	uint32 shf=BSF_32(bfa);	bfa >>= shf; bfa>>=1;inda += (bladeindex)shf;			// fast low->high scan for nonzero bit
		bladeindex indb=0;	bladeindex cntb=0;	bladesflag bfb=b->h.present;			// prepare for inner loop
 		while(bfb)
		{	 SHIFT_RGT_CNT(shfb,bfb,indb)
		//uint32 shfb=BSF_32(bfb);bfb >>= shfb;	bfb>>=1;	indb += (bladeindex)shfb;
if(cntb > b->h.capacity) REPORT2(MV_NEWS,1,("\n cntb=%X exceeds capacity %X prse=%X bfb=%X bfa=%X",cntb,b->h.capacity,b->h.present,bfb,bfa))
			mvproduct pro = product;
			bladeindex ri = BladeBladeProduct(inda, indb, &pro, 0, inhibit);				// Do blade product first  
REPORT2(MV_NEWS,5,(" \n pro=%X ri=%X = %O for %O * %O  ",pro, ri,ri,0,inda,0, indb,0));
			if(ri!=MV_INDEX_NIL)																										// and check non vanishing.
			{	bladesflag rf=(bladesflag)((1<<ri));
ASSERT2(ValidReal(a->r[cnta]) && ValidReal(b->r[cntb]) );					// Debug catch bad float data 
				Real c=rrMUL(a->r[cnta], b->r[cntb]);											// multiply real coordinates 
REPORT2(MV_NEWS,5,(" \n pro=%X ri=%X = %O for %O * %O  ",pro, ri,ri,0,inda,0, indb,0));
REPORT2(MV_NEWS,5,(" coeff=%r*%r=%r [%X,%X] prosign=%X ",a->r[cnta], b->r[cntb],c,cnta,cntb,pro & MV_LEFT_NEGATION));
				if(!EQUIVALENT_REALS(c,Real_0))
				{	if(pro & MV_LEFT_NEGATION) c=-c;  												// negate if BladaBladeProduct flipped NEGATION bit 
ASSERT2(CheckValidMV(mv_reg[0][0]));
					if(	mv_reg[0][0]->h.present & rf)	((multivector0)mv_reg[0][0])->r[ri] += c;
					else	((multivector0)mv_reg[0][0])->r[ri] = c;
					mv_reg[0][0]->h.present |= rf;
				}
			}	// endif ri
			indb++;	cntb++;
STRACE4
		} // endwhile bfb
		inda++;	cnta++;
STRACE4

	}	// endwhile bfa
STRACE2
ASSERT(r!=((multivector0)mv_reg[0][0]));
STRACEM2("...Condense(",2212)
	r=Condense0(r,(multivector0)(mv_reg[0][0]));	
STRACEM2("...Condense)",2213)
#if MV_DEBUG
{ int dd=DiagnoseMV((multivector)r); if(dd>1) ERRORCN(("MultivectorProduct0 result invlaide[%X]",dd))}
#endif
REPORT2(MV_NEWS,4,(" -- MultivectorProduct0 (%M)[%X] * (%M)[%X] = (%M) ",a,product & MV_CONJUGATION_ALL, b,(product>>MV_CONJUGATION_SHIFT)& MV_CONJUGATION_ALL, r));
STRACEM2("exit MultivectorProduct0(",product)
	return(r);	
}




#if MV_VSR_LEVEL
multivector FactoredProduct(multivector a, multivector b,mvconjugation acnj,mvconjugation bcnj,mvcapacity margin,mvlevel lev)
{	multivector r;
	if(a==MV_ZERO) return(MV_ZERO);
	if(b==MV_ZERO) return(MV_ZERO);

	if(a->h.level==lev)
	{	int spc;
		if(b->h.level==lev)
		{	spc=a->h.present + b->h.present;
			if(spc<MV_MAX_CAPACITY)
			{	r=NewMultivector((spc+margin),NULL,lev);
				ConjugatedCopies(&(r->m[0]),&(a->m[0]),a->h.present,acnj); 
				ConjugatedCopies(&(r->m[a->h.present]),&(b->m[0]),b->h.present,bcnj);
				r->h.present=spc;
	 			return(r);
			}
		}	
		else
		{	spc=a->h.present + 1;
			if(spc<MV_MAX_CAPACITY)
			{	r=NewMultivector((spc+margin),NULL,lev);
				ConjugatedCopies(&(r->m[0]),&(a->m[0]),a->h.present,acnj);
				r->m[a->h.present]=MultivectorConjugation(b,bcnj);
				r->h.present=spc;
				return(r);
			}
		}
	}
	else if(b->h.level==lev)
	{	int spc=b->h.present + 1;
		if(spc<MV_MAX_CAPACITY)
		{	r=NewMultivector((spc+margin),NULL,lev);
			r->m[0]=MultivectorConjugation(a,acnj);
			ConjugatedCopies(&(r->m[1]),&(b->m[0]),b->h.present,bcnj) ;
			r->h.present=spc;
			return(r);
		}
	}
	r=NewMultivector(2+margin,NULL,lev);
	r->m[0]=MultivectorConjugation(a,acnj);
	r->m[1]=MultivectorConjugation(b,bcnj);;
	r->h.present=2;
	return(r);
}

multivector GeometricFactoredProduct(multivector a, multivector b,mvconjugation acnj,mvconjugation bcnj,mvcapacity margin)
{	return(FactoredProduct(a,b,acnj,bcnj,margin,MV_VSR_LEVEL));}

multivector OutterFactoredProduct(multivector a, multivector b,mvconjugation acnj,mvconjugation bcnj,mvcapacity margin)
{	return(FactoredProduct(a,b,acnj,bcnj,margin,MV_OTR_LEVEL));}

#endif

#if MV_NEWS
void ShowExpandedMV(multivector a)
{	int bit=1;

	for(int cnt=0;cnt<(1<<MV_DIM0);cnt++)
	{	if(bit & a->h.present)
		{	if(a->h.level)
			{	REP2(("+(%M)",a->m[cnt]));
			}
			else
			{	REP2((" %r",((multivector0)a)->r[cnt]));
			}
			REP2(("%O",cnt,a->h.level));
		}
		bit<<=1;
	}
}
#endif


inline multivector SetNonCompacted(multivector a,mvlevel lev) 
{	a=EnsureCapacity(a, lev, MV_MAX_CAPACITY);	a->h.noncompacted=1;return(a);
}



multivector MultivectorProduct(multivector r, multivector a, multivector b, mvproduct product,bladesflag inhibit)
{	// return a*b where * is product specified by product	(does not work for commutator products)
STRACEM2("MultivectorProduct(",product) 

#if MV_DEBUG>1
if(DiagnoseMV(a)>1)
{  ERRORCN(("MultivectorProduct passed invalid multivector a=%p",a))
	 BACKTRACE("Hello world",123)
	 REPORT2(MV_NEWS,1,(" attempting to print"))
	 REPORT2(MV_NEWS,1,(" a=%M",a))

}
int db=DiagnoseMV(b);
if(db>1)
{  ERRORCN(("MultivectorProduct passed invalid[%X] multivector b=%p ",db,b))
	 BACKTRACE("Hello world",123)
	 REPORT2(MV_NEWS,1,("\n a was %M",a))
	 REPORT2(MV_NEWS,1,("\n Attempting to print"))
	 REPORT2(MV_NEWS,1,(" b=%M",b))

}
#endif



REPORT2(MV_NEWS,4,("\n MultivectorProduct (%M)[%X] ", a, product & MV_CONJUGATION_ALL ));
REPORT2(MV_NEWS,4,(">%X< ", product>>(2*MV_CONJUGATION_SHIFT) ));
REPORT2(MV_NEWS,4,("(%M)[%X] ",  b,(product>>MV_CONJUGATION_SHIFT)&MV_CONJUGATION_ALL));
//REPORT2(MV_NEWS,4,("(%M)[%X] ihbt=%X r=%p",  b,(product>>MV_CONJUGATION_SHIFT)&MV_CONJUGATION_ALL, inhibit,r));
//REPORT2(MV_NEWS,5,("\n MultivectorProduct (%M)[%X] >%X< (%M)[%X] inhibit=%X r=%p", a, product & MV_CONJUGATION_ALL,  product>>(2*MV_CONJUGATION_SHIFT), b,(product>>MV_CONJUGATION_SHIFT)&MV_CONJUGATION_ALL, inhibit,r));
	if((a==MV_ZERO)||(b==MV_ZERO)) {	if(r!=MV_ZERO) r->h.present=MV_FLAG_NONE; return(r);	}	 // rapid zero case

STRACE3
	if(r==a)	{	r = MultivectorProduct(NULL,a,b,product,inhibit);	KillMultivector(&a);
#if MV_DEBUG>1
{ int dd=DiagnoseMV(r); if(dd>1) ERRORCN(("MultivectorProduct returning invalid[%X] result",dd))}
#endif
	
	 return(r);	}
	if(r==b)  {	r = MultivectorProduct(NULL,a,b,product,inhibit);	KillMultivector(&b); 
#if MV_DEBUG>1
{ int dd=DiagnoseMV(r); if(dd>1) ERRORCN(("MultivectorProduct returning invalid[%X] result",dd))}
#endif
	
	return(r);	}

#if MV_VSR_LEVEL
	if(a->h.level>=MV_VSR_LEVEL)
	{
		//ERROREX(("Not expecting level %X >= %X versor a=%M",a->h.level, MV_VSR_LEVEL, a))
		mvproduct pro = product >> MV_PRODUCT_SHIFT;
		if( ((pro==MV_GEOMETRIC_PRODUCT) && (a->h.level==MV_VSR_LEVEL))
	 		||((pro==MV_OUTTER_PRODUCT) && (a->h.level==MV_OTR_LEVEL)) )
		{
			int used  = a->h.present-1;
ASSERT(used>=0);
			if(product & MV_LEFT_REVERSE)
			{
				r=MultivectorProduct(r,a->m[0], b, product,inhibit);
				product &= ~MV_LEFT_NEGATION;
				for(int i=1;i<=used;i++) r=MultivectorProduct(r,a->m[i], r, product,inhibit);
			}
			else
			{	
				r=MultivectorProduct(r,a->m[used], b, product,inhibit);
				product &= ~MV_LEFT_NEGATION;
				while((--used)>=0) r=MultivectorProduct(r, a->m[used], r, product,inhibit);
			}

		}
		else
		{
			multivector aa =DeFactorise(a);	
			r = MultivectorProduct(r,aa,b,product,inhibit);	KillMultivector(&aa);
		}
#if MV_DEBUG>1
{ int dd=DiagnoseMV(r); if(dd>1) ERRORCN(("MultivectorProduct returning invalid[%X] result",dd))}
#endif
		return(r);
	}

	if(b->h.level>=MV_VSR_LEVEL)
	{	
ERRORCN(("Not expecting Versors"))	
	
		mvproduct pro = product >> MV_PRODUCT_SHIFT;
		if( ((pro==MV_GEOMETRIC_PRODUCT) && (b->h.level==MV_VSR_LEVEL))
	 		||((pro==MV_OUTTER_PRODUCT) && (b->h.level==MV_OTR_LEVEL)) )

		{	int used  = b->h.present-1;
ASSERT(used>=0);
			if(product & MV_RIGHT_REVERSE)
			{	r=MultivectorProduct(r,a, b->m[used], product,inhibit);
				product &= ~MV_LEFT_NEGATION;
				while((--used)>=0) r=MultivectorProduct(r,r,b->m[used], product,inhibit);
			}
			else
			{	r=MultivectorProduct(r,a, b->m[0], product,inhibit);
				product &= ~MV_LEFT_NEGATION;
				for(int i=1;i<=used;i++) r=MultivectorProduct(r,r,b->m[i], product,inhibit);
			}
		}
		else
		{	multivector bb =DeFactorise(b);		r = MultivectorProduct(r,a,bb,product,inhibit);		KillMultivector(&bb);
		}
		return(r);
	}
#endif

	if(a->h.level >= b->h.level)
	{	if(a->h.level > b->h.level)
		{	// --------------------------------------------------
			//   can subsume b into a's multivector coefficients 
			// --------------------------------------------------
REPORT2(MV_NEWS,5,("\n subsuming b=%M into a=%M's coeffs ", b,a));
			bladeindex inda=0; bladeindex cnta=0;	bladeindex cntr=0;
 			bladesflag bfa=a->h.present;
			ZeroMultivector(r,a->h.level);
			r = EnsureCapacity2(r, a->h.level, BitsSetBf(a->h.present), MV_DIM0);
	 		while(bfa)
			{	SHIFT_RGT_CNT(shf,bfa,inda)
		 //	uint32 shf=BSF_32(bfa);	bfa >>= shf; bfa>>=1; 	inda += (bladeindex)shf;	// fast scan and move to nonzero bit

REPORT2(MV_NEWS,4,("\n \n  MVP a=%M *%O subsume %M ",a, inda,a->h.level, b))

				if((!(product & MV_ONTO_CONTRACTIVE_PRODUCT)) || (inda==0) )
				{	mvproduct pro=product;
					if(OddBlade(inda))
					{ pro ^= MV_RIGHT_INVOLUTION;
						if(product & MV_LEFT_REVERSE) pro ^= MV_LEFT_INVOLUTION;
					}
 			
if(!CheckValidMV(a->m[cnta]))
{  ERRORCN(("MultivectorProduct to be possed invlid multivector a->m[%X]=%p",cnta,a->m[cnta]))
	 BACKTRACE("Hello world",123)

}
 			
 					r->m[cntr] = MultivectorProduct(r->m[cntr], a->m[cnta], b, pro, inhibit);
					if(r->m[cntr]!=MV_ZERO) { r->h.present |= (1<<inda);cntr++;}
				}
				cnta++;	inda++;
			}	// endwhile
	REPORT2(MV_NEWS,4,("\nMvProda[%X]%M*%M= %M",product, a,b, r));

#if MV_DEBUG>1
{ int dd=DiagnoseMV(r); if(dd>1) ERRORCN(("MultivectorProduct returning invalid[%X] result",dd))}
#endif


			return(r);
		} // endif a->lev > b->lev
	 	
		if(a->h.level==0)
		{
r=(multivector)MultivectorProduct0((multivector0)r,(multivector0)a,(multivector0)b,product,inhibit);		 // Rapid both 0 level case
STRACEM2("exit MultivectorProduct 00 case",product)

#if MV_DEBUG>1
{ int dd=DiagnoseMV(r); if(dd>1) ERRORCN(("MultivectorProduct returning invalid[%X] result",dd))}
#endif

return(r);
//			 return((multivector)MultivectorProduct0((multivector0)r,(multivector0)a,(multivector0)b,product,inhibit));		 // Rapid both 0 level case
		}
		// --------------------------------------------------------------------------------
		//   a->h.level = b->h.level > 0		
		//   Bilinearly expand product into uncompressed multivector in mv_reg[a->h.level][0]
		// ------------------------------------------------------------------------------------
	 	ZeroMultivector(mv_reg[a->h.level][0],a->h.level);
		mv_reg[a->h.level][0] = SetNonCompacted(mv_reg[a->h.level][0],a->h.level);
		
		bladeindex inda=0; bladeindex cnta=0;	bladeindex cntr=0;
 		bladesflag bfa=a->h.present;
		while(bfa)
		{	SHIFT_RGT_CNT(shf,bfa,inda)
//		uint32 shf=BSF_32(bfa);	bfa >>= shf; bfa>>=1;	inda += (bladeindex)shf;	// fast scan and move to nonzero a bit
			boolean odda = OddBlade(inda);
			bladeindex indb=0;	bladeindex cntb=0;		bladesflag bfb=b->h.present;

REPORT2(MV_NEWS,4,("\n \n   MvPp a[%X] comp (%M)*%O  ",a->h.level, a->m[cnta], inda, a->h.level))


  		while(bfb)
			{	SHIFT_RGT_CNT(shf,bfb,indb)
//				uint32 shf=BSF_32(bfb);			bfb >>= shf; bfb>>=1;			indb += (bladeindex)shf;
//REPORT2(MV_NEWS,2,("\n subprod ((%M)* %O)[%X] * ((%M) * %O)[%X]",a->coeff[cnta], inda, a->h.level, product & MV_CONJUGATION_ALL, b->coeff[cntb], indb, b->h.level, (product>>MV_CONJUGATION_SHIFT) & MV_CONJUGATION_ALL));
				mvproduct pro = product;
				bladeindex ri = BladeBladeProduct(inda, indb, &pro, a->h.level, inhibit); // may toggle MV_LEFT_NEGATION bit in pro
REPORT2(MV_NEWS,6,("\n- return pro=%X ",pro))
				if(ri!=MV_INDEX_NIL)
				{	if(odda)
					{	pro ^= MV_RIGHT_INVOLUTION;	 	// need to commute b's coeff across a's blade  
REPORT2(MV_NEWS,6,("\n- a's %O odd so RgtIvl -> %X [%X]",inda,a->h.level,pro,MV_RIGHT_INVOLUTION))
						if(product&MV_LEFT_REVERSE) 
						{	pro ^= MV_LEFT_INVOLUTION; // and move a coeff accross a's blade if a reversed
REPORT2(MV_NEWS,6,("\n LftRev[%X] so commuting a coeff %M across a's %O -> LftIvl -> %X",product,a->m[cnta], inda, a->h.level,pro));
						}
					}
					if((product&MV_RIGHT_REVERSE)&& OddBlade(indb))
					{	pro ^= MV_RIGHT_INVOLUTION;	// and b's coeff across b's blade if b reversed
REPORT2(MV_NEWS,6,("\n RgtRev[%X] so commuting b coeff %M across b's %O -> RgtIvl -> %X",product, b->m[cntb], indb, b->h.level, pro));
					}
				 					
					bladesflag rf=(bladesflag)(1<<ri);
				 	if(	mv_reg[a->h.level][0]->h.present & rf)
				 	{	
#if MV_DEBUG>2
if(!CheckValidMV(a->m[cnta])){  ERRORCN(("MultivectorProduct passed invalid multivector a=%p",a->m[cnta])) BACKTRACE("Hello world",123) }
if(!CheckValidMV(b->m[cntb])){  ERRORCN(("MultivectorProduct passed invalid multivector b=%p ",b->m[cntb])) BACKTRACE("Hello world",123)}
#endif
				 		mv_reg[a->h.level-1][1] = MultivectorProduct( mv_reg[a->h.level-1][1], a->m[cnta] , b->m[cntb] , pro);
REPORT2(MV_NEWS,4,("\n- Add %M to %M in slot %O ",mv_reg[a->h.level-1][1], mv_reg[a->h.level][0]->m[ri] , ri, a->h.level))
				 		mv_reg[a->h.level][0]->m[ri] = MultivectorAdd(mv_reg[a->h.level][0]->m[ri], mv_reg[a->h.level-1][1]);
REPORT2(MV_NEWS,4,(" res is %M ",mv_reg[a->h.level][0]->m[ri] ))
				 	}
				 	else
				 	{	
#if MV_DEBUG>2
if(DiagnoseMV(a->m[cnta])>1) {  ERRORCN(("MultivectorProduct passed invalid multivector a=%p",a->m[cnta]))	 BACKTRACE("Hello world",123) }
if(DiagnoseMV(b->m[cntb])>1) {  ERRORCN(("MultivectorProduct passed invalid multivector b=%p ",b->m[cntb])) BACKTRACE("Hello world",123)}
#endif
				 	mv_reg[a->h.level][0]->m[ri] 
				 			= MultivectorProduct( mv_reg[a->h.level][0]->m[ri], a->m[cnta] , b->m[cntb] , pro);
				 		if(mv_reg[a->h.level][0]->m[ri] != MV_ZERO) mv_reg[a->h.level][0]->h.present |= rf;	


REPORT2(MV_NEWS,4,("\n Set %M in slot %O ", mv_reg[a->h.level][0]->m[ri] , ri, a->h.level))

				 	}
REPORT2(MV_NEWS,4,(" %O ",ri,a->h.level))
				}	// endif ri
				indb++;
				cntb++;
			} // endwhile bfb
REPORT2(MV_NEWS,4,("\n Tally[%X]=%2M ",cnta,mv_reg[a->h.level][0]))
			inda++;
			cnta++;
		}	// endwhile bfa
//	REPORT2(MV_NEWS,3,("\nReady to condense into  %M",r));
 		r=Condense(r,mv_reg[a->h.level][0]);
	REPORT2(MV_NEWS,4,("\nMvProdb[%X] (%M) * (%M) = %1M ",product, a, b, r));
	CHECKPOINT
#if MV_DEBUG>1
{ int dd=DiagnoseMV(r); if(dd>1) ERRORCN(("MultivectorProduct returning invalid[%X] result",dd))}
#endif
 		return(r);
 	}	// endif 	
 	{ // --------------------------------------------------------------------------
 		//   b->h.level > a->h.level so  subsume a into b's multivector coefficients 
		// --------------------------------------------------------------------------
REPORT2(MV_NEWS,5,("\n subsuming a=%M into b=%M's coeffs [b->pres=%X]", a,b,b->h.present));
		bladeindex indb=0;
		bladeindex cntb=0;
		bladeindex cntr=0;
 		bladesflag bfb=b->h.present;
		ZeroMultivector(r,b->h.level);
		r = EnsureCapacity(r, b->h.level, BitsSetBf(b->h.present));


ASSERT1(r->h.noncompacted==0);
	 	
 		while(bfb)
		{	SHIFT_RGT_CNT(shf,bfb,indb)
//			uint32 shf=BSF_32(bfb);			bfb >>= shf; bfb>>=1;		indb += (bladeindex)shf;

REPORT2(MV_NEWS,4,("\n \n   MVP b comp %X is blade %O * %M ",cntb, indb, b->h.level, b->m[cntb]))
	
			if((!(product & MV_BY_CONTRACTIVE_PRODUCT)) || (indb==0) )
			{	mvproduct pro = product;
				if( (pro & MV_RIGHT_REVERSE) && OddBlade(indb) ) pro ^= MV_RIGHT_INVOLUTION;
				r->m[cntr] = MultivectorProduct(r->m[cntr], a, b->m[cntb], pro, inhibit);
REPORT2(MV_NEWS,4,("\n \n   MVP b comp %X result is %M into slot %X",cntb, r->m[cntr], cntr))
ASSERT2(cntr<=r->h.capacity);

				if(r->m[cntr]!= MV_ZERO)	{	 r->h.present |= (1<<indb); cntr++;	}

REPORT2(MV_NEWS,4,("\n \n   next MVP b comp"))
			}
			cntb++;indb++;
		}	// endwhile
#if MV_DEBUG>1
{ int dd=DiagnoseMV(r);if(dd>1){  ERRORCN(("MultivectorProduct returning invalid[%X] multivector a=%p",dd,r))}}
#endif

	REPORT2(MV_NEWS,3,("\nMvProdc[%X] %M * %M = %M",product, a,b, r));
		return(r);
	}
}

#define MV_CNJ_NOTNEG (MV_CONJUGATION_ALL ^ MV_CONJUGATION_NEGATION)
mvproduct SwapConj(mvproduct pro) // swap left and right conjugation fields	excluiding negation bits
{	mvproduct r = ((pro & MV_CNJ_NOTNEG) << MV_CONJUGATION_SHIFT)
							 | ((pro & (MV_CNJ_NOTNEG << MV_CONJUGATION_SHIFT))>>MV_CONJUGATION_SHIFT)
							 | (pro & (~((MV_CNJ_NOTNEG << MV_CONJUGATION_SHIFT) | MV_CNJ_NOTNEG))) ;
	return(r);
}

#if MV_DEBUG
int CheckMVcode(void)
{ int d=DiagnoseMV(mv_reg[0][0]);
return(d);
	if(d) return(d<<16);
	return(DiagnoseMV(mv_reg[1][0]->m[0]));
}
#endif

multivector MultivectorCommutation(multivector r, multivector a, multivector b, mvproduct product,bladesflag inhibit)
{	// return a*b where * is product specified by product	(does not work for commutator products)
CHECKPOINT
STRACE1
REPORT2(MV_NEWS,4,("\n MultivectorCommutation (%M)[%X] \n >%X< (%M)[%X] inhibit=%X r=%p", a, product & MV_CONJUGATION_ALL,  product>>(2*MV_CONJUGATION_SHIFT), b,(product>>MV_CONJUGATION_SHIFT)&MV_CONJUGATION_ALL, inhibit,r));

#if MV_DEBUG
 { int d=DiagnoseMV(mv_reg[1][0]->m[0]); if (d>1) ERRORCN(("a=%p coeff 0 he fails d=%X",a,d));}
#endif
	if((a==MV_ZERO)||(b==MV_ZERO)) {	if(r!=MV_ZERO) r->h.present=MV_FLAG_NONE; return(r);	}						 // rapid zero case
	if(r==a)	{	r = MultivectorCommutation(NULL,a,b,product,inhibit);	KillMultivector(&a); return(r);	}	 // self action cases
	if(r==b)  {	r = MultivectorCommutation(NULL,a,b,product,inhibit);	KillMultivector(&b); return(r);	}

	if(a->h.level >= b->h.level)
	{	if(a->h.level > b->h.level)
		{	// --------------------------------------------------
			//   can subsume b into a's multivector coefficients 
			// --------------------------------------------------
REPORT2(MV_NEWS,4,("\n subsuming b=%M into a=%M's coeffs ", b,a));
			bladeindex inda=0; bladeindex cnta=0;	bladeindex cntr=0;
 			bladesflag bfa=a->h.present;
			ZeroMultivector(r,a->h.level);
			r = EnsureCapacity2(r, a->h.level, BitsSetBf(a->h.present), MV_DIM0);
	 		while(bfa)
			{	SHIFT_RGT_CNT(shf,bfa,inda)
//			uint32 shf=BSF_32(bfa);	bfa >>= shf; bfa>>=1; 	inda += (bladeindex)shf;
	 			if(OddBlade(inda))
				{	mvproduct pro=product & (~(MV_COMMUTATOR_PRODUCT | MV_ANTICOMMUTATOR_PRODUCT));
					if(product & MV_LEFT_REVERSE) pro ^= MV_LEFT_INVOLUTION;
STRACE3
 					mv_reg[a->h.level][0] = MultivectorProduct(mv_reg[a->h.level][0], a->m[cnta], b, pro ^ MV_RIGHT_INVOLUTION, inhibit);
 					pro = SwapConj(pro); if(product & MV_COMMUTATOR_PRODUCT) pro ^= MV_LEFT_NEGATION;
STRACE3
 					mv_reg[a->h.level][1] = MultivectorProduct(mv_reg[a->h.level][1], b, a->m[cnta], pro, inhibit);
 					r->m[cntr] = MultivectorAddScaled(r->m[cntr],	mv_reg[a->h.level][0], mv_reg[a->h.level][1], Real_HALF, Real_HALF);
				}
 				else r->m[cntr] = MultivectorCommutation(r->m[cntr], a->m[cnta], b, product, inhibit);
				if(r->m[cntr]!=MV_ZERO) { r->h.present |= (1<<inda);cntr++;}
				cnta++;	inda++;
			}	// endwhile
			return(r);
		} // endif
	 	
		if(a->h.level==0)	 return( (multivector)MultivectorProduct0((multivector0)r,(multivector0)a,(multivector0)b,product,inhibit));	 // both a and b level 0 so can use generic product
		
		// --------------------------------------------------------------------------------
		//   a->h.level = b->h.level > 0		
		//   Bilineraly expand product into uncompressed multivector in mv_reg[a->LEVEL][0]
		// ------------------------------------------------------------------------------------
	REPORT2(MV_NEWS,5,("\n Comm %M x %M at lev=%X", a,b, a->h.level));
	 	ZeroMultivector(mv_reg[a->h.level][0],a->h.level);
		mv_reg[a->h.level][0]=EnsureCapacity(mv_reg[a->h.level][0],a->h.level,MV_MAX_CAPACITY);
		bladeindex inda=0; bladeindex cnta=0;	bladeindex cntr=0;
 		bladesflag bfa=a->h.present;
		while(bfa)
		{	SHIFT_RGT_CNT(shf,bfa,inda)
//			uint32 shf=BSF_32(bfa);	bfa >>= shf; bfa>>=1;	inda += (bladeindex)shf;
			boolean odda = OddBlade(inda);
			bladeindex indb=0;	bladeindex cntb=0;		bladesflag bfb=b->h.present;
  		while(bfb)
			{	SHIFT_RGT_CNT(shf,bfb,indb)
//				uint32 shf=BSF_32(bfb);			bfb >>= shf; bfb>>=1;			indb += (bladeindex)shf;
REPORT2(MV_NEWS,4,("\n zubprod ((%M)*%O",a->m[cnta],inda,a->h.level));
REPORT2(MV_NEWS,4,("and again %O) ",inda,a->h.level ));
REPORT2(MV_NEWS,4,(" x ((%M) ", b->m[cntb] ));
REPORT2(MV_NEWS,4,("  * %O)", indb,b->h.level ));
REPORT2(MV_NEWS,4,(" with prodcut=%X inda=%X indb=%X",product,inda,indb));
				mvproduct pro0 = product & ~(MV_COMMUTATOR_PRODUCT | MV_ANTICOMMUTATOR_PRODUCT);
				mvproduct pro = pro0;
				bladeindex ri = BladeBladeProduct(inda, indb, &pro, a->h.level, inhibit); // may toggle MV_LEFT_NEGATION bit in pro
				if(ri!=MV_INDEX_NIL)
				{	mvproduct pro1 = product;
REPORT2(MV_NEWS,5,("\n .....coeff for blade %O ",ri,a->h.level, indb ));
					if(pro&MV_LEFT_NEGATION) pro1^=MV_LEFT_NEGATION;
					mvproduct pro2 = pro1;
REPORT2(MV_NEWS,6,("\n- return pro=%X ",pro))
					if(odda)
					{	pro1 ^= MV_RIGHT_INVOLUTION;	 	// need to commute b's coeff across a's blade  
REPORT2(MV_NEWS,6,("\n- a's %O odd so RgtIvl -> %X [%X]",inda,a->h.level,pro,MV_RIGHT_INVOLUTION))
						if(product&MV_LEFT_REVERSE) { pro1 ^= MV_LEFT_INVOLUTION; pro2 ^= MV_LEFT_INVOLUTION;}// and move a coeff accross a's blade if a reversed
					}
					if( OddBlade(indb))
					{	pro2 ^= MV_LEFT_INVOLUTION;	 
						if(product&MV_RIGHT_REVERSE)
						{	pro1 ^= MV_RIGHT_INVOLUTION; pro2 ^= MV_RIGHT_INVOLUTION;	// and b's coeff across b's blade if b reversed
REPORT2(MV_NEWS,6,("\n RgtRev[%X] so commuting b coeff %M across b's %O -> RgtIvl -> %X",product, b->m[cntb], indb, b->h.level, pro));
						}
					}
					bladesflag rf=(bladesflag)(1<<ri);
REPORT2(MV_NEWS,4,("\n pro1=%X pro2=%X",pro1,pro2));
					if(pro1==pro2)
					{	// Hooray. Can recurse as a commutation product..
						if(AntiCommutingBlades(inda,indb)) pro1 ^= (MV_COMMUTATOR_PRODUCT | MV_ANTICOMMUTATOR_PRODUCT) ;
						if(	mv_reg[a->h.level][0]->h.present & rf)
							 		mv_reg[a->h.level][0]->m[ri] 
				 					= MultivectorAddMerge(mv_reg[a->h.level][0]->m[ri], MultivectorCommutation( NULL, a->m[cnta] , b->m[cntb] , pro1, inhibit));
					 	else 	mv_reg[a->h.level][0]->m[ri] 
					 				= MultivectorCommutation( mv_reg[a->h.level][0]->m[ri], a->m[cnta] , b->m[cntb] , pro1,inhibit);
					}
					else
					{	// Bummer. Resorting to two seperate geoemetric products. 
						pro1 &= ~(MV_COMMUTATOR_PRODUCT | MV_ANTICOMMUTATOR_PRODUCT);
						pro2 &= ~(MV_COMMUTATOR_PRODUCT | MV_ANTICOMMUTATOR_PRODUCT);
				 	 
STRACE3
				 	 	multivector temp1 = MultivectorProduct( NULL, a->m[cnta] , b->m[cntb] , pro1);

REPORT2(MV_NEWS,4,("\n temp1 %M * %M [%X] = %M ",a->m[cnta] , b->m[cntb] , pro1,temp1));
						pro2 = SwapConj(pro2);
						if(product & MV_COMMUTATOR_PRODUCT) pro2 ^= MV_LEFT_NEGATION;
						if(AntiCommutingBlades(inda,indb))  pro2 ^= MV_LEFT_NEGATION;
STRACE3

			 			multivector temp2 = MultivectorProduct( NULL, b->m[cntb] , a->m[cnta] , pro2);
						temp1 = MultivectorAddMerge(temp1,temp2);	// clears temp2
						if(	mv_reg[a->h.level][0]->h.present & rf)
				 			mv_reg[a->h.level][0]->m[ri] = MultivectorAddMergeScaled(mv_reg[a->h.level][0]->m[ri], temp1,Real_HALF);
				 		else
				 		{	mv_reg[a->h.level][0]->m[ri] = CopyMultivectorScaled( mv_reg[a->h.level][0]->m[ri], temp1,Real_HALF);
							KillMultivector(&temp1);
						}
					}
REPORT2(MV_NEWS,4,("\n Commutation product sum = %M ",mv_reg[a->h.level][0]->m[ri]));
			 		if(mv_reg[a->h.level][0]->m[ri] != MV_ZERO) mv_reg[a->h.level][0]->h.present |= rf;	
					}	// endif ri
				indb++;
				cntb++;
			} // endwhile bfb
			inda++;
			cnta++;
		}	// endwhile bfa
	REPORT2(MV_NEWS,6,("\nReady to condense into  %M",r));
 		r=Condense(r,mv_reg[a->h.level][0]);
STRACE1
	REPORT2(MV_NEWS,4,("\nMvCommb[%X] %M * %M = %M",product, a, b, r));
 		return(r);
 	}	// endif 	

#if 1
#endif


STRACE1
 	{ // --------------------------------------------------------------------------
 		//   b->h.level > a->h.level so  subsume a into b's multivector coefficients 
		// --------------------------------------------------------------------------
REPORT2(MV_NEWS,4,("\n subsuming a=%M into b=%M's coeffs [b->pres=%X]", a,b,b->h.present));
		bladeindex indb=0; bladeindex cntb=0;	bladeindex cntr=0;
 		bladesflag bfb=b->h.present;
		ZeroMultivector(r,b->h.level);
		r = EnsureCapacity(r, b->h.level, BitsSetBf(b->h.present));
		while(bfb)
		{	SHIFT_RGT_CNT(shf,bfb,indb)
//		uint32 shf=BSF_32(bfb);	bfb >>= shf; bfb>>=1;	indb += (bladeindex)shf;
			if(OddBlade(indb))
			{	mvproduct pro=product  & (~(MV_COMMUTATOR_PRODUCT | MV_ANTICOMMUTATOR_PRODUCT));
				if(product & MV_RIGHT_REVERSE) pro ^= MV_RIGHT_INVOLUTION;
STRACE3
	
				mv_reg[a->h.level][0] = MultivectorProduct(mv_reg[a->h.level][0], a, b->m[cntb], pro, inhibit);
				pro = SwapConj(pro); if(product & MV_COMMUTATOR_PRODUCT) pro ^= MV_LEFT_NEGATION;
STRACE3
				mv_reg[a->h.level][1] = MultivectorProduct(mv_reg[a->h.level][1], b->m[cntb], a, pro^MV_RIGHT_INVOLUTION, inhibit);
				r->m[cntr] = MultivectorAddScaled(r->m[cntr],	mv_reg[a->h.level][0], mv_reg[a->h.level][1], Real_HALF, Real_HALF);
			}
 			else r->m[cntr] = MultivectorCommutation(r->m[cntr], a, b->m[cntb], product, inhibit);
	
			if(r->m[cntr]!= MV_ZERO) { r->h.present |= (1<<indb);cntr++;}
			cntb++;indb++;
		}	// endwhile
	REPORT2(MV_NEWS,5,("\nMvProdc[%X] %M * %M = %M",product, a,b, r));
		return(r);
	}
}	// end MultivectorCommutation

#if 1
#endif


multivector TrimMultivector(multivector mv)
{
REPORT2(MV_NEWS,2,("\n Trimming %M present=%X ",mv,mv->h.present))
	if(mv!=MV_ZERO)
	{
		while((mv->h.present==1)&&(mv->h.level>0))
		{	multivector mv2=mv;
			mv = mv->m[0];
			mv2->m[0]=MV_ZERO;
			KillMultivector(&mv2);
		}
	}
	return(mv);
}

Real ScalarPart(multivector a) // Faster than using general grade restriction call
{	if(a!=MV_ZERO)
	{
	
#if MV_VSR_LEVEL
		if(a->h.level==MV_OTR_LEVEL) return(Real_0);
		if(a->h.level==MV_VSR_LEVEL) // if versor multiply out then take scalar part :-(
		{	DeFactoriseMe(&a);
			Real r = ScalarPart(a);
			KillMultivector(&a);
			return(r);
		}
#endif	
	
		if(a->h.present & 1)
		{	if(a->h.level>0) return(ScalarPart(a->m[0]));
			Real r = ((multivector0)a)->r[0];
ASSERT2(ValidReal(r));
			if(!ValidReal(r)) r=Real_0;
			return(r);
		}
	}
	return(Real_0);
}


Real PseudoScalarPartCore(multivector a) // Faster than using general grade restriction call
{	
	if(a!=MV_ZERO)
	{
		if(a->h.present & MV_lev_ps)
		{	int i=BitsSetBf(a->h.present)-1;
			if(a->h.level==0) return(((multivector0)a)->r[i]);
			return(PseudoScalarPartCore(a->m[i-1]));
		}
	}
	return(Real_0);
}



Real PseudoScalarPart(multivector a) // Faster than using general grade restriction call
{	if(a!=MV_ZERO)
	{
	
#if MV_VSR_LEVEL
		if((a->h.level==MV_OTR_LEVEL)||(a->h.level==MV_VSR_LEVEL)) // if versor multiply out then take scalar part :-(
		{	DeFactoriseMe(&a);
			Real r = PseudoScalarPart(a);
			KillMultivector(&a);
			return(r);
		}
#endif	
		if(a->h.level==MV_toplev) 
		{	if(a->h.present & MV_top_ps)
			{	int i=BitsSetBf(a->h.present)-1;
				if(a->h.level==0) return(((multivector0)a)->r[i]);
				return(PseudoScalarPartCore(a->m[i-1]));
			}
		}
	}
	return(Real_0);
}

#if 0
#endif


Real MultivectorPseudoScalarProduct(multivector a,multivector b,mvproduct pro)
{
REPORT2(MV_NEWS,4,("\n PsuedoScalar prod %M x %M",a,b))
	if((a==MV_ZERO)||(b==MV_ZERO)) return(Real_0);

STRACE3
	multivector mv = MultivectorProduct(NULL,a,b,pro);

REPORT2(MV_NEWS,4,("\nr Prod[%X] of %M and %M is %M",pro,a,b,mv))
	Real co=PseudoScalarPart(mv);
	KillMultivector(&mv);
	return(co);
}

Real MultivectorScalarProduct(multivector a,multivector b,mvproduct pro)
{
REPORT2(MV_NEWS,4,("\n Scalar prod %M x %M",a,b))
	if((a==MV_ZERO)||(b==MV_ZERO)) return(Real_0);

STRACE3
	multivector mv = MultivectorProduct(NULL,a,b,pro, ~1);

REPORT2(MV_NEWS,4,("\nScalar Prod[%X] of %M and %M is %M",pro,a,b,mv))
	Real co=ScalarPart(mv);
	KillMultivector(&mv);
	return(co);
}

Real * MultivectorCoefficient(multivector a, fullbladeindex bi)
{	fullbladeindex loc = bi >> (a->h.level * MV_DIM0);
REPORT2(MV_NEWS,6,("\n local blade %X at level %X of %M",loc,a->h.level, a))
	if(loc >= (1<<MV_DIM0)) return(NULL);
	bladesflag bit = (bladesflag)(1<<loc);
REPORT2(MV_NEWS,7,("\n bit = %X vs prsent = %X",bit, a->h.present))
	if(! (bit  & a->h.present)) return(NULL);
	fint cnt = BitsSetBf((bladesflag)(a->h.present & (bit-1)));
REPORT2(MV_NEWS,8,("\n use slot %X ",cnt))
	if(a->h.level==0)  return( & (((multivector0)a)->r[cnt]));
	if (a->m[cnt]==MV_ZERO) return(NULL);
	bi ^= (loc << (a->h.level * MV_DIM0)); 
	return( MultivectorCoefficient(a->m[cnt], bi));
}


Real MultivectorSquaredModulus(multivector a)
{	return(MultivectorScalarProduct(a,a,MV_ONTO_CONTRACTIVE_PRODUCT | (mv_conjugation_modulatory << MV_CONJUGATION_SHIFT)	) );
}


Real MultivectorInvertedMagnitude(multivector a,mvproduct pro)
{	Real r = MultivectorScalarProduct(a,a,MV_ONTO_CONTRACTIVE_PRODUCT | pro) ; // (mv_conjugation_modulatory << MV_CONJUGATION_SHIFT)	) ;
	if(EQUIVALENT_REALS(r,Real_0)) { r=Real_1;}
	else
	{	if(r<0) { r=-r; a->h.negsig=1;} else a->h.negsig=0;
		r=Real_SQRT(r);
		r=Real_RECIP(r);
	}
	return(r);
}


multivector MultivectorNormalise(Real * mag,multivector tgt,multivector src )
{	Real r=MultivectorInvertedMagnitude(src,MV_LEFT_CLIFFORD);
	if(mag!=NULL) *mag=r; 
	tgt=CopyMultivectorScaled(tgt,src,r);
	return(tgt);		
}


#if 0
#endif


// -----------------------------------------
//      Multivector Inverse
// -----------------------------------------
multivector MultivectorInverseCnt(multivector a,mvproduct product,int attempt,int cnt)
{
	if(cnt==0) return(NULL);
	if(a==MV_ZERO) return(NULL);
REPORT2(MV_NEWS,3,("\n Inverting %M cnt=%X product=%X",a,cnt,product));

ASSERT2(NumBlades(a)==cnt);

#if MV_VSR_LEVEL
	if(a->h.level>=MV_VSR_LEVEL)
	{	int used = a->h.present;
		multivector r=NewMultivector(used,NULL,a->h.level);
		for(int i=0;i<used;i++)
		{	r->m[i] = MultivectorInverse(a->m[used-1-i],product,attempt);
			if(r->m[i] ==NULL) { KillMultivector(&r);	return(NULL);	}
		} 
	}
#endif

	if(cnt==1)
	{	if(BitsSetBf(a->h.present)!=1)
		{	ERRORCN(("Inverting %M have cnt=%X pres=%X lv=%X nb=%X. Correcting...",a,cnt,a->h.present, a->h.level,NumBlades(a)))
		}
		bladeindex bi = FirstBlade(a->h.present);
		mvproduct pro=product;
		bladeindex bj=BladeBladeProduct(bi,bi,&pro,a->h.level);
		if(bj==MV_INDEX_NIL) return(NULL);

ASSERT(bj==0);

		if(a->h.level>0)
		{
REPORT2(MV_NEWS,5,("   single component so invert coeff %M ",a->m[0]));
			multivector mv=MultivectorInverse(a->m[0],product,0);
			if(mv==NULL) return(NULL);
			mvconjugation cnj = ((pro^product)&MV_LEFT_NEGATION) ;
			if(OddBlade(bi)) cnj |= MV_CONJUGATION_INVOLUTION;
			
			if(cnj) MultivectorConjugate(mv,cnj);
			multivector r=NewMultivector(1,NULL,a->h.level);
			r->h.present = a->h.present;		r->m[0] = mv;
STRACE3


#if MV_DEBUG>2
if(!CheckValidMV(r))
{  ERRORCN(("Inverse about to mult by invalid multivector r=%p",r))
	 BACKTRACE("Hello world",123)
}
#endif

multivector test=MultivectorProduct(NULL,r,a,product);
REPORT2(MV_NEWS,4,("   Result of inverse is %M gives %M",r,test));
KillMultivector(&test);
			return(r);
		}
		else
		{
			Real co=((multivector0)a)->r[0];
REPORT2(MV_NEWS,4,("\n Inverting single basis blade %r * %O ",co,bi,0));
			if(EQUIVALENT_REALS(co,Real_0)) return(NULL);
			if(pro & MV_LEFT_NEGATION) co=-co;
			return(MultivectorBladeReal(Real_RECIP(co),bi));
		}
	}

	while(attempt<5)	// 1.2.3 modification  - if (a'a)^-1 exists then a^-1 = (a'a)^-1 a'  
	{	mvproduct trypro[5] = {
			MV_LEFT_CLIFFORD,
		 	MV_LEFT_REVERSE,
		  MV_LEFT_HERMITIAN,
		  MV_CONJUGATION_CLIFFORD | MV_CONJUGATION_EXT_NEGATION	,
		  MV_CONJUGATION_HERMITIAN | MV_CONJUGATION_EXT_NEGATION
 
		   };

STRACE3
		multivector aca = MultivectorProduct(NULL,a,a,trypro[attempt] ^ product);
		int cnt2=	NumBladesWithCorrection(aca);
REPORT2(MV_NEWS,3,("\n Inverting %M (pro=%X cnt=%X) attempt %X gives (%M) cnt=%X",a,product,cnt,attempt,aca,cnt2));
		if(cnt2<cnt)
		{	
ASSERT2(cnt2==NumBlades(aca));
		multivector aca_inv = MultivectorInverseCnt(aca,product,0,cnt2);
			if(aca_inv!=NULL)
			{	KillMultivector(&aca);
STRACE3
				multivector r = MultivectorProduct(NULL,aca_inv,a,(trypro[attempt]<<MV_CONJUGATION_SHIFT) ^ product);
				KillMultivector(&aca_inv);

STRACE3
multivector test=MultivectorProduct(NULL,r,a,product);
REPORT2(MV_NEWS,4,("   Result of attempt %X inverse is %M gives %M",attempt,r,test));
KillMultivector(&test);
				return(r);
			}
		}
		KillMultivector(&aca);
		attempt++;
	}	// endwhile

	if(attempt>=3) 
	{
REPORT2(MV_NEWS,2,("\n  Stymied in inverting %M ",a));
	}	
	return(NULL);
}

// -----------------------------------------
multivector MultivectorInverse(multivector a,mvproduct product,int attempt)
{
	if(a==MV_ZERO) return(NULL);
	int cnt =	NumBladesWithCorrection(a);
ASSERT2(cnt==NumBlades(a));
	return(MultivectorInverseCnt(a,product,attempt,cnt));
}


// -----------------------------------------
//      Fast tranformation of 1-vecto
// -----------------------------------------


multivector MultivectorTransform(multivector tgt,multivector A,multivector x,mvproduct pro) // returns non-normalised join of arbitary sign
{	// assumes A a pure k-blade 
STRACE3
REPORT2(MV_NEWS,3,("\n  Transforming %M by %M (pro=%X)", x, A, pro));
 	tgt = MultivectorProduct(tgt, x, A, pro);
 	tgt = MultivectorProduct(tgt, A, tgt);
	return(tgt);
}

// -----------------------------------------
//      Multivector Projection
// -----------------------------------------

multivector MultivectorProjectionFast(multivector tgt,multivector a,multivector b,multivector b_inv,mvproduct pro)
{	// (a _cnt b) b_inv
	if((a==MV_ZERO)||(b==MV_ZERO)||(b_inv==NULL)) {	KillMultivector(&tgt);return(NULL);	}
STRACE3
 	multivector ab = MultivectorProduct(NULL,a,b,MV_ONTO_CONTRACTIVE_PRODUCT | pro);
STRACE3
	multivector r = MultivectorProduct(tgt,ab,b_inv ,(0*MV_ONTO_CONTRACTIVE_PRODUCT) | pro);
	KillMultivector(&ab);
	return(r);
}


multivector MultivectorProjection(multivector tgt,multivector a,multivector b,mvproduct pro)
{	// (a _cnt b) _cnt b _inv
	if((a==MV_ZERO)||(b==MV_ZERO)) {	KillMultivector(&tgt);return(MV_ZERO);	}
	multivector b_inv = MultivectorInverse(b,pro,0);

REPORT2(MV_NEWS,3,("\n Projetion %M into %M (pro=%X)",a,b,pro))
{
STRACE3
 multivector bb = MultivectorProduct(NULL,b,b_inv,pro); 
REPORT2(MV_NEWS,3,("\n (inv=%M -> %M) ",b_inv,bb));
 KillMultivector(&bb);
}

	if(b_inv!=NULL)	{	tgt = MultivectorProjectionFast(tgt,a,b, b_inv,pro); 
STRACE3
{ multivector test = MultivectorProduct(NULL,tgt,b,MV_OUTTER_PRODUCT | pro);
REPORT2(MV_NEWS,3,("\n ....projection (%X) test (%M)",pro, test))
} 	
	
		KillMultivector(&b_inv);
	 	}
	else
	{	KillMultivector(&tgt);
REPORT2(MV_NEWS,2,("\n Projection into %M fails ",b))
	}
	return(tgt);
}

multivector MultivectorProjectionNew(multivector tgt,multivector a,multivector b,mvproduct pro)
{	// (a _cnt b) _cnt b _inv
	if((a==MV_ZERO)||(b==MV_ZERO)) {	KillMultivector(&tgt);return(MV_ZERO);	}
	multivector a_inv = MultivectorInverse(a,pro,0);

	if(a_inv!=NULL)
	{
STRACE3
 		tgt = MultivectorProduct(tgt,a_inv,b,MV_ONTO_CONTRACTIVE_PRODUCT | pro);
 		tgt = MultivectorProduct(tgt,tgt,b, pro);
	 	KillMultivector(&a_inv); 
	}
	else
	{	KillMultivector(&tgt);
REPORT2(MV_NEWS,2,("\n NewProjection into %M fails ",b))
	}
	return(tgt);
}





// -----------------------------------------
//      Multivector Meet and Join
// -----------------------------------------



multivector MeetAndJoinGrades(mvgrade * pgr,multivector adb,multivector a,multivector b)
{	// Work with Euclidean innerproduct since join signatures-invarient.

	pgr[3]=MaxGrade(a);	pgr[4]=MaxGrade(b);
	// Predict grade of join from grade of delta product (Bourn's Rule)
	adb = MultivectorProduct(adb, a ,b, MV_EUCLIDEAN_PRODUCT);
	pgr[2] = MaxGrade(adb); // grade of disjoint
	pgr[0]=(mvgrade)((pgr[3]+pgr[4] - pgr[2])/2); // meet grade
	pgr[1]=(mvgrade)((pgr[3]+pgr[4] + pgr[2])/2); // join grade
	return(adb);
}



mvgrade MultivectorJoin(multivector * join,multivector a,multivector b,boolean usenew) // returns non-normalised join of arbitary sign
{	// Work with Euclidean innerproduct since join signatures-invarient.
#define NON_SMALL (Real_1/32)
	mvgrade s_gr;
	mvgrade mjd[5];
	
//	bool swapped;
//	a_gr=MaxGrade(a);	b_gr=MaxGrade(b);
//	swapped=(a_gr > b_gr);
	//if(swapped)	{	SWAPPTRS(a,b); SWAP(a_gr,b_gr);	} 

	// Predict grade of join from grade of delta product (Bourn's Rule)
STRACE3
REPORT2(MV_NEWS,2,("\n --------- \n Join&Disjoint of %M and %M  (usenew=%X)",a,b,usenew));
	multivector adb = MeetAndJoinGrades(mjd, NULL, a, b);
REPORT2(MV_NEWS,2,("\n delta_gr=%X join_gr=%X meet_gr=%X adb=%M",mjd[2],mjd[1],mjd[0],adb));
	if(mjd[0]==0) // scalar meet
	{	*join = MultivectorRestrictGrade(*join, adb, mjd[2]); // effects restriction to outter product 
	//	if(disj!=NULL) *disj=CopyMultivector(*disj,*join);
		KillMultivector(&adb);	
		return(mjd[1]);
	}

	adb = MultivectorRestrictGrade(adb,adb, mjd[2]);
REPORT2(MV_NEWS,2,("\n Deltaproduct is %M",adb));
 	KillMultivector(join);

 	multivector j = MultivectorPseudoScalar(Real_1,MV_DIM0); // j=i

	multivector auc,c,d,ca,m,s;	auc=c=d=ca=m=s= NULL;

	int Ej = g_N - mjd[1];
	int Em = mjd[0];

REPORT2(MV_NEWS,2,("\n Ej=%X",Ej));
	if(Ej>0)
	{	m = MultivectorScalarReal(Real_1,MV_DIM0); // m = 1
		s = MultivectorDual(adb,g_N, MV_EUCLIDEAN_PRODUCT);

REPORT2(MV_NEWS,2,("\n Dual of Deltaproduct is %M",s));

//	s=s.nrm(MV_LEFT_CLIFFORD); // Precise renormalise prior to looping. Slow but safe.


#if MV_DEBUG>2
{ int dd=DiagnoseMV(s); if(dd) ERRORCN(("s fails diag %X ",dd)) }
#endif
		s_gr = (mvgrade)(g_N-mjd[2]);
		
		for(int i=g_N-1;i>=0;i--)	// start with ghc extenders
		{
#if MV_DEBUG>2
{ int dd=DiagnoseMV(s); if(dd) ERRORCN(("s fails diag %X ",dd)) }
#endif
			if(s_gr==1) { c=s; s=NULL;
ASSERT(!IsSmall(c,NON_SMALL));
			}
			else
			{	

{ multivector ss= MultivectorProduct(NULL,s,s,MV_EUCLIDEAN_PRODUCT);
REPORT2(MV_NEWS,2,("\n project basis[%X] %M into =%M of sqr %M ",i,MV_basis[i], s, ss));
KillMultivector(&ss);
}

if(usenew) 	c = MultivectorProjectionNew(c,MV_basis[i],s, MV_EUCLIDEAN_PRODUCT);
else	c = MultivectorProjection(c,MV_basis[i],s, MV_EUCLIDEAN_PRODUCT);
REPORT2(MV_NEWS,2,("\n result is %M small?=%X ", c, IsSmall(c,Real_1/32)));
				if(IsSmall(c,NON_SMALL)) continue;
			}
ASSERT(!IsSmall(c, NON_SMALL));

REPORT2(MV_NEWS,2,("\n Project %M into =%M",c,a));
if(usenew) 	ca = MultivectorProjectionNew(ca,c,a,MV_EUCLIDEAN_PRODUCT);
else	ca = MultivectorProjection(ca,c,a,MV_EUCLIDEAN_PRODUCT);
REPORT2(MV_NEWS,2,("\n Result is =%M",ca));

#if MV_NEWS>2
{ multivector test = MultivectorProduct(NULL,ca,a,MV_OUTTER_PRODUCT);
REPORT2(MV_NEWS,2,("\n test is =[[%M]]",test));
	KillMultivector(&test);
}
#endif
			d = MultivectorAddScaled1(d,c, ca, -Real_1);

			if(!IsSmall(d,NON_SMALL))
			{	
REPORT2(MV_NEWS,2,("\n Factor %M out of %M",d,j));
			
				j=MultivectorProduct(j,d,j,MV_ONTO_CONTRACTIVE_PRODUCT | MV_EUCLIDEAN_PRODUCT);
				if(--Ej==0) break;
			}
			if(!IsSmall(ca,NON_SMALL))
			{	
REPORT2(MV_NEWS,2,("\n Add %M into meet %M",ca,m));
				m=MultivectorProduct(m,m,ca,MV_OUTTER_PRODUCT | MV_EUCLIDEAN_PRODUCT);
		 		if(--Em<=0)	{	j=MultivectorProduct(j,adb,m,MV_OUTTER_PRODUCT | MV_EUCLIDEAN_PRODUCT);
		 		
REPORT2(MV_NEWS,2,("\n consturct j=%M from m=%M",j,m));
		 		
		 			break;}
			}	
ASSERT(s_gr>1);

//REP2(("\n Contract %M into s=%M gr=%X",c,s,s_gr));
			s = MultivectorProduct(s,c,s,MV_ONTO_CONTRACTIVE_PRODUCT | MV_EUCLIDEAN_PRODUCT); s_gr--;// remove c from s
	
REPORT2(MV_NEWS,2,("\n Renormalising s=%M",s));
				s=MultivectorNormalise(NULL,s,s );	// Precise renormalise each cycle. Slow but safe.
REPORT2(MV_NEWS,2,("\n ...result is =%M",s));
	
	
			KillMultivector(&c);
		} // endfor
	}
REPORT2(MV_NEWS,3,("\n consturct j=%M from m=%M",j,m));
	*join=j;
	//if(disj!=NULL) { KillMultivector(disj); *disj=adb;} else

REPORT2(MV_NEWS,2,("\n ..Final Join is %M",j));

	KillMultivector(&adb);	

	KillMultivector(&s);	KillMultivector(&m);	KillMultivector(&ca);	KillMultivector(&d);	KillMultivector(&c);
	return(mjd[1]);
}


#if 0
#endif



inline boolean StrictSameSign(Real a,Real b)
{	return(((a>Real_TINY)&&(b>Real_TINY)) || ((a<-Real_TINY)&&(b<-Real_TINY))); 
}

inline boolean StrictOppSign(Real a,Real b)
{	return(((a>Real_TINY)&&(b<-Real_TINY)) || ((a<-Real_TINY)&&(b>Real_TINY))); 
}


multivector RandomMultivector0(mvlevel lev,bladesflag bf)
{	// Creates a full multivector with random coordinates in rang [-1,1] 
	mvcapacity used = BitsSetBf(bf);
	multivector mv=NewMultivector(used,NULL,lev);
	mv->h.present=bf;
	int i=0;
 	if(lev>0) for(;i<used;i++) mv->m[i]=RandomMultivector0((mvlevel)(lev-1),(bladesflag)((1<<(1<<MV_DIM0))-1));
 	else for(;i<used;i++) ((multivector0)mv)->r[i]=Real_RANDOM(2*Real_1)-Real_1;
	return(mv);
}

multivector RandomMultivector(void)
 { return(RandomMultivector0(
 (mvlevel)MV_toplev, 
 (bladesflag)(1<<(1<<MV_topdim))-1))
 ;}

multivector FullOneVector(int n)
{	int i;
ASSERT(n<=MV_DIM1);
	int k= MIN(n,MV_DIM0);
REPORT2(MV_NEWS,4,("\n FullOneVec n=%X k=%X ",n,k))
	multivector0 mv0=NewMultivector00((mvcapacity)k,NULL);
	mv0->h.present=MV_FLAG_NONE;	

	for(i=0;i<k;i++) {	mv0->h.present |= (1<<(1<<i)); mv0->r[i]=Real_1; }

	n-=k;
ASSERT2(mv0!=NULL);
	
	multivector res=(multivector)mv0;

REPORT2(MV_NEWS,5,("\n FillOneVec lv 0 is %M ",res))

	mvlevel lv=1;
	while(n>0)
	{	multivector mv=NewMultivector((mvcapacity)(n+1),NULL,lv);
		mv->m[0]=res;
		mv->h.present=1;
		k= MIN(n,MV_DIM0-1);
		for(i=0;i<k;i++)
		{ mv->m[i+1]=MultivectorScalarReal(Real_1,1);
			mv->h.present |= (1<<(1<<i));
		}
		res=mv;
REPORT2(MV_NEWS,4,("\n FillOneVec lv %X is %M [%X]",lv,res,res->h.level))
		lv++;
		n-=k;
	}
REPORT2(MV_NEWS,4,("\n Final FillVec is %M lv=%X",res,res->h.level ))
	return(res);
}

multivector RandomOneVector(int n)
{	multivector mv=NewMultivector((mvcapacity)MIN(n,MV_DIM0+1),NULL,0);
	mv->h.present=(1<<1);
	((multivector0)mv)->r[0]=Real_RANDOM(2*Real_1)-Real_1 ;
	for(int i=1;i<n;i++)
	{ multivector bld=MultivectorBladeReal(Real_RANDOM(2*Real_1)-Real_1, 1<<i);
//reattribute(bld,"blade",12340000);
		mv = MultivectorAddMerge(mv,bld);
	}
REPORT2(MV_NEWS,4,("\n Unscaled random %X-D vector is %M (scl slot=%p)",n, mv,mv->m[0] ));
return(mv);
}

multivector RandomOneVectorScaled(int n,Real mag,mvconjugation conj)
{	multivector mv=RandomOneVector(n);
	Real mod=	MultivectorScalarProduct(mv,mv,MV_ONTO_CONTRACTIVE_PRODUCT | (conj << MV_CONJUGATION_SHIFT)	);
REPORT2(MV_NEWS,5,("\n mod=%r while mag = %r",mod,mag ));
	if(mod<Real_0) mod=-mod;
ASSERT(mod>Real_TINY);
	Real scl = Real_SQRT(rDIVrr(mag,mod));	
	mv = CopyMultivectorScaled(mv, mv,scl );
//REPORT2(MV_NEWS,1,("\n Final %M has mag=%r mod=%r vs %r",mv,MultivectorScalarProduct(mv,mv),MultivectorSquaredModulus(mv),mag));
	return(mv);
}

 
void InitCms(void);

// --------------------------------------

void InitMultivectors(mvgrade N,fullbladeindex negs,fullbladeindex nulls,int ghc)
{	g_ghc = ghc;
REPORT2(MV_NEWS,1,("\n InitMutivectors N=%i L=%i ghc=%i",N,MV_DIM0,ghc))
	
	ASSERT(N>=1);
	ASSERT(N<=MV_DIM1);
	int i;
	
	g_N0=N;
	for(i=0;i<ghc;i++)
	{	mv_bf_em[i] = (1<<N);		mv_bf_ep[i] = mv_bf_em[i] <<1; 
		negs |= mv_bf_em[i];	// comment out to force em +ve sig
		N+=2;
	}
	for(;i<MV_MAX_GHC;i++) {	mv_bf_ep[i]=mv_bf_em[i]=0;	}

	g_N = N;
	
	g_negs=negs;
	g_nulls=nulls;


	MV_toplev  = (mvlevel)((1+((N-1)/MV_DIM0))-1) ;
	MV_topdim  = (mvgrade)(N-(MV_toplev*MV_DIM0)) ;
	MV_top_ps = 1<<((1<<MV_topdim)-1); // bladesflag bit for pseudoscalar at top level 




REPORT2(MV_NEWS,1,("\n use N=%X toplev=%X has %X axies",N,MV_toplev,MV_topdim))


	fullbladeindex neg2=negs;
	fullbladeindex nul2=nulls;

	mv_conjugation_modulatory = MV_CONJUGATION_REVERSE;

	int num_negs = BitsSetFbi(negs);
	if(num_negs)
	{	mv_conjugation_modulatory |= MV_CONJUGATION_MITIAN ;
		if(num_negs & 1) mv_conjugation_modulatory |= MV_CONJUGATION_CLIFFORD ;
	}

	g_i_neg = ((N*(N-1)/2) ^ num_negs)&1;

	{	// prepare negsigs and nullsigs containing squares of basis blades at each level
		for(mvlevel lv=0;lv<=MV_toplev;lv++)
		{	mv_negsigs[lv] = (bladesflag)(neg2 & MV_INDEX_MAX);		mv_nullsigs[lv] = (bladesflag)(nul2 & MV_INDEX_MAX);
			neg2>>=MV_DIM0;		nul2>>=MV_DIM0;
		}	// endfor lv
	}

	MV_pool=NULL;
	MV_pool=NewMultivector(MV_NUM_LEVEL,NULL,1);

 	{	for(mvlevel lv=0;lv<=MV_toplev;lv++)
 		{	for(int i=0;i<MV_NUM_REGISTERS;i++) mv_reg[lv][i]= NewMultivector(1<<MV_DIM0,NULL,lv);
 		}
	}

	// Prepare conjugation table - slow and inefficient but only done on initialisation or when signatures change
 	for(mvlevel lev=0;lev<=MV_toplev;lev++)
	{	int numneg =BitsSetFbi((fullbladeindex)(negs & (MV_INDEX_MAX<<(lev*MV_DIM0))) );
		mv_numpos[lev] = MV_DIM0-numneg;
		if(lev==MV_toplev) mv_numpos[lev] -= (MV_DIM0*(MV_toplev+1)	-N);
REPORT2(MV_NEWS,1,("\n %X postive, %X negative at level %X", mv_numpos[lev],numneg,lev)); 


		//	Fill the six primary conjugations
		mv_conj[MV_CONJUGATION_INVOLUTION][lev] = MV_FLAG_NONE;
	 	mv_conj[MV_CONJUGATION_REVERSE][lev]    = MV_FLAG_NONE;
		mv_conj[MV_CONJUGATION_MITIAN][lev]  		= MV_FLAG_NONE;

		mv_conj[MV_CONJUGATION_NEGATION][lev]   = MV_FLAG_ALL;

		mv_conj[MV_CONJUGATION_EXT_NEGATION][lev]   = MV_FLAG_NONE;
		mv_conj[MV_CONJUGATION_EXT_MODULATION][lev]   = MV_FLAG_NONE;


	
		for(i=0;i<=MV_INDEX_MAX;i++)
		{	fint grd  = BitsSetBi((bladeindex)i);
			fint grd2 = BitsSetBi((bladeindex)(i & (~(negs>>(lev*MV_DIM0)))));
			fint grd3 = BitsSetBi((bladeindex)(i & ( negs>>(lev*MV_DIM0))));
			bladesflag bit = (bladesflag)(1<<i);

			if(grd&1)	mv_conj[MV_CONJUGATION_INVOLUTION][lev] |= bit;
			if(grd&2) mv_conj[MV_CONJUGATION_REVERSE   ][lev] |= bit;

			// mitian is move across negatives spanner
			
			if(numneg>0)
			{ if(numneg & 1) // odd number of -ve sig axies
	 			{	if(grd2 &1) mv_conj[MV_CONJUGATION_MITIAN ][lev] ^= bit; // and odd number of +ve	in this blade
				}
				else
				{	// even number of -ve sig axies
	 				if(grd3 &1) mv_conj[MV_CONJUGATION_MITIAN ][lev] ^= bit; // and odd number of -ve	in this blade
				}
			}

			if(ghc)
			{	if (i & (mv_bf_em[0]>>(lev*MV_DIM0)))
				{ mv_conj[MV_CONJUGATION_EXT_NEGATION  ][lev] ^= bit;
					mv_conj[MV_CONJUGATION_EXT_MODULATION][lev] ^= bit; 
				}
				if (i & (mv_bf_ep[0]>>(lev*MV_DIM0))) mv_conj[MV_CONJUGATION_EXT_NEGATION  ][lev] ^= bit;
			}
		}	// endfor i

		// Fill composite conjugations
		for(mvconjugation con=0;con<=MV_CONJUGATION_ALL;con++)
		{	if(BitsSetCountLUT[con]!=1)
			{	mv_conj[con][lev] = MV_FLAG_NONE;
				mvconjugation bit=1;
				while(bit && (bit<=con))	{	if(con & bit) mv_conj[con][lev] ^= mv_conj[bit][lev];	bit<<=1;}	// endwhile
			}	// endif
// 			else REP2(("\n conjugation[%X][%X]=%X",con,lev,mv_conj[con][lev]));
		}	// endfor con
	}	// endfor lev

REPORT2(MV_NEWS,4,("\n Clifford  is %X ",mv_conj[MV_CONJUGATION_CLIFFORD][0])); 
REPORT2(MV_NEWS,4,("\n Hermitian is %X ",mv_conj[MV_CONJUGATION_HERMITIAN][0])); 

	for(i=0;i<MV_DIM1;i++)	MV_basis[i] = BuildMultivectorReal(1,Real_1, 1<<i );
REPORT2(MV_NEWS,4,("\n Basis built ")); 

#if !MV_RELEASE
	InitCms();
#endif

REPORT2(MV_NEWS,4,("\n Cms Basis built ")); 
	for(i=0;i<ghc;i++)
	{	MV_em[i] = BuildMultivectorReal(1,Real_1, mv_bf_em[i]);
		MV_ep[i] = BuildMultivectorReal(1,Real_1, mv_bf_ep[i]);
		MV_e0[i] = BuildMultivectorReal(2,-Real_HALF, mv_bf_ep[i], Real_HALF, mv_bf_em[i] );  // e0   = (-ep+em)/2
		MV_einf[i] = BuildMultivectorReal(2,Real_1, mv_bf_ep[i], Real_1, mv_bf_em[i] ); 			 // einf = ep+em
		MV_e0_inv[i] = CopyMultivectorScaled(NULL,MV_einf[i], -Real_1); 
		MV_einfi[i] = CopyMultivectorScaled(NULL,MV_e0[i], -Real_1); 
REPORT2(MV_NEWS,1,("\n GHC level %X has e_m = %M %X e_p=%M %X e_0=%M e_inf=%M ",i,MV_em[i],MV_em[i]->h.present, MV_ep[i], MV_ep[i]->h.present, MV_e0[i],MV_einf[i])); 
REPORT2(MV_NEWS,1,("\n e_oi = %M e_infi=%M ",MV_e0_inv[i],MV_einfi[i])); 
	}
REPORT2(MV_NEWS,1,("\n e1 = %M e2=%M e5=%M e6=%M ",MV_basis[0],MV_basis[1],MV_basis[4],MV_basis[5])); 
	MV_scalar = (multivector0)BuildMultivectorReal(1,Real_1, 0x0 );

 	InitMvs(ghc);
}

void ExitMultivectors(void)
{
REPORT2(MV_NEWS,1,("\n ExitMultivectors (MV_toplev=%X)",MV_toplev))   
 	KillMvs(g_ghc);

	for(mvlevel lv=0;lv<=MV_NUM_LEVEL;lv++) {	for(int i=0;i<MV_NUM_REGISTERS;i++) KillMultivector(mv_reg[lv]+i);}
	int i;
	
	for(i=0;i<MV_MAX_GHC;i++)
	{	KillMultivector(&MV_e0_inv[i]);KillMultivector(&MV_einfi[i]);
		KillMultivector(&MV_em[i]);		KillMultivector(&MV_ep[i]);		KillMultivector(&MV_e0[i]);	KillMultivector(&MV_einf[i]);
	}
	KillMultivector0(&MV_scalar);
	for(i=0;i<MV_DIM1;i++)	KillMultivector(MV_basis+i);
	multivector t=MV_pool; MV_pool=NULL; KillMultivector(&t);
}


#line 3619 "multivec.c00"

// ---------------------------------------------
// quick and dirty text output
// ---------------------------------------------

char hexsymbol0(fint v)	{ if(v<10) return((uint8)(v+'0'));  return((uint8)(v+'A'-10));}

char hexsymbol(fint v)	{ return(hexsymbol0(v&0xF));}


void pf_MV(const multivector mv,fullbladeindex bi,Real r)
{ // Textually outputs multivector in levility-independant format
	if(mv==MV_ZERO) 	{	printf("0");	return;	}
 	if(mv->h.freed) printf("(freed!)");
	bladeindex i=0;
	int cell=0;
	uint32 bit=1;
fint safety=100;
	bi<<=MV_DIM0;
 //printf("<sub>[lv=%X cp=%X pr=%X rf=%X]</sub>",mv->h.level, mv->h.capacity, mv->h.present,mv->h.refcount);
	if(mv->h.level>0)
	{	if(mv->h.level>=MV_VSR_LEVEL)
		{	printf("%+g*(",r); pf_MV(mv->m[cell],0,Real_1);	printf(")");
			cell++;
		 	for(;cell<mv->h.present;cell++)
		 	{ if(mv->h.level==MV_VSR_LEVEL)	printf("*");else printf("^");
		 		printf("(") ;		pf_MV(mv->m[cell],0,Real_1);	printf(")");
			}
		}
		else
		while(bit && (bit<=mv->h.present))
	 	{

ASSERT(--safety>0);
if(safety<0) break;
			if(bit & mv->h.present)
			{	pf_MV(mv->m[cell],bi|i,r);
			//	bpf(pbh,"\n");
				cell++;
			}	// endif bit
		 	i++;	bit<<=1;
		}	// endwhile
	}
	else
	{	while(bit && (bit<=mv->h.present))
	 	{

ASSERT(--safety>0);
if(safety<0) break;
			if(bit & mv->h.present)
			{	Real rr= rrMUL(r,((multivector0)mv)->r[cell]);
				if(!EQUIVALENT_REALS(rr,Real_0))
				{
				 	fullbladeindex b=bi|i;
					if(b)
					{	if EQUIVALENT_REALS(rr,Real_1) printf("+");
						else if EQUIVALENT_REALS(rr,-Real_1) printf("-");
						else	printf("%+g",rr);	// output Real number
				 		int j=1; // 1=>first axis labelled e1
						printf("e[");
						while(b)
						{	if(b&1)	printf("%c",hexsymbol0(j));
							j++;b>>=1;
						}
						printf("]");
					}
					else printf("%+g",rr);
				}
				cell++;
			}
		 	i++; 	bit<<=1;
 		}
	} // endelse
}

