
/*
   MV 1.3.0

	 Encapsulating C++ class for C hierachical multivector type .
	  Copyright (c) Ian C.G.Bell 2004.
	 For a description of methadologies in this code see www.ibell.co.uk/maths/geoalg.htm

*/

 #include "stdafx.h"

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <assert.h>

#include "global.hpp"
#include "multivec.hpp"


void pf_mv(mv a) { a.setOp(0); pf_MV(a.m, 0, a.s);}

#ifndef _TCHAR
#define _TCHAR char
#endif

int _tmain(int argc, _TCHAR * argv[])
{
	if(argc>0)
	{	if(argv[0][0]=='?') printf("\n MV version 1.%X.%X ",MV_RELEASE,MV_RELEASE_LOW);
	}

	printf("\n Hello world!");

	int N=24;
	int ghc=0; // number of genmeral homogenised coords (typically 1 or 0)

	InitMultivectors(N,0,0,ghc);

	// ---------------------------------
	//	Test stuff goes here
	// ---------------------------------

	mv a = 1.1*e1 + 2.2*e2;

	a = a ^ e4;

	mv b = RandomBlade(3,6); // random 3-blade in 6D space

	printf("\n a ="); 	pf_mv(a);
	printf("\n b =");   pf_mv(b);


	printf("\n Thats all folks. \n");

	// ---------------------------------


	ExitMultivectors();

	return 0;
}



// globally visible flags 
boolean g_mvfactored=0;	 // 1=> build factored products 
mvcapacity	g_MVmargin=1;

// globally visible multivectors 
mv mv_0; mv mv_1;
mv e1,e2,e3;
mv e13,e23,e12,e123;
#if MV_DIM1>=4
	mv e4,e124,e1234;
	#if MV_DIM1>=5
		mv e5,e345,e12345;
			#if MV_DIM1>=6
				mv e6,e123456;
					#if MV_DIM1>=7
						mv e7,e1234567;
					#endif // 7
			#endif // 6
	#endif // 5
#endif // 4

mv mv_p_reg[MV_MAX_GHC][MV_NUM_POINT_REG];




mv mv_i;mv mv_i_inv;

#if MV_MAX_GHC
mv mv_Ee123[MV_MAX_GHC];
mv mv_E[MV_MAX_GHC];
mv mv_e0[MV_MAX_GHC],mv_ee[MV_MAX_GHC],mv_e0_inv[MV_MAX_GHC],mv_ee_inv[MV_MAX_GHC];
mv mv_ep[MV_MAX_GHC]; mv mv_em[MV_MAX_GHC];

mv mv_I[MV_MAX_GHC+1];
mv mv_I_inv[MV_MAX_GHC+1];
#endif

mv mv_basis[MV_DIM1];
mv mv_inv_basis[MV_DIM1];


mv mv_unit_cube[MV_DIM1];


mv mv_product(const mv &a,const mv &b,mvproduct pro)
{	pro ^= ( a.conj() | ( b.conj() << MV_CONJUGATION_SHIFT ) );
	if(a.getDual() && (!(g_N & 1))) pro ^= MV_RIGHT_INVOLUTION; // need to commute i across b
	mv r = mv(MultivectorProduct(NULL, a.m, b.m, pro ));	r.s = rrMUL(a.s,b.s);
	if(a.getDual() ^ b.getDual()) r.setDual(true);
	if(a.getDual() && b.getDual() && g_i_neg) r.s = -r.s;			// double dual may entail sign flip
	return(r);
}

Real mv_scalar_product(const mv &a,const mv &b,mvproduct pro)
{
	Real r,s;
	s=rrMUL(a.s, b.s);
	if(EQUIVALENT_REALS(s,Real_0)) return(REAL_0);
	 

	pro ^= ( a.conj() | ( b.conj() << MV_CONJUGATION_SHIFT ) );
	if(a.getDual() && (!(g_N & 1))) pro ^= MV_RIGHT_INVOLUTION; // need to commute i across b
	if(a.getDual() ^ b.getDual())
	{ r = MultivectorPseudoScalarProduct(a.m, b.m, pro );
		if(g_i_neg) r=-r;
	}
	else
	{ r = MultivectorScalarProduct(a.m, b.m, MV_ONTO_CONTRACTIVE_PRODUCT | pro );
		if(a.getDual() && b.getDual() && g_i_neg) r = -r;			// double dual may entail sign flip
	}
	r = rrMUL(r,s);
#if MV_DEBUG>2
	mv test=mv_product(a,b,pro);
	Real r2=test.scalarPart();
	if(!EQUIVALENT_REALS(r,r2)) ERRORCN(("Scalar part suspect %r vs %r ",r,r2)) 
#endif
	return(r);

}


mv mv_left_contraction(const mv &a,const mv &b) { return(mv_product(a,b,MV_ONTO_CONTRACTIVE_PRODUCT));}
mv mv_right_contraction(const mv &a,const mv &b) { return(mv_product(a,b,MV_BY_CONTRACTIVE_PRODUCT));}


mv mv_delta_product(const mv &a,const mv &b,mvproduct pro) { return(mv_product(a,b,pro)[-2]);}
mv mv_delta2_product(const mv &a,const mv &b) { return(mv_product(a,b,MV_GEOMETRIC_PRODUCT)[-1]);}


extern int CheckMVcode(void);

#if MV_DEBUG
int Diagnosemv(mv a)
{	if(!ValidReal(a.s)) return(999);
	return(DiagnoseMV(a.m));
}
#endif



mv mv_commutation(mv &a,mv &b,mvproduct pro)
{	a.setOp(0);	b.setOp(0);
	mv r = mv(MultivectorCommutation(NULL, a.m, b.m, pro ));
	r.s = rrMUL(a.s,b.s);
	return(r);
}
// =======================================
//       mv class methods
// =======================================

// Constructors
mv::mv() 													: op(op_NIL),	s(Real_1)	,	m(NULL)	{} 
mv::mv(const Real c)							: op(op_NIL),	s(c) ,			m(MultivectorBladeReal(Real_1, 0,1)) {} 
mv::mv(const vector3D * v)				:	op(op_NIL),	s(Real_1) , m(Multivector3D1(v->x,v->y,v->z)) {} 
mv::mv(fullbladeindex bi, Real c)	: op(op_NIL),	s(c) 			,	m(MultivectorBladeReal(Real_1, bi,1)) {} 
mv::mv(mvoperation o,Real c,const multivector a): op(o),	s(c) , m(a) {}
mv::mv(Real c,const multivector a): op(op_NIL),	s(c) , m(a) {} 
mv::mv(const multivector a)				: op(op_NIL),	s(Real_1),  m(a) {} 


// Shallow copy primatives
mv::mv(const mv & a)							: op(a.op), 	s(a.s) 	, m(Reference(a.m)) {} // initialise

mv mv::operator=(mv a)	// assignment
{	 op = a.op; 	s = a.s;
	if(m!=a.m) 	{	KillMultivector(&m);	m = Reference(a.m); } 
	return(*this) ;
}


mv::~mv() {	KillMultivector(&m);}

mvconjugation mv::conj(void) const				{ return(op & MV_CONJUGATION_ALL);}
bool 					mv::getDual(void)	const			{ return((op & MV_DUAL)!=0);}
void 					mv::setDual(bool b) 				{ if(b) op |= MV_DUAL; else op &= ~MV_DUAL; }
void 					mv::setConj(mvconjugation c){ op &= ~MV_CONJUGATION_ALL; op|=c; }

void					mv::setOp(mvoperation o) 
{ mvoperation co = o ^ op;
REPORT2(MV_NEWS,4,("\n setOp co=%X of %m ",co,this))	
	if(co)
	{	m=EnsureWritable(m);
		mvconjugation conj = co & MV_CONJUGATION_ALL;
		if(conj) MultivectorConjugate(m, conj);
		if(co & MV_DUAL)
		{
REPORT2(MV_NEWS,4,("\n setOp taking dual of %m ",this))	
			multivector r=MultivectorDual(m, g_N,MV_GEOMETRIC_PRODUCT);
			KillMultivector(&m);
			m=r;
REPORT2(MV_NEWS,4,("\n setOp gets dual of %m ",&m))	
				if((o & MV_DUAL) && g_i_neg) s=-s; // applied inverse dual to force dual bit set
		}
		op=o;
	}
}


mv 						scalar(const Real b)	{ return(mv(0,b));} // an implicit mv-from-Real construtor prangs C++ binary operator overloadings 

mv 			 			blade(const fullbladeindex bi, const real c)	{ return( mv(bi,realToReal(c)));}

mv 			 			versor(Real s,const fullbladeindex bi,int k)
{	mv r;	r.s = s;
	int used = BitsSetFbi(bi);
	if(k<used) k=used;
	r.m = NewMultivector((mvcapacity)k,NULL,MV_VSR_LEVEL);
	r.m->h.present  = used;
	fullbladeindex bit=1;
	int cell =0;
	while((bit) && (bit<=bi))
	{	if(bit & bi) r.m->m[cell++] = MultivectorBladeReal(Real_1,bit,1);
		bit<<=1;
	}
	return(r);
}


mv factorise(mv a)
{

REPORT2(MV_NEWS,0,("\nFactorising %m ",&a))
	mv r=versor(Real_1,0,MV_DIM1);
REPORT2(MV_NEWS,0,("\n Init versor %m ",&r))
	mv a2=a;
	fullbladeindex bi = a.principleBladeIndex();
	fullbladeindex bit=1;

	while((bit) && (bit<=bi))
	{
		if(bit & bi)
		{	mv c = mv(bit,Real_1);
REPORT2(MV_NEWS,0,("\n Use blade factor c=%m for bit %X ",&c,bit))
			c = c >>= a;
REPORT2(MV_NEWS,0,("\n got factor c=%m ",&c))
			r = r * c;
REPORT2(MV_NEWS,0,("\n built to %m ",&r))
			a2 = c | a2;
		}
		bit<<=1;
	}
	return(r);
}




void 					mv::writeEnable(void) {	m=EnsureWritable(m); }

int						mv::numBlades(void) 		const {	return( NumBlades(m) );} // nonsparsity in the universal basis
bool 					mv::isZero(void) 		const {	return( EQUIVALENT_REALS(s,Real_0) || IsZero(m) );}
bool 					mv::isScalar(void)	const {	return( EQUIVALENT_REALS(s,Real_0) || IsScalar(m) );}
bool					mv::pureGrade(int gr)	const		{ if(gr) return(grades()==(1<<gr)); else	return(isScalar());}

Real					mv::scalarPart(void) const {	return(rrMUL(s,ScalarPart(m))); }

mvgradesflag  mv::grades(void) const
{	if EQUIVALENT_REALS(s,Real_0) return(0);
	mvgradesflag gf= GradesPresent(m);
	if(op & MV_DUAL)
	{	// have to reverse N-bit gradesflag
		mvgradesflag gf2=0;
		mvgradesflag top=1<<g_N;
		mvgradesflag bot=1;
		do
		{	if(gf&top) gf2|=bot;
			bot<<=1;	top>>=1;
		} while(top);
		gf=gf2;
	}
	return(gf);
}

fullbladeindex mv::principleBladeIndex(void) const 
{ fullbladeindex b; 
	Real r = PrincipleBlade(&b,m);
	if EQUIVALENT_REALS(r,Real_0) return(MV_INDEX_NIL);
	return(b);
}
	
mv						mv::principleBlade(void) const
{ fullbladeindex b;
	mv r; r.s=PrincipleBlade(&b,m); r.s=rrMUL(s,r.s);
	r.m = MultivectorBladeReal(Real_1, b,1);
	return(r);
}


mv 						mv::operator[](int i) 				{ return( mv(op, s, MultivectorRestrictGrade(NULL, m, i)));}
mv 						mv::operator[](int i) const 	{ return( mv(op, s, MultivectorRestrictGrade(NULL, m, i)));}


mv 						mv::inv(mvproduct pro) //=MV_GEOMETRIC_PRODUCT)
{	mv r;	Real v;
	if(EQUIVALENT_REALS(s,Real_0))	v =  Real_INFINITY;	else v = rDIVrr(Real_1,s);
	setOp(0);

	multivector rm= MultivectorInverse(m,pro,0);

	if(rm==NULL)
	{ r = mv(Reference(m));
REPORT2(MV_NEWS,2,("\n Inverse of %m fails ",this))
	}
	else	r = mv(rm);
	r.s = v;
	return(r);
}

mv 						mv::dual(void)
{ mv r = mv(*this);
	if (r.getDual()) {	r.setDual(0);	if(g_i_neg) r.s=-r.s;	}	 else r.setDual(1);
	r.setOp(0); // force explicit dual
	return(r);
}	

	
mv 						mv::antiDual(void)						{ mv r = dual();	if(g_i_neg) r.s=-r.s;	return(r);}	


mv 						mv::unit(void)
{
	if(EQUIVALENT_REALS(s,Real_0)) return(mv_0);
	Real mag = MultivectorScalarProduct(m,m);
REP2(("\n mag of %m is %r ",this, mag));

	mv r;
	if(!EQUIVALENT_REALS(mag,Real_0)) r= mv(op, rDIVrr(Real_1 , Real_SQRT(fabs(mag))), Reference(m)) ;
	else r = mv(op, s, Reference(m));
	return(r);
}

mv 						mv::nrm(mvproduct pro)
{	if(EQUIVALENT_REALS(s,Real_0)) return(mv_0);
	Real mag = MultivectorScalarProduct(m,m,pro);
//REP2(("\n mag of %m is %r ",this, mag));

	mv r;
	if(!EQUIVALENT_REALS(mag,Real_0)) r= mv(op, rDIVrr(Real_1 , Real_SQRT(fabs(mag))), Reference(m)) ;
	else r = mv(op, s, Reference(m));
	return(r);
}

mv						mv::exp(int sumTerms)
{	
	mv sq =	(*this) * (*this);
#if 1
	if(sq.isScalar())
	{	Real ssq = sq.scalarPart();
		if EQUIVALENT_REALS(ssq,Real_0) return(*this + Real_1);
		if(ssq<Real_0)
		{	Real mag = Real_SQRT(-ssq);
			return(cos(mag) + rDIVrr(sin(mag),mag) * (*this));
		}
		Real mag = Real_SQRT(ssq);
		return(cosh(mag) + rDIVrr(sinh(mag),mag) * (*this));
	}
#endif
	sq /= 2;
	mv r = Real_1 + *this + sq;
	for(int i=3;i<=sumTerms;i++)
	{	sq = sq * (*this);
		sq = sq* (Real_1/i);
		r += sq;
	}
	return(r);
}


mv mv::operator*=(mv & b)
{	
#if MV_VSR_LEVEL
	if(g_mvfactored)
	{	mvconjugation bcnj=b.conj();
		if(getDual() && (!(g_N & 1))) bcnj ^= MV_CONJUGATION_INVOLUTION; // need to commute i across b
		multivector a= GeometricFactoredProduct(m, b.m, conj(), bcnj, g_MVmargin);
		KillMultivector(&m); m=a;
		s = rrMUL(s,b.s) ;
		if(getDual() && b.getDual() && g_i_neg) s = -s;		// double dual may entail sign flip
		setDual(getDual() ^ b.getDual());
		setConj(MV_CONJUGATION_NONE);
	}
	else
#endif
	{	writeEnable();
		mvproduct pro = (conj() | (b.conj() << MV_CONJUGATION_SHIFT));

		if(getDual() && (!(g_N & 1))) pro ^= MV_RIGHT_INVOLUTION;

		m = MultivectorProduct(m, m, b.m, pro );
		s = rrMUL(s,b.s);
		if(getDual() && b.getDual() && g_i_neg) s = -s;
		op ^= (b.op & MV_DUAL);
	}
	return(*this);
}


 
mv operator*(const mv &a, const mv &b)
{
#if MV_VSR_LEVEL
	if(g_mvfactored)
	{	mvconjugation bcnj=b.conj();

REPORT2(MV_NEWS,4,("\n factored %m * %m ",&a,&b))

		if(a.getDual() && (!(g_N & 1))) bcnj ^= MV_CONJUGATION_INVOLUTION; // need to commute i across b

		mv r = mv( rrMUL(a.s,b.s) , GeometricFactoredProduct(a.m,b.m,a.conj(),bcnj,g_MVmargin));
		if(a.getDual() ^ b.getDual()) r.setDual(true);
		if(a.getDual() && b.getDual() && g_i_neg) r.s = -r.s;			// double dual may entail sign flip
		return(r);
	}
#endif
	return(mv_product(a,b,MV_GEOMETRIC_PRODUCT));
}
//mv operator.(const mv &a, const mv &b)	{	return(mv_product(a,b,MV_INNER_PRODUCT	));}

mv operator^(const mv &a, const mv &b)
{	
#if MV_VSR_LEVEL
	if(g_mvfactored)
	{	mvconjugation bcnj=b.conj();
		if(a.getDual() && (!(g_N & 1))) bcnj ^= MV_CONJUGATION_INVOLUTION; // need to commute i across b

		mv r = mv(rrMUL(a.s,b.s) , OutterFactoredProduct(a.m,b.m,a.conj(),bcnj,g_MVmargin));
		if(a.getDual() ^ b.getDual()) r.setDual(true);
		if(a.getDual() && b.getDual() && g_i_neg) r.s = -r.s;			// double dual may entail sign flip
		return(r);
	}
#endif
	return(mv_product(a,b,MV_OUTTER_PRODUCT	));
}

mv operator|(const mv &a, const mv &b)	{	return(mv_product(a,b,MV_ONTO_CONTRACTIVE_PRODUCT	));}

mv operator&(mv &a,mv &b)	{	return(mv_commutation(a,b,MV_COMMUTATOR_PRODUCT				));}


#if 1
mv operator^(mv &a, int i)
{	//if(a.isZero()) return(mv_0);
	mv r;
	switch(i){
 		case -3: r= Real_1/(a*a*a);	return(r);
 		case -2: return(Real_1/(a*a));
 		case -1: return(Real_1/a);
		case 0: return(mv_1);
		case 1: return(a);
		case 2: { return(a*a);}
		case 3: { return(a*a*a);}
	} // endswitch
	return(a);
}
#endif

Real operator%(const mv &a,const mv &b) // % is scalar product
{
REPORT2(MV_NEWS,5,("\n ScalarProduct %m * %m ",&a,&b))
	
	Real r = mv_scalar_product(a,b,MV_GEOMETRIC_PRODUCT);
	return(r);
}

mv operator*(const mv &a, const Real b) {	mv r= mv(a.op , rrMUL(a.s, b) , Reference(a.m)); return(r); }

mv operator*(const Real b,const mv &a) {	mv r= mv(a.op , rrMUL(b, a.s) , Reference(a.m)); return(r); }

mv operator/(const mv &a, const Real b)
{	mv r=mv( Reference(a.m));	r.op = a.op;
	if(EQUIVALENT_REALS(b,Real_0)) r.s = Real_INFINITY; else	r.s =	rDIVrr(a.s, b);
	return(r);
}


mv operator/(const Real b, mv &a)
{	
	mv r;	Real v;
	if(EQUIVALENT_REALS(a.s,Real_0))	v =  Real_INFINITY;	else v = rDIVrr(b,a.s);
	a.setOp(0);

	multivector rm= MultivectorInverse(a.m,MV_GEOMETRIC_PRODUCT,0);

	if(rm==NULL) r = mv(Reference(a.m)); else r = mv(rm);
	r.s = v;
	return(r);
}

mv operator/(mv & a, mv &b)
{	mv r;	Real v;
	if(EQUIVALENT_REALS(b.s,Real_0))	v =  Real_INFINITY;	else v = rDIVrr(a.s,b.s);
	b.setOp(0);

	multivector b_inv= MultivectorInverse(b.m,MV_GEOMETRIC_PRODUCT,0);

REPORT2(MV_NEWS,5,("\n mv %m / %m gives inv %M",&a,&b, b_inv)) 


	if(b_inv==NULL) 
	{	
REPORT2(MV_NEWS,1,("\n Can't invert %m for division",&b)) 
		r = mv(Reference(a.m));
	}
	else
	{
#if MV_VSR_LEVEL
		if(g_mvfactored)  
		{	mvconjugation bcnj=0;
			if(a.getDual() && (!(g_N & 1))) bcnj ^= MV_CONJUGATION_INVOLUTION; // need to commute i across b
			mv r = mv( v , GeometricFactoredProduct(a.m, b_inv, a.conj(), bcnj,g_MVmargin));
			if(a.getDual()) r.setDual(true);
			return(r);
		}
		else
#endif
		{	mvproduct pro = a.conj() | MV_GEOMETRIC_PRODUCT;
			if(a.getDual() && (!(g_N & 1))) pro ^= MV_RIGHT_INVOLUTION;
			r = mv(MultivectorProduct(NULL, a.m, b_inv, pro ));
			KillMultivector(&b_inv);
			r.setDual(a.getDual());
		}
	}
	r.s = v;
	return(r);
}



mv operator+(const mv &a, mv &b)
{
REPORT2(MV_NEWS,5,("\n opr-+ adding %m to %m",&a,&b))
	b.setOp(a.op);
REPORT2(MV_NEWS,6,("\n post conj is %m to %m",&a,&b))
	multivector t = MultivectorAddScaled(NULL, a.m, b.m, a.s, b.s );
REPORT2(MV_NEWS,6,("\n MV res is %M \n",t))
	mv r = mv(t);
REPORT2(MV_NEWS,5,(" res is %m",&r))
	return(r);
}

#if 0
mv operator+(const mv &a, const mv &b)
{
REPORT2(MV_NEWS,5,("\n opr-+ adding %m to %m",&a,&b))
	b.setOp(a.op);
REPORT2(MV_NEWS,6,("\n post conj is %m to %m",&a,&b))
	multivector t = MultivectorAddScaled(NULL, a.m, b.m, a.s, b.s );
REPORT2(MV_NEWS,6,("\n MV res is %M \n",t))
	mv r = mv(t);
REPORT2(MV_NEWS,5,(" res is %m",&r))
	return(r);
}
#endif

mv operator+(const mv &a, const Real b)
{
	if EQUIVALENT_REALS(a.s, Real_0) return( scalar(b)); 
	Real q= rDIVrr( b,a.s);
	if(a.op & MV_CONJUGATION_NEGATION) q=-q;
	mv r= mv( MultivectorAddScaled(NULL, a.m, mv_1.m, REAL_1, q) );
	r.s = a.s;
	r.op = a.op; 
	return(r);
}

mv operator-(const Real b,const mv &a)
{
	if EQUIVALENT_REALS(a.s, Real_0) return( scalar(b)); 
	Real q= rDIVrr( b,a.s);
	if(a.op & MV_CONJUGATION_NEGATION) q=-q;
	mv r= mv( MultivectorAddScaled(NULL, a.m, mv_1.m, -REAL_1, q) );
	r.s = a.s;
	r.op = a.op; 
	return(r);
}

mv operator+(const Real b,const mv &a)
{
	if EQUIVALENT_REALS(a.s, Real_0) return( scalar(b) );
	Real q= rDIVrr( b,a.s);
	if(a.op & MV_CONJUGATION_NEGATION) q=-q;
	mv r= mv( MultivectorAddScaled(NULL, a.m, mv_1.m, Real_1, q) );
	r.s = a.s;
	r.op = a.op; 
	return(r);
}


mv operator-(const mv &a, mv &b) {	b.setOp(a.op);return( mv(MultivectorAddScaled(NULL, a.m, b.m, a.s, -b.s )));}


mv mv::operator+=(Real b)		{	this->writeEnable();	MV_scalar->r[0]= b; m = MultivectorAdd(m,(multivector)MV_scalar);return(*this);}

mv mv::operator-=(Real b)	 	{	this->writeEnable();	MV_scalar->r[0]=-b; m = MultivectorAdd(m,(multivector)MV_scalar);return(*this);}


mv mv::operator+=(mv & b)
{	this->writeEnable();
	setOp(b.op);
	m = MultivectorAddScaled(m, m, b.m, s, b.s );
	s = Real_1;
	return(*this);
}

mv mv::operator-=(mv & b)
{	this->writeEnable();
	setOp(b.op);
	m = MultivectorAddScaled(m, m, b.m, s, -b.s );
	s = Real_1;
	return(*this);
}

mv mv::operator*=(Real b)	 {	s = rrMUL(s, b);	return(*this);}

mv mv::operator/=(Real b)	 {	if(EQUIVALENT_REALS(b,Real_0)) s = Real_INFINITY; else s=rDIVrr(s, b);	return(*this);}


mv mv::operator/=(mv & b)
{	mv r;	Real v;
	if(EQUIVALENT_REALS(b.s,Real_0))	v =  Real_INFINITY;	else v = rDIVrr(s , b.s);
	b.setOp(0);
	multivector b_inv= MultivectorInverse(b.m,MV_GEOMETRIC_PRODUCT ,0);
	if(b_inv!=NULL)
	{	writeEnable();
		m = MultivectorProduct(m,m,b_inv,conj() | MV_GEOMETRIC_PRODUCT);
		setConj(0);
	}
	return(r);
}

mv mv::operator>>=(mv & b) {	b.setOp(0);	setOp(0);	return( mv(0, s, MultivectorProjection(NULL, m, b.m)));}
//mv mv::operator<<=(mv & b) {	mv r = this - (this>>=b); return(r); }

mv mv::operator-()	 {	return( mv(this->op,-this->s, Reference(this->m)));}

//mv mv::operator~()	 {	return( mv(this->op^ MV_CONJUGATION_REVERSE, this->s, Reference(this->m)));}

mv mv::operator!()	 {	mv r = mv(this->op^ MV_CONJUGATION_INVOLUTION, this->s, Reference(this->m));return(r);}


mv mv::hrm(void)	 {	return( mv(this->op^ MV_CONJUGATION_HERMITIAN, this->s, Reference(this->m)));}
mv mv::rev(void)	 {	return( mv(this->op^ MV_CONJUGATION_REVERSE, this->s, Reference(this->m)));}
mv mv::clf(void)	 {	return( mv(this->op^ MV_CONJUGATION_CLIFFORD, this->s, Reference(this->m)));}
mv mv::ivl(void)	 {	return( mv(this->op^ MV_CONJUGATION_INVOLUTION, this->s, Reference(this->m)));}


mv operator||(mv &a, mv &b)
{
	a.setOp(0);	b.setOp(0);
	mv r;
REPORT2(MV_NEWS,5,("\n Joining %m and %m ",&a,&b))
	(void)MultivectorJoin(&r.m, a.m, b.m);
	r.s = MultivectorInvertedMagnitude(r.m,MV_LEFT_CLIFFORD|MV_EUCLIDEAN_PRODUCT);
	if Real_OPPOSITESIGN(a.s,b.s) r.s=-r.s;

REPORT2(MV_NEWS,5,("\n Join(||) res is %m",&r))
	return(r);
}

mv operator&&(mv &a, mv &b)
{
REPORT2(MV_NEWS,2,("\n Meet(&&) of %m and %m ",&a,&b))

	a.setOp(0);	b.setOp(0);
	mv r,d;;
	d.s = rrMUL(a.s, b.s);
	(void)MultivectorJoin(&r.m, a.m, b.m);

REPORT2(MV_NEWS,2,("\n join is %M ",r.m))

	d.m = MultivectorProduct(NULL, a.m, b.m, MV_EUCLIDEAN_PRODUCT);
	int	adb_gr = MaxGrade(d.m);
	d.m = MultivectorRestrictGrade(d.m, d.m, adb_gr);


REPORT2(MV_NEWS,5,("\n Join=%M disjoint=%M ",r.m, d.m))
	r.s = MultivectorInvertedMagnitude(r.m, MV_LEFT_CLIFFORD|MV_EUCLIDEAN_PRODUCT);

REPORT2(MV_NEWS,2,("\n join.... is %7m ", &r))
REPORT2(MV_NEWS,2,("\n disjoint is %7m ", &d))

//	if Real_OPPOSITESIGN(a.s,b.s) r.s=-r.s;

	r = mv_product( d , r.inv(MV_EUCLIDEAN_PRODUCT), MV_ONTO_CONTRACTIVE_PRODUCT | MV_EUCLIDEAN_PRODUCT);

//	r = d.inv() | r;

REPORT2(MV_NEWS,2,("\n Meet(||) res is [%7m]",&r))

	return(r);

}


boolean operator<(mv &a,mv &b) {	return ( ((a ^ b).isZero()) && (!((a | b).isScalar())));	} // is a strictly inside space spanned by b ?

boolean operator<=(mv &a,mv &b) { return((a ^ b).isZero());} // is inside or spans space spanned by b ?


// ---------------------------------------
//	    	mv creation functions
// ---------------------------------------


mv pointswedge(int k,posn3D * p) // k-blade formed by outter product of k embedded point vectors 
{	if(k<=0) return(mv_0);
	mv a = point(p+0);
	for(int i=1;i<k;i++) a = a ^ point(p+i);
	return(a);
}

mv kplane(int k,posn3D * p) { return(mv_ee[0] ^ pointswedge(k,p));}


mv sphere2(posn3D * c,real radius) 
{
#if MV_MAX_GHC>1
	if(g_ghc>1)
	{	multivector m = FullOneVector(g_N0+4);
		MultivectorFromSphereFastDbl(m,c,RSQUARE(radius));
		return(mv(m));
	}
#endif
	return( mv(MultivectorSphere3D(c,RSQUARE(radius),false)));
}

mv sphere(posn3D * c,real radius) { mv r=mv(MultivectorSphere3D(c,RSQUARE(radius),true)); return(r);} 	 // c + e0 + (c^2-rad^2)ee /2 

mv point(posn3D *	p) { return(mv(MultivectorPoint(p))); } // as sphere of zero radius 


void point(mv &x, posn3D *	p) // fast copy into existant point mv
{
	ASSERT2(x.m->h.level==0);

	x.op=0;
	x.s=Real_1;
 	((multivector0)x.m)->r[0]=realToReal(p->x);
 	((multivector0)x.m)->r[1]=realToReal(p->y);
 	((multivector0)x.m)->r[2]=realToReal(p->z);
 			


}


mv vec4D(vector4D *	p) { return(mv(Multivector4D1(p->x,p->y,p->z,p->s))); } 


#if !MV_RELEASE
mv	spinorform(Real * mag,Real * rapid,mv x,mv a)
{	Real xx=x % x;
	if(xx<Real_0)
	{	*mag = Real_SQRT(-xx);
		Real xu4 = -a % x;
ASSERT(xu4 >=Real_0);
		*rapid = acosh( rDIVrr(xu4,*mag)); 
	}
	mv r = x ^ a;
	return(r.nrm());
}
#endif

mv RandomMv(void) { return(mv(RandomMultivector()));}


mv RandomOneVec(mvgrade n, Real sq, mvproduct pro)
{ mv r;
	Real s;
	
 //	r = mv ( RandomOneVectorScaled(n,n*Real_1,MV_CONJUGATION_HERMITIAN));
	r = mv ( RandomOneVector(n));


REPORT2(MV_NEWS,4,("\n RandomOneVector returns %m ",&r))


	s = mv_scalar_product(r, r, pro);

REPORT2(MV_NEWS,5,("\n gives prod=%r vis desired %r ",s,sq))
	 	
#if 0
	if Real_OPPOSITESIGN(s,sq)
	{
REPORT2(MV_NEWS,5,("\n Wanted %r >0 but got %r <0 ",sq,s))
			Real * c_e4 = MultivectorCoefficient(r.m, 0x8);
ASSERT(c_e4!=NULL);
			Real sm = rrMUL((*c_e4),(*c_e4));
			Real sp = s + sm;
ASSERT(sp > Real_0);
			Real l2 = rDIVrr(sq + sm, sp);
			r.s= Real_SQRT(l2);
			(*c_e4) /= r.s;
 	}
	else
#endif
	{
ASSERT(r.s==Real_1);
ASSERT((s>Real_TINY)||(s<-Real_TINY));
		r.s = rDIVrr(sq, s);
ASSERT(r.s>=0);
		r.s = Real_SQRT( r.s );	
	}
REPORT2(MV_NEWS,5,("\n so RandomOneVec returns %m ",&r))
 	return(r);
}

mv RandomBlade(mvgrade grade,mvgrade n)
{	mv r = RandomOneVec(n,Real_2,MV_EUCLIDEAN_PRODUCT);
	for(mvgrade i=1;i<grade;i++) r = r ^ RandomOneVec(n,Real_2,MV_EUCLIDEAN_PRODUCT);
	return(r);
}


mv transform(mv a,mv x) // a x a_cnj
{ return(mv( rrMUL(x.s, rSQUARE(a.s)), MultivectorTransform(NULL,a.m,x.m,MV_RIGHT_CLIFFORD) ));}

mv exponentiatedtransform(mv a,mv x,Real theta) // e^(theta*a) x e^(-theta*a)
{ 
#if 0
	mv aa=a*a;
	if(aa.isScalar())
	{
		Real amag = aa.scalarPart();
		if(amag<-Real_TINY)
		{	amag = Real_SQRT(-amag);
			theta = rrMUL(amag,theta);
			Real ct = cos(theta);
			Real st = sin(theta);

			mv axa = transform(a,x);
			return(ct*ct*x + st*st*axa + 2*st*ct* mv_commutation(a,x,MV_COMMUTATOR_PRODUCT) );
		}

	}
#endif
	mv eat=(theta*a).exp();
	return( transform(eat,x));
}

#if !MV_RELEASE
void ToVector(vector3D *pv,mv a)
{
	VectorFromMultivectorFast((real *)pv,(multivector0)a.m,3);
	real r=RealToreal(a.s);
	if(a.op & MV_CONJUGATION_INVOLUTION ) r=-r;
	if(a.op & MV_CONJUGATION_NEGATION ) r=-r;
	VEC3D_RRMUL(*pv,*pv,r);
}
#endif

// ----------------------------------------------


int FactoriseMV(mv * tgt,multivector a,mvconjugation cnj,mvlevel levpro,int max)
{	if(a==MV_ZERO) return(-1);
	if(max<=0) return(0);
	int num=0;
	mvconjugation neg = cnj & MV_CONJUGATION_NEGATION;
	cnj ^= neg;
	
#if MV_VSR_LEVEL	
	if(a->h.level==levpro)	
	{	if(cnj & MV_CONJUGATION_REVERSE)
		{	for(int i=a->h.present-1;i>=0;i--)
			{ int n = FactoriseMV(tgt+num, a->m[i], cnj, levpro, max-num);
				if(n<0) goto abort;
				num+=n;
			}
		}
		else
		{	for(int i=0;i<a->h.present;i++)
			{ int n=FactoriseMV(tgt+num, a->m[i], cnj, levpro, max-num);
				if(n<0) goto abort;
				num+=n;
			}
		} 		
	}		
	else
#endif
	{	// Add factorisation attempts here 
		tgt[0]=mv(Reference(a)); tgt[0].setConj(cnj); num=1;
	}
	if(neg) tgt[0].s=-tgt[0].s;
	return(num);

#if MV_VSR_LEVEL	
abort: for(int i=0;i<num;i++) (tgt[i]).~mv(); // clear previous sucessful factors	(1.2.3 fix)
	return(-1);	
#endif
}


int FactoriseGeometric(mv * tgt,mv a,mvconjugation cnj,int max)
{	int n = FactoriseMV(tgt, a.m, a.conj()^cnj ,MV_VSR_LEVEL, max);
	if(n>0)	tgt[0].s = rrMUL(tgt[0].s, a.s);
	return(n);
}

int FactoriseOutter(mv * tgt,mv a,mvconjugation cnj,int max)
{	int n = FactoriseMV(tgt, a.m, a.conj()^cnj ,MV_OTR_LEVEL, max);
	if(n>0)	tgt[0].s = rrMUL(tgt[0].s, a.s);
	return(n);
}


// ----------------------------------------------

// =======================================
//       mvfunc class methods
// =======================================
// Constructors
mvfunc::mvfunc() 									: name("uninit")	{	 } 
mvfunc::mvfunc(mv_functionptr f)	:	name("unnamed"), func(f) { } 



// =======================================
//       Gradient methods
// =======================================

mv	BladeGradient(mvfunc &f,mv &x,fullbladeindex bi,Real epsilon) // Evaluate bi-directed derivative of f() at x
{	mv r;
	Real * coeff=MultivectorCoefficient(x.m,bi); // get pointer to bi coefficient for rapid additions
	if(coeff==NULL)
	{	mv xx = x + mv(bi,epsilon);;
		r = f.func(	xx,	f.para); 	r.writeEnable(); // r may reference xx
		coeff=MultivectorCoefficient(xx.m,bi);
		coeff[0]=-coeff[0];
		r -= f.func(xx,f.para);
	}
	else
	{
		Real d = rDIVrr(epsilon , x.s);
		coeff[0]+=d;
		r = f.func(x,f.para);		r.writeEnable();
		coeff[0]-=2*d;
		r -= f.func(x,f.para);;
		coeff[0]+=d;	// restore x to original value
	}
	r.s /= (2*epsilon);
	return(r);
}


mv	OneGradient(mvfunc &f,mv &x,int N,Real epsilon)
{	mv r;
	if(N<0) N=g_N;
	for(int i=0;i<N;i++)
	{	mv a = BladeGradient(f,x,(1<<i),epsilon);
		if(!a.isZero()) r = r + mv_inv_basis[i]*a;
	}
	return(r);
}


void InitMvs(int ghc)
{	int i;

STRACE1
REPORT2(MV_NEWS,1,("\n Init mv ghc=%i g_N=%i",ghc,g_N))   

	mv_0  = mv(0,Real_1); ((multivector0)mv_0.m)->r[0] = Real_0;
	mv_1  = mv(0,Real_1);
	
	int N = g_N;
	for(i=0;i<N;i++)
	{	mv_basis[i]			=	mv(Reference(MV_basis[i]));
		mv_inv_basis[i]	=	mv(Reference(MV_basis[i]));
		Real sig = mv_basis[i] % mv_basis[i];
//Real sig = Real_1;
		if(!EQUIVALENT_REALS(sig,Real_0)) mv_inv_basis[i].s /= sig; 
#if MV_NEWS
		else
		{
REPORT2(MV_NEWS,0,("\n Null signature inverse frame not supported."))
		}
#endif
	}
STRACE1


	// Set globally visible e1 , e23, etc. to taste  
	e12 = mv( (1<<0)|(1<<1), Real_1);
	e13 = mv( (1<<0)|(1<<2), Real_1);
	e23 = mv( (1<<1)|(1<<2), Real_1);

	e1 = mv(Reference(MV_basis[0]));
	e2 = mv(Reference(MV_basis[1]));
	e3 = mv(Reference(MV_basis[2]));

 	e123 = mv( (1<<0)|(1<<1)|(1<<2), Real_1);
	if(N>=4)
	{	e4 = mv(Reference(MV_basis[3]));
 		e124 = mv( (1<<0)|(1<<1)|(1<<3), Real_1);
 		e1234  = mv( (1<<0)|(1<<1)|(1<<2)|(1<<3), Real_1);
	}
STRACE1

	if(N>=5)
	{	e5 = mv(Reference(MV_basis[4]));
		e345 = mv( (1<<2)|(1<<3)|(1<<4), Real_1);
		e12345 = mv( (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4), Real_1);
	}

	if(N>=6)
	{	e6 = mv(Reference(MV_basis[5]));
		e123456 = mv( (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<5), Real_1);
	}
	
	if(N>=7)
	{	e7 = mv(Reference(MV_basis[6]));
		e1234567 = mv( (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6), Real_1);
	}
 
	{	fullbladeindex i_bi = (fullbladeindex)((1<<g_N)-1);
		mv_i 		 = mv(i_bi, Real_1);
		Real i_sig = Real_1;	if (g_i_neg) i_sig=-Real_1;
 		mv_i_inv = mv(i_bi,i_sig);
	}

	for(i=0;i<ghc;i++)
	{	mv_e0[i]  = mv(Reference(MV_e0[i]));mv_ee[i]  = mv(Reference(MV_einf[i]));
 		mv_ep[i]	= mv(Reference(MV_ep[i]));mv_em[i]	= mv(Reference(MV_em[i]));
 		mv_e0_inv[i] = -mv_ee[i];			mv_ee_inv[i] = -mv_e0[i];
		mv_E[i] = mv_ep[i] ^ mv_em[i];
		mv_Ee123[i] = mv_E[i] ^ e123;
REPORT2(MV_NEWS,0,("\n E[%X]=%m I[%X]=%m ", i,&mv_E[i],i,&mv_I[i]))
	}

	{	fullbladeindex i_bi = (fullbladeindex)((1<<g_N0)-1);
		mv_I[0] 		 = mv(i_bi, Real_1);
 		mv_I_inv[0] = (mv_I[0]%mv_I[0])*mv_I[0];
	}

	for(i=1;i<ghc;i++)
	{	mv_I[i]=mv_E[i-1] ^ mv_I[i-1];
 		mv_I_inv[i] = (mv_I[i]%mv_I[i])*mv_I[i];
	}


	{
		for(int g=0;g<g_ghc;g++)
		for(i=0;i<MV_NUM_POINT_REG;i++)
		{
ASSERT(mv_p_reg[g][i].m==NULL);
			mv_p_reg[g][i].m=FullOneVector(g_N-2*g_ghc + 2 + 2*g);
			mv_p_reg[g][i].op =0;
			mv_p_reg[g][i].s =Real_1;
		}
	}

	for(i=0;i<MV_DIM1;i++)	mv_unit_cube[i]=mv_basis[i] + mv_e0[0] + Real_HALF*mv_ee[0];


REPORT2(MV_NEWS,0,("\n p_reg[0][0] %m s=%r %X ", &mv_p_reg[0][0], mv_p_reg[0][0].s,mv_p_reg[0][0].m->h.present))
REPORT2(MV_NEWS,0,("\n p_reg[1][0] %m s=%r ", &mv_p_reg[1][0].m, mv_p_reg[1][0].s))
REPORT2(MV_NEWS,0,("\n ========================== \n     ================ \n        =====   "));
}

void KillMvs(int ghc)
{	int i;
	// Kill globals here rather than relying on C++ shutdown tidy
	mv_0.~mv();	mv_1.~mv();

	e1.~mv();	e2.~mv();	e3.~mv();	e4.~mv();	e5.~mv();e6.~mv();e7.~mv();
	e13.~mv();	e23.~mv();	e124.~mv();	e345.~mv();
	e12.~mv(); 	e123.~mv();	e1234.~mv();e12345.~mv();e123456.~mv();e1234567.~mv();
	mv_i.~mv();	mv_i_inv.~mv();

	for(i=0;i<MV_DIM1;i++) 	{	mv_basis[i].~mv(); 	mv_inv_basis[i].~mv();	}

	for(i=0;i<ghc;i++)
	{	mv_e0[i].~mv();	mv_ee[i].~mv();					mv_ep[i].~mv();	mv_em[i].~mv();
		mv_e0_inv[i].~mv();	mv_ee_inv[i].~mv();	mv_E[i].~mv();
		mv_I[i].~mv();	mv_I_inv[i].~mv();
		mv_Ee123[i].~mv();
	}

	for(int g=0;g<g_ghc;g++)
	for(i=0;i<MV_NUM_POINT_REG;i++)	mv_p_reg[g][i].~mv();

	for(i=0;i<MV_DIM1;i++)	mv_unit_cube[i].~mv();
//	DISCARD(&sourcename);
}


